@@ -1,19 +1,18 @@
-# -*- coding: utf-8 -*-
 """Pywikibot metadata file."""
 #
-# (C) Pywikibot team, 2020
+# (C) Pywikibot team, 2020-2021
 #
 # Distributed under the terms of the MIT license.
 #
 __name__ = 'pywikibot'
-__version__ = '4.2.0'
+__version__ = '6.0.1'
 __description__ = 'Python MediaWiki Bot Framework'
 __maintainer__ = 'The Pywikibot team'
 __maintainer_email__ = 'pywikibot@lists.wikimedia.org'
 __license__ = 'MIT License'
 __url__ = 'https://www.mediawiki.org/wiki/Manual:Pywikibot'
 __download_url__ = 'https://pywikibot.toolforge.org/'
-__copyright__ = '(C) Pywikibot team, 2003-2020'
-__keywords__ = 'API bot client framework mediawiki pwb python pywiki ' \
+__copyright__ = '(C) Pywikibot team, 2003-2021'
+__keywords__ = 'API bot client framework mediawiki pwb pybot python pywiki ' \
                'pywikibase pywikibot pywikipedia pywikipediabot wiki ' \
                'wikibase wikidata wikimedia wikipedia'
