@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 User-interface related functions for building bots.
 
@@ -23,7 +22,7 @@ conjunction. Each bot should subclass at least one of these four classes:
   scripts.ini configuration file. That file consists of sections, led by a
   C{[section]} header and followed by C{option: value} or C{option=value}
   entries. The section is the script name without .py suffix. All options
-  identified must be predefined in availableOptions dictionary.
+  identified must be predefined in available_options dictionary.
 
 * L{Bot}: The previous base class which should be avoided. This class is mainly
   used for bots which work with wikibase or together with an image repository.
@@ -54,7 +53,7 @@ L{CurrentPageBot} and automatically defines the summary when C{put_current} is
 used.
 """
 #
-# (C) Pywikibot team, 2008-2020
+# (C) Pywikibot team, 2008-2021
 #
 # Distributed under the terms of the MIT license.
 #
@@ -63,7 +62,7 @@ __all__ = (
     'VERBOSE', 'critical', 'debug', 'error', 'exception', 'log', 'warning',
     'output', 'stdout', 'LoggingFormatter', 'RotatingFileHandler',
     'init_handlers', 'writelogheader',
-    'input', 'input_choice', 'input_yn', 'inputChoice', 'input_list_choice',
+    'input', 'input_choice', 'input_yn', 'input_list_choice', 'ui',
     'Option', 'StandardOption', 'NestedOption', 'IntegerOption',
     'ContextOption', 'ListOption', 'ShowingListOption', 'MultipleChoiceList',
     'ShowingMultipleChoiceList', 'OutputProxyOption',
@@ -71,8 +70,8 @@ __all__ = (
     'Choice', 'StaticChoice', 'LinkChoice', 'AlwaysChoice',
     'QuitKeyboardInterrupt',
     'InteractiveReplace',
-    'calledModuleName', 'handle_args', 'handleArgs',
-    'showHelp', 'suggest_help',
+    'calledModuleName', 'handle_args',
+    'show_help', 'showHelp', 'suggest_help',
     'writeToCommandLogFile', 'open_webbrowser',
     'OptionHandler',
     'BaseBot', 'Bot', 'ConfigParserBot', 'SingleSiteBot', 'MultipleSitesBot',
@@ -95,13 +94,16 @@ import time
 import warnings
 import webbrowser
 
+from collections.abc import Generator
 from contextlib import closing
 from importlib import import_module
 from pathlib import Path
 from textwrap import fill
+from typing import Any, Optional, Union
 from warnings import warn
 
 import pywikibot
+from pywikibot.backports import Dict, Iterable, List, Sequence
 from pywikibot import config2 as config
 from pywikibot import daemonize
 from pywikibot import i18n
@@ -118,16 +120,13 @@ from pywikibot.logging import CRITICAL, ERROR, INFO, WARNING
 from pywikibot.logging import DEBUG, INPUT, STDOUT, VERBOSE
 from pywikibot.logging import (
     add_init_routine,
-    debug, error, exception, log, output, stdout, warning,
+    critical, debug, error, exception, log, output, stdout, warning,
 )
-from pywikibot.logging import critical
 from pywikibot.tools import (
     deprecated, deprecate_arg, deprecated_args, issue_deprecation_warning,
+    PYTHON_VERSION, suppress_warnings
 )
-from pywikibot.tools._logging import (
-    LoggingFormatter as _LoggingFormatter,
-    RotatingFileHandler,
-)
+from pywikibot.tools._logging import LoggingFormatter, RotatingFileHandler
 from pywikibot.tools.formatter import color_format
 
 
@@ -152,10 +151,14 @@ GLOBAL OPTIONS
 
 -lang:xx          Set the language of the wiki you want to work on, overriding
                   the configuration in user-config.py. xx should be the
-                  language code.
+                  site code.
 
 -family:xyz       Set the family of the wiki you want to work on, e.g.
-                  wikipedia, wiktionary, wikitravel, ...
+                  wikipedia, wiktionary, wikivoyage, ...
+                  This will override the configuration in user-config.py.
+
+-site:xyz:xx      Set the wiki site you want to work on, e.g.
+                  wikipedia:test, wiktionary:de, wikivoyage:en, ...
                   This will override the configuration in user-config.py.
 
 -user:xyz         Log in as user 'xyz' instead of the default username.
@@ -167,7 +170,7 @@ GLOBAL OPTIONS
 -help             Show this help text.
 
 -log              Enable the log file, using the default filename
-                  '%s-bot.log'
+                  '{}-bot.log'
                   Logs will be stored in the logs subdirectory.
 
 -log:xyz          Enable the log file, using 'xyz' as the filename.
@@ -204,6 +207,13 @@ GLOBAL OPTIONS
 
 """
 
+_GLOBAL_HELP_NOTE = """
+GLOBAL OPTIONS
+==============
+For global options use -help:global or run pwb.py -help
+
+"""
+
 
 class UnhandledAnswer(Exception):
 
@@ -214,15 +224,6 @@ class UnhandledAnswer(Exception):
         self.stop = stop
 
 
-class LoggingFormatter(_LoggingFormatter):
-
-    """Logging formatter that uses config.console_encoding."""
-
-    def __init__(self, fmt=None, datefmt=None):
-        """Initializer setting underlying encoding to console_encoding."""
-        _LoggingFormatter.__init__(self, fmt, datefmt, config.console_encoding)
-
-
 # Initialize the handlers and formatters for the logging system.
 #
 # This relies on the global variable 'ui' which is a UserInterface object
@@ -355,35 +356,29 @@ def writelogheader():
 
     This may help the user to track errors or report bugs.
     """
-    # If a http thread is not available, it's too early to print a header
-    # that includes version information, which may need to query a server.
-    # The http module can't be imported due to circular dependencies.
-    http = sys.modules.get('pywikibot.comms.http', None)
-    if not http or not hasattr(http, 'threads') or not len(http.threads):
-        return
-
+    log('')
     log('=== Pywikibot framework v{} -- Logging header ==='
         .format(pywikibot.__version__))
 
     # script call
-    log('COMMAND: {0}'.format(sys.argv))
+    log('COMMAND: {}'.format(sys.argv))
 
     # script call time stamp
-    log('DATE: %s UTC' % str(datetime.datetime.utcnow()))
+    log('DATE: {} UTC'.format(datetime.datetime.utcnow()))
 
     # new framework release/revision? (handle_args needs to be called first)
     try:
-        log('VERSION: %s' %
-            version.getversion(online=config.log_pywiki_repo_version).strip())
+        log('VERSION: {}'.format(
+            version.getversion(online=config.log_pywiki_repo_version).strip()))
     except version.ParseError:
         exception()
 
     # system
     if hasattr(os, 'uname'):
-        log('SYSTEM: {0}'.format(os.uname()))
+        log('SYSTEM: {}'.format(os.uname()))
 
     # config file dir
-    log('CONFIG FILE DIR: %s' % pywikibot.config2.base_dir)
+    log('CONFIG FILE DIR: {}'.format(pywikibot.config2.base_dir))
 
     all_modules = sys.modules.keys()
 
@@ -397,33 +392,37 @@ def writelogheader():
     if config.verbose_output:
         check_package_list += all_modules
 
-    packages = version.package_versions(check_package_list)
-
     log('PACKAGES:')
+    packages = version.package_versions(check_package_list)
     for name in sorted(packages.keys()):
         info = packages[name]
         info.setdefault('path',
                         '[{}]'.format(info.get('type', 'path unknown')))
         info.setdefault('ver', '??')
         if 'err' in info:
-            log('  %(name)s: %(err)s' % info)
+            log('  {name}: {err}'.format_map(info))
         else:
-            log('  %(name)s (%(path)s) = %(ver)s' % info)
+            log('  {name} ({path}) = {ver}'.format_map(info))
 
     # imported modules
     log('MODULES:')
     for module in sys.modules.values():
         filename = version.get_module_filename(module)
-        ver = version.get_module_version(module)
-        mtime = version.get_module_mtime(module)
-        if filename and ver and mtime:
-            log('  {0} {1} {2}'
-                .format(filename, ver[:7], mtime.isoformat(' ')))
+        if not filename:
+            continue
+
+        param = {'sep': ' '}
+        if PYTHON_VERSION >= (3, 6, 0):
+            param['timespec'] = 'seconds'
+        mtime = version.get_module_mtime(module).isoformat(**param)
+
+        log('  {} {}'
+            .format(mtime, filename))
 
     if config.log_pywiki_repo_version:
-        log('PYWIKI REPO VERSION: %s' % version.getversion_onlinerepo())
+        log('PYWIKI REPO VERSION: {}'.format(version.getversion_onlinerepo()))
 
-    log('=== ' * 14)
+    log('=' * 57)
 
 
 add_init_routine(init_handlers)
@@ -432,20 +431,16 @@ add_init_routine(init_handlers)
 # User input functions
 
 
-def input(question, password=False, default='', force=False):
+def input(question: str, password: bool = False,
+          default: str = '', force: bool = False) -> str:
     """Ask the user a question, return the user's answer.
 
     @param question: a string that will be shown to the user. Don't add a
         space after the question mark/colon, this method will do this for you.
-    @type question: str
     @param password: if True, hides the user's input (for password entry).
-    @type password: bool
     @param default: The default answer if none was entered. None to require
         an answer.
-    @type default: basestring
     @param force: Automatically use the default
-    @type force: bool
-    @rtype: str
     """
     # make sure logging system has been initialized
     if not _handlers_initialized:
@@ -455,13 +450,13 @@ def input(question, password=False, default='', force=False):
     return data
 
 
-def input_choice(question, answers, default=None, return_shortcut=True,
-                 automatic_quit=True, force=False):
+def input_choice(question: str, answers, default: Optional[str] = None,
+                 return_shortcut: bool = True,
+                 automatic_quit: bool = True, force: bool = False):
     """
     Ask the user the question and return one of the valid answers.
 
     @param question: The question asked without trailing spaces.
-    @type question: basestring
     @param answers: The valid answers each containing a full length answer and
         a shortcut. Each value must be unique.
     @type answers: iterable containing a sequence of length two or instances of
@@ -469,19 +464,15 @@ def input_choice(question, answers, default=None, return_shortcut=True,
     @param default: The result if no answer was entered. It must not be in the
         valid answers and can be disabled by setting it to None. If it should
         be linked with the valid answers it must be its shortcut.
-    @type default: basestring
     @param return_shortcut: Whether the shortcut or the index of the answer is
         returned.
-    @type return_shortcut: bool
     @param automatic_quit: Adds the option 'Quit' ('q') and throw a
         L{QuitKeyboardInterrupt} if selected.
-    @type automatic_quit: bool
     @param force: Automatically use the default
-    @type force: bool
     @return: The selected answer shortcut or index. Is -1 if the default is
         selected, it does not return the shortcut and the default is not a
         valid shortcut.
-    @rtype: int (if not return shortcut), basestring (otherwise)
+    @rtype: int (if not return shortcut), str (otherwise)
     """
     # make sure logging system has been initialized
     if not _handlers_initialized:
@@ -491,48 +482,22 @@ def input_choice(question, answers, default=None, return_shortcut=True,
                            automatic_quit=automatic_quit, force=force)
 
 
-@deprecated('input_choice', since='20140825', future_warning=True)
-def inputChoice(question, answers, hotkeys, default=None):
-    """Ask the user a question with several options, return the user's choice.
-
-    DEPRECATED: Use L{input_choice} instead!
-
-    The user's input will be case-insensitive, so the hotkeys should be
-    distinctive case-insensitively.
-
-    @param question: a string that will be shown to the user. Don't add a
-        space after the question mark/colon, this method will do this for you.
-    @type question: basestring
-    @param answers: a list of strings that represent the options.
-    @type answers: list of basestring
-    @param hotkeys: a list of one-letter strings, one for each answer.
-    @param default: an element of hotkeys, or None. The default choice that
-        will be returned when the user just presses Enter.
-    @return: a one-letter string in lowercase.
-    @rtype: str
-    """
-    return input_choice(question, zip(answers, hotkeys), default=default,
-                        automatic_quit=False)
-
-
-def input_yn(question, default=None, automatic_quit=True, force=False):
+def input_yn(question: str,
+             default: Union[str, bool, None] = None,
+             automatic_quit: bool = True,
+             force: bool = False) -> bool:
     """
     Ask the user a yes/no question and return the answer as a bool.
 
     @param question: The question asked without trailing spaces.
-    @type question: basestring
     @param default: The result if no answer was entered. It must be a bool or
         'y' or 'n' and can be disabled by setting it to None.
-    @type default: basestring or bool
     @param automatic_quit: Adds the option 'Quit' ('q') and throw a
         L{QuitKeyboardInterrupt} if selected.
-    @type automatic_quit: bool
     @param force: Automatically use the default
-    @type force: bool
     @return: Return True if the user selected yes and False if the user
         selected no. If the default is not None it'll return True if default
         is True or 'y' and False if default is False or 'n'.
-    @rtype: bool
     """
     if default not in ['y', 'Y', 'n', 'N']:
         if default:
@@ -546,21 +511,18 @@ def input_yn(question, default=None, automatic_quit=True, force=False):
                         automatic_quit=automatic_quit, force=force) == 'y'
 
 
-def input_list_choice(question, answers, default=None, force=False):
+def input_list_choice(question: str, answers: Sequence[Any],
+                      default: Union[int, str, None] = None,
+                      force: bool = False) -> str:
     """
     Ask the user the question and return one of the valid answers.
 
     @param question: The question asked without trailing spaces.
-    @type question: basestring
     @param answers: The valid answers each containing a full length answer.
-    @type answers: Iterable of basestring
     @param default: The result if no answer was entered. It must not be in the
         valid answers and can be disabled by setting it to None.
-    @type default: basestring
     @param force: Automatically use the default
-    @type force: bool
     @return: The selected answer.
-    @rtype: basestring
     """
     if not _handlers_initialized:
         init_handlers()
@@ -594,7 +556,8 @@ class InteractiveReplace:
     the Choice instance returned and created by this class are too.
     """
 
-    def __init__(self, old_link, new_link, default=None, automatic_quit=True):
+    def __init__(self, old_link, new_link, default: Optional[str] = None,
+                 automatic_quit: bool = True):
         """
         Initializer.
 
@@ -607,10 +570,8 @@ class InteractiveReplace:
             with allow_replace are ignored.
         @type new_link: pywikibot.page.Link or pywikibot.page.Page or False
         @param default: The default answer as the shortcut
-        @type default: None or str
         @param automatic_quit: Add an option to quit and raise a
             QuitKeyboardException.
-        @type automatic_quit: bool
         """
         if isinstance(old_link, pywikibot.Page):
             self._old = old_link._link
@@ -672,8 +633,7 @@ class InteractiveReplace:
                     break
             self._current_match = None  # don't reset in case of an exception
             return answer
-        else:
-            return None
+        return None
 
     @property
     def choices(self):
@@ -751,18 +711,18 @@ class InteractiveReplace:
 
 
 # Command line parsing and help
-def calledModuleName():
+def calledModuleName() -> str:
     """Return the name of the module calling this function.
 
     This is required because the -help option loads the module's docstring
     and because the module name will be used for the filename of the log.
 
-    @rtype: str
     """
     return Path(pywikibot.argvu[0]).stem
 
 
-def handle_args(args=None, do_help=True):
+def handle_args(args: Optional[Iterable[str]] = None,
+                do_help: bool = True) -> List[str]:
     """
     Handle standard command line arguments, and return the rest as a list.
 
@@ -776,11 +736,8 @@ def handle_args(args=None, do_help=True):
     args may be passed as an argument, thereby overriding sys.argv
 
     @param args: Command line arguments
-    @type args: typing.Iterable
     @param do_help: Handle parameter '-help' to show help and invoke sys.exit
-    @type do_help: bool
     @return: list of arguments not recognised globally
-    @rtype: list of str
     """
     if pywikibot._sites:
         warn('Site objects have been created before arguments were handled',
@@ -802,9 +759,11 @@ def handle_args(args=None, do_help=True):
     for arg in args:
         option, _, value = arg.partition(':')
         if do_help is not False and option == '-help':
-            do_help = True
+            do_help = value or True
         elif option == '-dir':
             pass
+        elif option == '-site':
+            config.family, config.mylang = value.split(':')
         elif option == '-family':
             config.family = value
         elif option == '-lang':
@@ -812,7 +771,7 @@ def handle_args(args=None, do_help=True):
         elif option == '-user':
             username = value
         elif option in ('-putthrottle', '-pt'):
-            config.put_throttle = int(value)
+            config.put_throttle = float(value)
         elif option == '-log':
             if module_name not in config.log:
                 config.log.append(module_name)
@@ -874,11 +833,12 @@ def handle_args(args=None, do_help=True):
                 # argument not global -> specific bot script will take care
                 non_global_args.append(arg)
 
-    try:
-        pywikibot.Site()
-    except (UnknownFamily, UnknownSite):
-        pywikibot.exception()
-        sys.exit(1)
+    if calledModuleName() != 'generate_user_files':  # T261771
+        try:
+            pywikibot.Site()
+        except (UnknownFamily, UnknownSite):
+            pywikibot.exception()
+            sys.exit(1)
 
     if username:
         config.usernames[config.family][config.mylang] = username
@@ -890,20 +850,14 @@ def handle_args(args=None, do_help=True):
         pywikibot.output('Python ' + sys.version)
 
     if do_help:
-        showHelp()
+        show_help(show_global=do_help == 'global')
         sys.exit(0)
 
     debug('handle_args() completed.', _logger)
     return non_global_args
 
 
-@deprecated('handle_args', since='20150409')
-def handleArgs(*args):
-    """DEPRECATED. Use handle_args()."""
-    return handle_args(args)
-
-
-def showHelp(module_name=None):
+def show_help(module_name=None, show_global=False):
     """Show help for the Bot."""
     if not module_name:
         module_name = calledModuleName()
@@ -913,45 +867,53 @@ def showHelp(module_name=None):
         except NameError:
             module_name = 'no_module'
 
-    global_help = _GLOBAL_HELP % module_name
     try:
-        module = import_module('%s' % module_name)
+        module = import_module(module_name)
         help_text = module.__doc__
         if hasattr(module, 'docuReplacements'):
             for key, value in module.docuReplacements.items():
                 help_text = help_text.replace(key, value.strip('\n\r'))
-        pywikibot.stdout(help_text)  # output to STDOUT
     except Exception:
         if module_name:
-            pywikibot.stdout('Sorry, no help available for %s' % module_name)
-        pywikibot.log('showHelp:', exc_info=True)
-    pywikibot.stdout(global_help)
+            pywikibot.stdout('Sorry, no help available for ' + module_name)
+        pywikibot.log('show_help:', exc_info=True)
+    else:
+        pywikibot.stdout(help_text)  # output to STDOUT
+
+    if show_global or module_name == 'pwb':
+        pywikibot.stdout(_GLOBAL_HELP.format(module_name))
+    else:
+        pywikibot.stdout(_GLOBAL_HELP_NOTE)
 
 
-def suggest_help(missing_parameters=[], missing_generator=False,
-                 unknown_parameters=[], exception=None,
-                 missing_action=False, additional_text='',
-                 missing_dependencies=[]):
+@deprecated('show_help', since='20200705')
+def showHelp(module_name=None):  # pragma: no cover
+    """DEPRECATED. Use show_help()."""
+    return show_help(module_name)
+
+
+def suggest_help(missing_parameters: Optional[List[str]] = None,
+                 missing_generator: bool = False,
+                 unknown_parameters: Optional[List[str]] = None,
+                 exception=None,
+                 missing_action: bool = False,
+                 additional_text: str = '',
+                 missing_dependencies: Optional[List[str]] = None) -> bool:
     """
     Output error message to use -help with additional text before it.
 
     @param missing_parameters: A list of parameters which are missing.
-    @type missing_parameters: list of str
     @param missing_generator: Whether a generator is missing.
-    @type missing_generator: bool
     @param unknown_parameters: A list of parameters which are unknown.
-    @type unknown_parameters: list of str
     @param exception: An exception thrown.
     @type exception: Exception
     @param missing_action: Add an entry that no action was defined.
     @type missing_action: bool
     @param additional_text: Additional text added to the end.
-    @type additional_text: str
-    @param missing_dependencies: A list of dependencies which can not
-        be imported.
+    @param missing_dependencies: A list of dependencies which cannot be
+        imported.
     @type missing_dependencies: list of str
     @return: True if an error message was printed, False otherwise
-    @rtype: bool
     """
     messages = []
     if exception:
@@ -1011,58 +973,175 @@ def writeToCommandLogFile():
 def open_webbrowser(page):
     """Open the web browser displaying the page and wait for input."""
     webbrowser.open(page.full_url())
-    i18n.input('pywikibot-enter-finished-browser')
+    i18n.input('pywikibot-enter-finished-browser',
+               fallback_prompt='Press Enter when finished in browser.')
+
+
+class _OptionDict(dict):
+
+    """The option dict which holds the options of OptionHandler."""
+
+    def __init__(self, classname, options):
+        self._classname = classname
+        self._options = {}
+        super().__init__(options)
+
+    def __missing__(self, key):
+        raise pywikibot.Error("'{}' is not a valid option for {}."
+                              .format(key, self._classname))
+
+    def __getattr__(self, name):
+        """Get item from dict."""
+        return self.__getitem__(name)
+
+    def __setattr__(self, name, value):
+        """Set item or attribute."""
+        if name not in ('_classname', '_options'):
+            self.__setitem__(name, value)
+        else:
+            super().__setattr__(name, value)
+
+    def __getitem__(self, key):
+        """Update options fro backward compatibility and get item."""
+        self.update_options()
+        return super().__getitem__(key)
+
+    def __setitem__(self, key, value):
+        """Set item in dict and deprecated option dict."""
+        if key in self._options:
+            self._options[key] = value
+        super().__setitem__(key, value)
+
+    def update_options(self):
+        """Update dict from deprecated options for backward compatibility."""
+        self.update(self._options)
+
+
+_DEPRECATION_MSG = 'Optionhandler.opt.option attribute ' \
+                   'or Optionhandler.opt[option] item'
 
 
 class OptionHandler:
 
-    """Class to get and set options."""
+    """Class to get and set options.
+
+    How to use options of OptionHandler and its BaseBot subclasses:
+    First define an available_options class attribute for your own
+    option handler to define all available options:
+
+    >>> default_options = {'foo': 'bar', 'bar': 42, 'baz': False}
+    >>> class MyHandler(OptionHandler): available_options = default_options
+
+    Or you may update the predefined setting in the class initializer.
+    BaseBot predefines a 'always' options and sets it to False:
+
+    self.available_options.update(always=True, another_option='Yes')
+
+    Now you can instantiate an OptionHandler or BaseBot class passing
+    options other than default values:
+
+    >>> bot = MyHandler(baz=True)
+
+    You can access bot options either as keyword item or attribute:
+
+    >>> bot.opt.foo
+    'bar'
+    >>> bot.opt['bar']
+    42
+    >>> bot.opt.baz  # default was overridden
+    True
+
+    You can set the options in the same way:
+
+    >>> bot.opt.bar = 4711
+    >>> bot.opt['baz'] = None
+    >>>
+
+    Or you can use the option as a dict:
+
+    >>> 'Option opt.{foo} is {bar}'.format_map(bot.opt)
+    'Option opt.bar is 4711'
+    """
 
     # Handler configuration.
     # Only the keys of the dict can be passed as init options
     # The values are the default values
     # Overwrite this in subclasses!
 
-    availableOptions = {}
+    available_options = {}  # type: Dict[str, Any]
 
     def __init__(self, **kwargs):
         """
-        Only accept options defined in availableOptions.
+        Only accept options defined in available_options.
 
         @param kwargs: bot options
         """
-        self.setOptions(**kwargs)
+        self.set_options(**kwargs)
 
-    def setOptions(self, **kwargs):
-        """
-        Set the instance options.
+    @property
+    @deprecated('available_options', since='20201006')
+    def availableOptions(self):
+        """DEPRECATED. Available_options class property."""
+        return self.available_options
 
-        @param kwargs: options
-        """
-        valid_options = set(self.availableOptions)
-        received_options = set(kwargs)
+    @deprecated('set_options', since='20201006')
+    def setOptions(self, **kwargs):  # pragma: no cover
+        """DEPRECATED. Set the instance options."""
+        self.set_options(**kwargs)
 
-        # contains the options overridden from defaults
-        self.options = {
-            opt: kwargs[opt] for opt in received_options & valid_options}
+    def set_options(self, **options):
+        """Set the instance options."""
+        warning = 'pywikibot.bot.OptionHandler.availableOptions'
+        with suppress_warnings(warning.replace('.', r'\.') + ' is deprecated',
+                               category=DeprecationWarning):
+            old_options = self.availableOptions is not self.available_options
+        if old_options:  # old options were set and not updated
+            self.available_options = self.availableOptions
+            issue_deprecation_warning(warning, 'available_options',
+                                      since='20201006')
+
+        valid_options = set(self.available_options)
+        received_options = set(options)
+
+        # self.opt contains all available options including defaults
+        self.opt = _OptionDict(self.__class__.__name__, self.available_options)
+        # self.options contains the options overridden from defaults
+        self.opt._options = {opt: options[opt]
+                             for opt in received_options & valid_options}
+        self.opt.update(self.opt._options)
 
         for opt in received_options - valid_options:
-            pywikibot.warning('%s is not a valid option. It was ignored.'
-                              % opt)
+            pywikibot.warning('{} is not a valid option. It was ignored.'
+                              .format(opt))
 
-    def getOption(self, option):
-        """
-        Get the current value of an option.
+    @deprecated(_DEPRECATION_MSG, since='20201006')
+    def getOption(self, option):  # pragma: no cover
+        """DEPRECATED. Get the current value of an option.
 
-        @param option: key defined in OptionHandler.availableOptions
+        @param option: key defined in OptionHandler.available_options
         @raise pywikibot.exceptions.Error: No valid option is given with
             option parameter
         """
-        try:
-            return self.options.get(option, self.availableOptions[option])
-        except KeyError:
-            raise pywikibot.Error("'{0}' is not a valid option for {1}."
-                                  .format(option, self.__class__.__name__))
+        return self.opt[option]
+
+    @property
+    @deprecated(_DEPRECATION_MSG, since='20201006', future_warning=True)
+    def options(self):  # pragma: no cover
+        """DEPRECATED. Return changed options."""
+        return self.opt._options
+
+    @options.setter
+    @deprecated(_DEPRECATION_MSG, since='20201006', future_warning=True)
+    def options(self, options):  # pragma: no cover
+        """DEPRECATED. Return changed options."""
+        self.set_options(**options)
+
+    def __getattribute__(self, name):
+        """Update options for backward compatibility of options property."""
+        attr = super().__getattribute__(name)
+        if name == 'opt':
+            attr.update_options()
+        return attr
 
 
 class BaseBot(OptionHandler):
@@ -1079,26 +1158,32 @@ class BaseBot(OptionHandler):
 
     If the subclass does not set a generator, or does not override
     treat() or run(), NotImplementedError is raised.
+
+    For bot options handling refer OptionHandler class above.
     """
 
     # Handler configuration.
     # The values are the default values
     # Extend this in subclasses!
 
-    availableOptions = {
+    available_options = {
         'always': False,  # By default ask for confirmation when putting a page
     }
 
     _current_page = None
 
     def __init__(self, **kwargs):
-        """
-        Only accept options defined in availableOptions.
+        """Only accept 'generator' and options defined in available_options.
 
         @param kwargs: bot options
+        @keyword generator: a generator processed by run method
         """
         if 'generator' in kwargs:
-            self.generator = kwargs.pop('generator')
+            if hasattr(self, 'generator'):
+                pywikibot.warn('{} has a generator already. Ignoring argument.'
+                               .format(self.__class__.__name__))
+            else:
+                self.generator = kwargs.pop('generator')
 
         super().__init__(**kwargs)
 
@@ -1137,7 +1222,7 @@ class BaseBot(OptionHandler):
 
     def user_confirm(self, question):
         """Obtain user response if bot option 'always' not enabled."""
-        if self.getOption('always'):
+        if self.opt.always:
             return True
 
         choice = pywikibot.input_choice(question,
@@ -1156,7 +1241,7 @@ class BaseBot(OptionHandler):
 
         if choice == 'a':
             # Remember the choice
-            self.options['always'] = True
+            self.opt.always = True
 
         return True
 
@@ -1187,7 +1272,7 @@ class BaseBot(OptionHandler):
         if oldtext.rstrip() == newtext.rstrip():
             pywikibot.output('No changes were needed on %s'
                              % page.title(as_link=True))
-            return
+            return False
 
         self.current_page = page
 
@@ -1222,7 +1307,7 @@ class BaseBot(OptionHandler):
         if not self.user_confirm('Do you want to accept these changes?'):
             return False
 
-        if 'asynchronous' not in kwargs and self.getOption('always'):
+        if 'asynchronous' not in kwargs and self.opt.always:
             kwargs['asynchronous'] = True
 
         ignore_save_related_errors = kwargs.pop('ignore_save_related_errors',
@@ -1259,7 +1344,7 @@ class BaseBot(OptionHandler):
         return False
 
     @deprecated('generator.close()', since='20200804')
-    def stop(self):
+    def stop(self):  # pragma: no cover
         """Stop iterating."""
         pywikibot.output('Generator has been stopped.')
         self.generator.close()
@@ -1312,10 +1397,7 @@ class BaseBot(OptionHandler):
         # exc_info contains exception from self.run() while terminating
         exc_info = sys.exc_info()
         pywikibot.output('Script terminated ', newline=False)
-        # Python 2 also needs QuitKeyboardInterrupt
-        # to be compared with exc_info[0] (T195687)
-        if exc_info[0] is None or exc_info[0] in (KeyboardInterrupt,
-                                                  QuitKeyboardInterrupt):
+        if exc_info[0] is None or exc_info[0] is KeyboardInterrupt:
             pywikibot.output('successfully.')
         else:
             pywikibot.output('by exception:\n')
@@ -1377,6 +1459,9 @@ class BaseBot(OptionHandler):
         if not hasattr(self, 'generator'):
             raise NotImplementedError('Variable {}.generator not set.'
                                       .format(self.__class__.__name__))
+        if not isinstance(self.generator, Generator):
+            # to provide close() method
+            self.generator = (item for item in self.generator)
         try:
             for item in self.generator:
                 # preprocessing of the page
@@ -1462,15 +1547,14 @@ class Bot(BaseBot):
             return
 
         if site not in self._sites:
-            log('LOADING SITE %s VERSION: %s'
-                % (site, site.version()))
+            log('LOADING SITE {} VERSION: {}'.format(site, site.mw_version))
 
             self._sites.add(site)
             if len(self._sites) == 2:
-                log('%s uses multiple sites' % self.__class__.__name__)
+                log('{} uses multiple sites'.format(self.__class__.__name__))
         if self._site and self._site != site:
-            log('%s: changing site from %s to %s'
-                % (self.__class__.__name__, self._site, site))
+            log('{}: changing site from {} to {}'
+                .format(self.__class__.__name__, self._site, site))
         self._site = site
 
     def run(self):
@@ -1579,7 +1663,8 @@ class MultipleSitesBot(BaseBot):
         super().__init__(**kwargs)
 
     @property
-    @deprecated("the page's site property", since='20150615')
+    @deprecated("the page's site property", since='20150615',
+                future_warning=True)
     def site(self):
         """
         Return the site if it's set and ValueError otherwise.
@@ -1608,7 +1693,7 @@ class ConfigParserBot(BaseBot):
 
     """A bot class that can read options from scripts.ini file.
 
-    All options must be predefined in availableOptions dictionary. The type
+    All options must be predefined in available_options dictionary. The type
     of these options is responsible for the correct interpretation of the
     options type given by the .ini file. They can be interpreted as bool,
     int, float or str (default). The settings file may be like:
@@ -1622,40 +1707,40 @@ class ConfigParserBot(BaseBot):
 
     The option values are interpreted in this order::
 
-    - availableOptions default setting
+    - available_options default setting
     - script.ini options settings
     - command line arguments
     """
 
     INI = 'scripts.ini'
 
-    def setOptions(self, **kwargs):
+    def set_options(self, **kwargs):
         """Read settings from scripts.ini file."""
         conf = configparser.ConfigParser(inline_comment_prefixes=[';'])
         section = calledModuleName()
 
         if (conf.read(self.INI) == [self.INI] and conf.has_section(section)):
             pywikibot.output('Reading settings from {} file.'.format(self.INI))
-            args = {}
-            for option, value in self.availableOptions.items():
+            options = {}
+            for option, value in self.available_options.items():
                 if not conf.has_option(section, option):
                     continue
                 # use a convenience parser method, default to get()
-                default = getattr(conf, 'get')
+                default = conf.get
                 value_type = type(value).__name__
                 if value_type == 'bool':
-                    method = getattr(conf, 'getboolean')
+                    method = conf.getboolean
                 else:
                     method = getattr(conf, 'get' + value_type, default)
-                args[option] = method(section, option)
-            for opt in set(conf.options(section)) - set(args):
+                options[option] = method(section, option)
+            for opt in set(conf.options(section)) - set(options):
                 pywikibot.warning(
                     '"{}" is not a valid option. It was ignored.'.format(opt))
-            args.update(kwargs)
+            options.update(kwargs)
         else:
-            args = kwargs
+            options = kwargs
 
-        super().setOptions(**args)
+        super().set_options(**options)
 
 
 class CurrentPageBot(BaseBot):
@@ -1681,15 +1766,14 @@ class CurrentPageBot(BaseBot):
         self.treat_page()
 
     @deprecated_args(comment='summary')
-    def put_current(self, new_text, ignore_save_related_errors=None,
-                    ignore_server_errors=None, **kwargs):
+    def put_current(self, new_text: str, ignore_save_related_errors=None,
+                    ignore_server_errors=None, **kwargs) -> bool:
         """
         Call L{Bot.userPut} but use the current page.
 
         It compares the new_text to the current page text.
 
         @param new_text: The new text
-        @type new_text: basestring
         @param ignore_save_related_errors: Ignore save related errors and
             automatically print a message. If None uses this instances default.
         @type ignore_save_related_errors: bool or None
@@ -1698,7 +1782,6 @@ class CurrentPageBot(BaseBot):
         @type ignore_server_errors: bool or None
         @param kwargs: Additional parameters directly given to L{Bot.userPut}.
         @return: whether the page was saved successfully
-        @rtype: bool
         """
         if ignore_save_related_errors is None:
             ignore_save_related_errors = self.ignore_save_related_errors
@@ -1832,7 +1915,7 @@ class WikidataBot(Bot, ExistingPageBot):
 
     @ivar create_missing_item: If True, new items will be created if the
         current page doesn't have one. Subclasses should override this in the
-        initializer with a bool value or using self.getOption.
+        initializer with a bool value or using self.opt attribute.
 
     @type create_missing_item: bool
     """
@@ -2089,8 +2172,7 @@ class WikidataBot(Bot, ExistingPageBot):
         result = self.user_edit_entity(item, data, summary=summary, **kwargs)
         if result:
             return item
-        else:
-            return None
+        return None
 
     def treat_page(self):
         """Treat a page."""
