@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Server-Sent Events client.
 
@@ -18,9 +17,11 @@ from functools import partial
 import json
 import socket
 
+from typing import Optional
+
 from requests import __version__ as requests_version
 from requests.packages.urllib3.exceptions import ProtocolError
-from requests.packages.urllib3.response import httplib
+from requests.packages.urllib3.util.response import httplib
 
 try:
     from sseclient import SSEClient as EventSource
@@ -87,7 +88,7 @@ class EventStreams:
             Multiple streams may be given as a string with comma separated
             stream types or an iterable of strings
             Refer https://stream.wikimedia.org/?doc for available
-            wikimedia stream types.
+            Wikimedia stream types.
         @type streams: str or iterable
         @keyword timeout: a timeout value indication how long to wait to send
             data before giving up
@@ -116,7 +117,7 @@ class EventStreams:
             if isinstance(self._since, str) and '-' not in self._since:
                 self._since = Timestamp.fromtimestampformat(self._since)
             if isinstance(self._since, Timestamp):
-                self._since = self._since.isoformat
+                self._since = self._since.isoformat()
 
         self._url = kwargs.get('url') or self.url
         kwargs.setdefault('url', self._url)
@@ -157,7 +158,7 @@ class EventStreams:
                                         if self._since else '')))
         return self._url
 
-    def set_maximum_items(self, value):
+    def set_maximum_items(self, value: int):
         """
         Set the maximum number of items to be retrieved from the stream.
 
@@ -166,7 +167,6 @@ class EventStreams:
 
         @param value: The value of maximum number of items to be retrieved
             in total to set.
-        @type value: int
         """
         if value is not None:
             self._total = int(value)
@@ -226,7 +226,7 @@ class EventStreams:
         @param args: You may pass your own filter functions here.
             Every function should be able to handle the data dict from events.
         @type args: callable
-        @param kwargs: Any key returned by event data with a event data value
+        @param kwargs: Any key returned by event data with an event data value
             for this given key.
         @type kwargs: str, list, tuple or other sequence
         @raise TypeError: A given args parameter is not a callable.
@@ -261,13 +261,12 @@ class EventStreams:
             else:
                 self.filter[ftype].append(partial(_in, key=key, value=value))
 
-    def streamfilter(self, data):
+    def streamfilter(self, data: dict):
         """Filter function for eventstreams.
 
         See the description of register_filter() how it works.
 
         @param data: event data dict used by filter functions
-        @type data: dict
         """
         if any(function(data) for function in self.filter['none']):
             return False
@@ -328,13 +327,12 @@ class EventStreams:
         del self.source
 
 
-def site_rc_listener(site, total=None):
+def site_rc_listener(site, total: Optional[int] = None):
     """Yield changes received from EventStream.
 
     @param site: the Pywikibot.Site object to yield live recent changes for
     @type site: Pywikibot.BaseSite
     @param total: the maximum number of changes to return
-    @type total: int
 
     @return: pywikibot.comms.eventstream.rc_listener configured for given site
     @raises ImportError: sseclient installation is required
