@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Module to define and load pywikibot configuration default and user preferences.
 
@@ -7,20 +6,15 @@ may be located in directory specified by the environment variable
 PYWIKIBOT_DIR, or the same directory as pwb.py, or in a directory within
 the users home. See get_base_dir for more information.
 
-If user-config.py can not be found in any of those locations, this module
+If user-config.py cannot be found in any of those locations, this module
 will fail to load unless the environment variable PYWIKIBOT_NO_USER_CONFIG
 is set to a value other than '0'. i.e. PYWIKIBOT_NO_USER_CONFIG=1 will
 allow config to load without a user-config.py. However, warnings will be
-shown if user-config.py was not loaded.
-To prevent these warnings, set PYWIKIBOT_NO_USER_CONFIG=2.
+shown if user-config.py was not loaded. To prevent these warnings, set
+PYWIKIBOT_NO_USER_CONFIG=2. If pywikibot is installed as a site-package
+the behaviour is like PYWIKIBOT_NO_USER_CONFIG=2 is set.
 
-Provides two functions to register family classes which can be used in
-the user-config:
-
- - register_family_file
- - register_families_folder
-
-Other functions made available to user-config:
+Functions made available to user-config:
 
  - user_home_path
 
@@ -32,7 +26,7 @@ build paths relative to base_dir:
  - shortpath
 """
 #
-# (C) Pywikibot team, 2003-2020
+# (C) Pywikibot team, 2003-2021
 #
 # Distributed under the terms of the MIT license.
 #
@@ -45,16 +39,18 @@ import stat
 import sys
 import types
 
-from distutils.version import StrictVersion
 from locale import getdefaultlocale
 from os import getenv, environ
-from requests import __version__ as requests_version
+from pathlib import Path
 from textwrap import fill
+from typing import Optional, Union
 from warnings import warn
 
-from pywikibot import __version__ as pwb_version
+from pywikibot.__metadata__ import __version__ as pwb_version
+from pywikibot.backports import Dict, List, removesuffix, Tuple
 from pywikibot.logging import error, output, warning
-from pywikibot.tools import issue_deprecation_warning
+from pywikibot.tools import issue_deprecation_warning, deprecated
+
 
 OSWIN32 = (sys.platform == 'win32')
 
@@ -83,7 +79,9 @@ for env_name in (
 _imports = frozenset(name for name in globals() if not name.startswith('_'))
 
 __no_user_config = getenv('PYWIKIBOT_NO_USER_CONFIG')
-if __no_user_config == '0':
+if __no_user_config is None and 'site-packages' in Path(__file__).parts:
+    __no_user_config = '2'
+elif __no_user_config == '0':
     __no_user_config = None
 
 
@@ -106,9 +104,20 @@ class _ConfigurationDeprecationWarning(UserWarning):
 
 _private_values = {'authenticate', 'db_password'}
 _deprecated_variables = {
-    'available_ssl_project', 'fake_user_agent', 'interwiki_contents_on_disk',
-    'line_separator', 'LS', 'panoramio', 'proxy', 'special_page_limit',
+    'available_ssl_project', 'copyright_check_in_source_google',
+    'copyright_check_in_source_msn', 'copyright_check_in_source_section_names',
+    'copyright_check_in_source_yahoo', 'copyright_connection_tries',
+    'copyright_economize_query', 'copyright_exceeded_in_queries',
+    'copyright_exceeded_in_queries_sleep_hours', 'copyright_google',
+    'copyright_max_query_for_page', 'copyright_msn', 'copyright_show_date',
+    'copyright_show_length', 'copyright_skip_query', 'copyright_yahoo',
+    'db_hostname', 'deIndentTables', 'fake_user_agent', 'flickr',
+    'interwiki_contents_on_disk', 'interwiki_backlink', 'interwiki_graph',
+    'interwiki_graph_formats', 'interwiki_graph_url', 'interwiki_min_subjects',
+    'interwiki_shownew', 'line_separator', 'LS', 'msn_appid', 'panoramio',
+    'persistent_http', 'proxy', 'special_page_limit', 'splitLongParagraphs',
     'sysopnames', 'use_mwparserfromhell', 'use_SSL_onlogin', 'use_SSL_always',
+    'without_interwiki', 'yahoo_appid',
 }
 _future_variables = {'absolute_import', 'division', 'unicode_literals'}
 
@@ -135,7 +144,7 @@ mylang = 'language'
 # usernames['wikibooks']['*'] = 'mySingleUsername'
 # You may use '*' for family name in a similar manner.
 #
-usernames = collections.defaultdict(dict)
+usernames = collections.defaultdict(dict)  # type: Dict[str, Dict[str, str]]
 disambiguation_comment = collections.defaultdict(dict)
 
 # User agent format.
@@ -272,7 +281,7 @@ def user_home_path(path):
     return os.path.join(os.path.expanduser('~'), path)
 
 
-def get_base_dir(test_directory=None):
+def get_base_dir(test_directory: Optional[str] = None) -> str:
     r"""Return the directory in which user-specific information is stored.
 
     This is determined in the following order:
@@ -288,20 +297,18 @@ def get_base_dir(test_directory=None):
          '.pywikibot' directory (Unix and similar) under the user's home
          directory.
 
-    Set PYWIKIBOT_NO_USER_CONFIG=1 to disable loading user-config.py
+    Set PYWIKIBOT_NO_USER_CONFIG=1 to disable loading user-config.py or
+    install pywikibot as a site-package.
 
     @param test_directory: Assume that a user config file exists in this
         directory. Used to test whether placing a user config file in this
         directory will cause it to be selected as the base directory.
-    @type test_directory: str or None
-    @rtype: str
     """
     def exists(directory):
         directory = os.path.abspath(directory)
         if directory == test_directory:
             return True
-        else:
-            return os.path.exists(os.path.join(directory, 'user-config.py'))
+        return os.path.exists(os.path.join(directory, 'user-config.py'))
 
     if test_directory is not None:
         test_directory = os.path.abspath(test_directory)
@@ -354,17 +361,18 @@ def get_base_dir(test_directory=None):
         raise RuntimeError("Directory '%s' does not exist." % base_dir)
     # check if user-config.py is in base_dir
     if not exists(base_dir):
-        exc_text = "No user-config.py found in directory '%s'.\n" % base_dir
-        if __no_user_config:
-            if __no_user_config != '2':
-                output(exc_text)
-        else:
+        exc_text = 'No user-config.py found in directory {!r}.\n'.format(
+            base_dir)
+
+        if __no_user_config is None:
             exc_text += (
                 '  Please check that user-config.py is stored in the correct '
                 'location.\n'
                 '  Directory where user-config.py is searched is determined '
                 'as follows:\n\n    ') + get_base_dir.__doc__
             raise RuntimeError(exc_text)
+        elif __no_user_config != '2':
+            output(exc_text)
 
     return base_dir
 
@@ -379,24 +387,23 @@ for arg in sys.argv[1:]:
 family_files = {}
 
 
+@deprecated('family_files[family_name] = file_path', since='20210305')
 def register_family_file(family_name, file_path):
     """Register a single family class file.
 
     Parameter file_path may be a path or an url.
     family.AutoFamily function is used when the url is given.
     """
-    usernames[family_name] = {}
-    disambiguation_comment[family_name] = {}
     family_files[family_name] = file_path
 
 
 def register_families_folder(folder_path):
     """Register all family class files contained in a directory."""
+    suffix = '_family.py'
     for file_name in os.listdir(folder_path):
-        if file_name.endswith('_family.py'):
-            family_name = file_name[:-len('_family.py')]
-            register_family_file(family_name, os.path.join(folder_path,
-                                                           file_name))
+        if file_name.endswith(suffix):
+            family_name = removesuffix(file_name, suffix)
+            family_files[family_name] = os.path.join(folder_path, file_name)
 
 
 # Get the names of all known families, and initialize with empty dictionaries.
@@ -443,7 +450,7 @@ userinterface = 'terminal'
 # this can be used to pass variables to the UI init function
 # useful for e.g.
 # userinterface_init_kwargs = {'default_stream': 'stdout'}
-userinterface_init_kwargs = {}
+userinterface_init_kwargs = {}  # type: Dict[str, str]
 
 # i18n setting for user interface language
 # default is obtained from L{locale.getdefaultlocale}
@@ -509,15 +516,14 @@ editor_filename_extension = 'wiki'
 # saved in the 'logs' subdirectory.
 #
 # Example:
-#     log = ['interwiki', 'weblinkchecker', 'table2wiki']
+#     log = ['redirect', 'replace', 'weblinkchecker']
 # It is also possible to enable logging for all scripts, using this line:
 #     log = ['*']
 # To disable all logging, use this:
 #     log = []
-# Per default, logging of interwiki.py is enabled because its logfiles can
-# be used to generate so-called warnfiles.
+# Per default, no logging is enabled.
 # This setting can be overridden by the -log or -nolog command-line arguments.
-log = ['interwiki']
+log = []
 # filename defaults to modulename-bot.log
 logfilename = None
 # maximal size of a logfile in kilobytes. If the size reached that limit the
@@ -554,50 +560,22 @@ debug_log = []
 #
 # sample:
 # user_script_paths = ['scripts.myscripts']
-user_script_paths = []
+user_script_paths = []  # type: List[str]
 
-# ############# INTERWIKI SETTINGS ##############
-
-# Should interwiki.py report warnings for missing links between foreign
-# languages?
-interwiki_backlink = True
-
-# Should interwiki.py display every new link it discovers?
-interwiki_shownew = True
-
-# Should interwiki.py output a graph PNG file on conflicts?
-# You need pydot for this:
-# https://pypi.org/project/pydot/
-interwiki_graph = False
-
-# Specifies that the robot should process that amount of subjects at a time,
-# only starting to load new pages in the original language when the total
-# falls below that number. Default is to process (at least) 100 subjects at
-# once.
-interwiki_min_subjects = 100
-
-# If interwiki graphs are enabled, which format(s) should be used?
-# Supported formats include png, jpg, ps, and svg. See:
-# http://www.graphviz.org/doc/info/output.html
-# If you want to also dump the dot files, you can use this in your
-# user-config.py:
-# interwiki_graph_formats = ['dot', 'png']
-# If you need a PNG image with an HTML image map, use this:
-# interwiki_graph_formats = ['png', 'cmap']
-# If you only need SVG images, use:
-# interwiki_graph_formats = ['svg']
-interwiki_graph_formats = ['png']
-
-# You can post the contents of your autonomous_problems.dat to the wiki,
-# e.g. to https://de.wikipedia.org/wiki/Wikipedia:Interwiki-Konflikte .
-# This allows others to assist you in resolving interwiki problems.
-# To help these people, you can upload the interwiki graphs to your
-# webspace somewhere. Set the base URL here, e.g.:
-# 'https://www.example.org/~yourname/interwiki-graphs/'
-interwiki_graph_url = None
-
-# Save file with local articles without interwikis.
-without_interwiki = False
+# ############# EXTERNAL FAMILIES SETTINGS ##############
+# Set your own family path to lookup for your family files.
+#
+# Your private family path may be either an absolute or a relative path.
+# You may have multiple paths defined in user_families_paths list.
+#
+# You may also define various family files stored directly in
+# family_files dict. Use the family name as dict key and the path or an
+# url as value.
+#
+# samples:
+# family_files['mywiki'] = 'https://de.wikipedia.org'
+# user_families_paths = ['data/families']
+user_families_paths = []  # type: List[str]
 
 # ############# SOLVE_DISAMBIGUATION SETTINGS ############
 #
@@ -632,7 +610,7 @@ maxthrottle = 60
 
 # Slow down the robot such that it never makes a second page edit within
 # 'put_throttle' seconds.
-put_throttle = 10
+put_throttle = 10  # type: Union[int, float]
 
 # Sometimes you want to know when a delay is inserted. If a delay is larger
 # than 'noisysleep' seconds, it is logged on the screen.
@@ -659,15 +637,6 @@ retry_wait = 5
 # Maximum time to wait before resubmitting a failed API request.
 retry_max = 120
 
-# ############# TABLE CONVERSION BOT SETTINGS ##############
-
-# Will split long paragraphs for better reading the source.
-# Only table2wiki.py use it by now.
-splitLongParagraphs = False
-# sometimes HTML-tables are indented for better reading.
-# That can do very ugly results.
-deIndentTables = True
-
 # ############# WEBLINK CHECKER SETTINGS ##############
 
 # How many external links should weblinkchecker.py check at the same time?
@@ -684,11 +653,11 @@ weblink_dead_days = 7
 # Setting to connect the database or replica of the database of the wiki.
 # db_name_format can be used to manipulate the dbName of site.
 #
-# Example for a pywikibot running on wmflabs:
-# db_hostname = 'enwiki.analytics.db.svc.eqiad.wmflabs'
+# Example for a pywikibot running on Wikimedia Cloud (Toolforge):
+# db_hostname_format = '{0}.analytics.db.svc.wikimedia.cloud'
 # db_name_format = '{0}_p'
 # db_connect_file = user_home_path('replica.my.cnf')
-db_hostname = 'localhost'
+db_hostname_format = 'localhost'
 db_username = ''
 db_password = ''
 db_name_format = '{0}'
@@ -698,99 +667,18 @@ db_connect_file = user_home_path('.my.cnf')
 #     user@login.toolforge.org
 db_port = 3306
 
-# ############# SEARCH ENGINE SETTINGS ##############
-# Live search web service appid settings.
-#
-# Yahoo! Search Web Services are not operational.
-# See https://phabricator.wikimedia.org/T106085
-yahoo_appid = ''
-
-# To use Windows Live Search web service you must get an AppID from
-# http://www.bing.com/dev/en-us/dev-center
-msn_appid = ''
-
-# ############# FLICKR RIPPER SETTINGS ##############
-
-# Using the Flickr api
-flickr = {
-    'api_key': '',  # Provide your key!
-    'api_secret': '',  # Api secret of your key (optional)
-    'review': False,  # Do we use automatically make our uploads reviewed?
-    'reviewer': '',  # If so, under what reviewer name?
-}
-
-# ############# COPYRIGHT SETTINGS ##############
-
-# Enable/disable search engine in copyright.py script
-copyright_google = True
-copyright_yahoo = True
-copyright_msn = False
-
-# Perform a deep check, loading URLs to search if 'Wikipedia' is present.
-# This may be useful to increase the number of correct results. If you haven't
-# a fast connection, you might want to keep them disabled.
-copyright_check_in_source_google = False
-copyright_check_in_source_yahoo = False
-copyright_check_in_source_msn = False
-
-# Web pages may contain a Wikipedia text without the word 'Wikipedia' but with
-# the typical '[edit]' tag as a result of a copy & paste procedure. You want
-# no report for this kind of URLs, even if they are copyright violations.
-# However, when enabled, these URLs are logged in a file.
-copyright_check_in_source_section_names = False
-
-# Limit number of queries for page.
-copyright_max_query_for_page = 25
-
-# Skip a specified number of queries
-copyright_skip_query = 0
-
-# Number of attempts on connection error.
-copyright_connection_tries = 10
-
-# Behavior if an exceeded error occur.
-#
-# Possibilities:
-#
-#    0 = None
-#    1 = Disable search engine
-#    2 = Sleep (default)
-#    3 = Stop
-copyright_exceeded_in_queries = 2
-copyright_exceeded_in_queries_sleep_hours = 6
-
-# Append last modified date of URL to script result
-copyright_show_date = True
-
-# Append length of URL to script result
-copyright_show_length = True
-
-# By default the script tries to identify and skip text that contains a large
-# comma separated list or only numbers. But sometimes that might be the
-# only part unmodified of a slightly edited and not otherwise reported
-# copyright violation. You can disable this feature to try to increase the
-# number of results.
-copyright_economize_query = True
-
 # ############# HTTP SETTINGS ##############
-# Use a persistent http connection. An http connection has to be established
-# only once per site object, making stuff a whole lot faster. Do NOT EVER
-# use this if you share Site objects across threads without proper locking.
-#
-# DISABLED FUNCTION. Setting this variable will not have any effect.
-persistent_http = False
-
 # Default socket timeout in seconds.
 # DO NOT set to None to disable timeouts. Otherwise this may freeze your
 # script.
 # You may assign either a tuple of two int or float values for connection and
-# read timeout, or a single value for both in a tuple (since requests 2.4.0).
+# read timeout, or a single value for both in a tuple.
 socket_timeout = (6.05, 45)
 
 
 # ############# COSMETIC CHANGES SETTINGS ##############
 # The bot can make some additional changes to each page it edits, e.g. fix
-# whitespace or positioning of interwiki and category links.
+# whitespace or positioning category links.
 
 # This is an experimental feature; handle with care and consider re-checking
 # each bot edit if enabling this!
@@ -809,7 +697,7 @@ cosmetic_changes_mylang_only = True
 # (if cosmetic_changes_mylang_only is set)
 # Please set your dictionary by adding such lines to your user-config.py:
 # cosmetic_changes_enable['wikipedia'] = ('de', 'en', 'fr')
-cosmetic_changes_enable = {}
+cosmetic_changes_enable = {}  # type: Dict[str, Tuple[str, ...]]
 
 # The dictionary cosmetic_changes_disable should contain a tuple of languages
 # for each site where you wish to disable cosmetic changes. You may use it with
@@ -817,7 +705,7 @@ cosmetic_changes_enable = {}
 # language. This also overrides the settings in the cosmetic_changes_enable
 # dictionary. Please set your dict by adding such lines to your user-config.py:
 # cosmetic_changes_disable['wikipedia'] = ('de', 'en', 'fr')
-cosmetic_changes_disable = {}
+cosmetic_changes_disable = {}  # type: Dict[str, Tuple[str, ...]]
 
 # cosmetic_changes_deny_script is a list of scripts for which cosmetic changes
 # are disabled. You may add additional scripts by appending script names in
@@ -839,7 +727,7 @@ cosmetic_changes_deny_script = ['category_redirect', 'cosmetic_changes',
 #
 # to replace all occurrences of 'Hoofdpagina' with 'Veurblaad' when writing to
 # liwiki. Note that this does not take the origin wiki into account.
-replicate_replace = {}
+replicate_replace = {}  # type: Dict[str, Dict[str, str]]
 
 # ############# FURTHER SETTINGS ##############
 
@@ -849,7 +737,7 @@ replicate_replace = {}
 # on the wiki server. Allows simulation runs of bots to be carried out without
 # changing any page on the server side. Use this setting to add more actions
 # in user-config.py for wikis with extra write actions.
-actions_to_block = []
+actions_to_block = []  # type: List[str]
 
 # Set simulate to True or use -simulate option to block all actions given
 # above.
@@ -875,16 +763,10 @@ pickle_protocol = 2
 # End of configuration section
 # ============================
 
-# ############# OBSOLETE SETTINGS #############
-# This section contains configuration options that are no longer in use.
-# They are kept here to prevent warnings about undefined parameters.
-
-# No obsolete settings currently
-
 # #############################################
 
 
-def makepath(path, create=True):
+def makepath(path: str, create: bool = True):
     """Return a normalized absolute version of the path argument.
 
     If the given path already exists in the filesystem or create is False
@@ -896,10 +778,8 @@ def makepath(path, create=True):
     from holger@trillke.net 2002/03/18
 
     @param path: path in the filesystem
-    @type path: str
     @param create: create the directory if it is True. Otherwise do not change
         the filesystem. Default is True.
-    @type create: bool
     """
     dpath = os.path.normpath(os.path.dirname(path))
     if create and not os.path.exists(dpath):
@@ -973,6 +853,7 @@ def _detect_win32_editor():
                     break
             else:
                 return editor
+    return None
 
 
 # System-level and User-level changes.
@@ -986,26 +867,21 @@ _public_globals = {
 # we can detect modified config items easily.
 _exec_globals = copy.deepcopy(_public_globals)
 
-# Get the user files
-if __no_user_config:
-    if __no_user_config != '2':
-        warning('Skipping loading of user-config.py.')
-else:
-    _filename = os.path.join(base_dir, 'user-config.py')
-    if os.path.exists(_filename):
-        _filestatus = os.stat(_filename)
-        _filemode = _filestatus[0]
-        _fileuid = _filestatus[4]
-        if OSWIN32 or _fileuid in [os.getuid(), 0]:
-            if OSWIN32 or _filemode & 0o02 == 0:
-                with open(_filename, 'rb') as f:
-                    exec(compile(f.read(), _filename, 'exec'), _exec_globals)
-            else:
-                warning("Skipped '%(fn)s': writeable by others."
-                        % {'fn': _filename})
-        else:
-            warning("Skipped '%(fn)s': owned by someone else."
-                    % {'fn': _filename})
+# Always try to get the user files
+_filename = os.path.join(base_dir, 'user-config.py')
+if os.path.exists(_filename):
+    _filestatus = os.stat(_filename)
+    _filemode = _filestatus[0]
+    _fileuid = _filestatus[4]
+    if not OSWIN32 and _fileuid not in [os.getuid(), 0]:
+        warning('Skipped {fn!r}: owned by someone else.'.format(fn=_filename))
+    elif OSWIN32 or _filemode & 0o02 == 0:
+        with open(_filename, 'rb') as f:
+            exec(compile(f.read(), _filename, 'exec'), _exec_globals)
+    else:
+        warning('Skipped {fn!r}: writeable by others.'.format(fn=_filename))
+elif __no_user_config and __no_user_config != '2':
+    warning('user-config.py cannot be loaded.')
 
 
 class _DifferentTypeError(UserWarning, TypeError):
@@ -1013,7 +889,7 @@ class _DifferentTypeError(UserWarning, TypeError):
     """An error when the required type doesn't match the actual type."""
 
     def __init__(self, name, actual_type, allowed_types):
-        super(_DifferentTypeError, self).__init__(
+        super().__init__(
             'Configuration variable "{0}" is defined as "{1.__name__}" in '
             'your user-config.py but expected "{2}".'
             .format(name, actual_type, '", "'.join(t.__name__
@@ -1025,18 +901,16 @@ def _assert_default_type(name, value, default_value):
     if (value is None or default_value is None
             or isinstance(value, type(default_value))):
         return value
-    elif isinstance(value, int) and isinstance(default_value, float):
+    if isinstance(value, int) and isinstance(default_value, float):
         return float(value)
-    else:
-        raise _DifferentTypeError(name, type(value), [type(default_value)])
+    raise _DifferentTypeError(name, type(value), [type(default_value)])
 
 
 def _assert_types(name, value, types):
     """Return the value if it's one of the types."""
     if isinstance(value, types):
         return value
-    else:
-        raise _DifferentTypeError(name, type(value), types)
+    raise _DifferentTypeError(name, type(value), types)
 
 
 DEPRECATED_VARIABLE = (
@@ -1064,11 +938,11 @@ def _check_user_config_types(user_config, default_values, skipped):
                 warn('\n' + fill(DEPRECATED_VARIABLE.format(name)),
                      _ConfigurationDeprecationWarning)
             elif name not in _future_variables:
-                warn('\n'
-                     + fill('Configuration variable "{0}" is defined in your '
-                            'user-config.py but unknown. It can be a '
-                            'misspelled one or a variable that is no longer '
-                            'supported.'.format(name)), UserWarning)
+                warn('\n' + fill('Configuration variable "{0}" is defined in '
+                                 'your user-config.py but unknown. It can be '
+                                 'a misspelled one or a variable that is no '
+                                 'longer supported.'.format(name)),
+                     UserWarning)
 
 
 _check_user_config_types(_exec_globals, _public_globals, _imports)
@@ -1119,7 +993,7 @@ if OSWIN32 and editor:
 if userinterface_lang is None:
     userinterface_lang = os.getenv('PYWIKIBOT_USERINTERFACE_LANG') \
         or getdefaultlocale()[0]
-    if userinterface_lang in [None, 'C']:
+    if userinterface_lang is None or userinterface_lang == 'C':
         userinterface_lang = 'en'
     else:
         userinterface_lang = userinterface_lang.split('_')[0]
@@ -1131,13 +1005,6 @@ if family == 'wikipedia' and mylang == 'language':
                 "Defaulting to family='wikipedia' and mylang='test'.")
     mylang = 'test'
 
-# Fix up socket_timeout
-# Older requests library expect a single value whereas newer versions also
-# accept a tuple (connect timeout, read timeout).
-if (isinstance(socket_timeout, tuple)
-        and StrictVersion(requests_version) < StrictVersion('2.4.0')):
-    socket_timeout = max(socket_timeout)
-
 # SECURITY WARNINGS
 if (not ignore_file_security_warnings
         and private_files_permission & (stat.S_IRWXG | stat.S_IRWXO) != 0):
@@ -1149,6 +1016,9 @@ if (not ignore_file_security_warnings
           " permission or set 'ignore_file_security_warnings' to true.")
     sys.exit(1)
 
+# Setup custom family files
+for file_path in user_families_paths:
+    register_families_folder(file_path)
 #
 # When called as main program, list all configuration variables
 #
