@@ -1,5 +1,4 @@
 #!/usr/bin/python
-# -*- coding: utf-8 -*-
 """
 This module can do slight modifications to tidy a wiki page's source code.
 
@@ -52,12 +51,14 @@ or by adding a list to the given one:
                                      'your_script_name_2']
 """
 #
-# (C) Pywikibot team, 2006-2020
+# (C) Pywikibot team, 2006-2021
 #
 # Distributed under the terms of the MIT license.
 #
 import re
 
+from typing import Optional
+
 import pywikibot
 
 from pywikibot.page import url2unicode
@@ -66,7 +67,8 @@ from pywikibot.textlib import (
     _get_regexes, _MultiTemplateMatchBuilder, FILE_LINK_REGEX
 )
 from pywikibot.tools import (
-    deprecated, deprecated_args, first_lower, first_upper
+    deprecated, deprecated_args, first_lower, first_upper,
+    issue_deprecation_warning,
 )
 
 try:
@@ -74,9 +76,9 @@ try:
 except ImportError:
     stdnum_isbn = None
 
+
 # Subpage templates. Must be in lower case,
 # whereas subpage itself must be case sensitive
-# This is also used by interwiki.py
 # TODO: Maybe move it to family file and implement global instances
 moved_links = {
     'ar': (['documentation', 'template documentation', 'شرح', 'توثيق'],
@@ -202,20 +204,47 @@ class CosmeticChangesToolkit:
 
     """Cosmetic changes toolkit."""
 
-    @deprecated_args(debug='show_diff', redirect=None, diff='show_diff')
-    def __init__(self, site, show_diff=False, namespace=None, pageTitle=None,
-                 ignore=CANCEL_ALL):
-        """Initializer."""
-        self.site = site
+    @deprecated_args(redirect=None, diff='show_diff', site='page')
+    def __init__(self, page, *,
+                 show_diff: bool = False,
+                 namespace: Optional[int] = None,
+                 pageTitle: Optional[str] = None,
+                 ignore: int = CANCEL_ALL):
+        """Initializer.
+
+        @param page: the Page object containing the text to be modified
+        @type page: pywikibot.Page
+        @param show_diff: show difference after replacements (default: False)
+        @param namespace: DEPRECATED namespace parameter
+        @param pageTitle: DEPRECATED page title parameter
+        @param ignore: ignores if an error occurred and either skips the page
+            or only that method. It can be set one of the CANCEL constants
+        """
+        if isinstance(page, pywikibot.BaseSite):
+            self.site = page
+            self.title = pageTitle
+            try:
+                self.namespace = self.site.namespaces.resolve(namespace).pop(0)
+            except (KeyError, TypeError, IndexError):
+                raise ValueError('{} needs a valid namespace'
+                                 .format(self.__class__.__name__))
+            issue_deprecation_warning(
+                'site parameter of CosmeticChangesToolkit',
+                'a pywikibot.Page object as first parameter',
+                warning_class=FutureWarning,
+                since='20201102')
+        else:
+            if namespace is not None or pageTitle is not None:
+                raise TypeError(
+                    "'namespace' and 'pageTitle' arguments are invalid with "
+                    'a given Page object')
+            self.site = page.site
+            self.title = page.title()
+            self.namespace = page.namespace()
+
         self.show_diff = show_diff
-        try:
-            self.namespace = self.site.namespaces.resolve(namespace).pop(0)
-        except (KeyError, TypeError, IndexError):
-            raise ValueError('{0} needs a valid namespace'
-                             .format(self.__class__.__name__))
         self.template = (self.namespace == 10)
         self.talkpage = self.namespace >= 0 and self.namespace % 2 == 1
-        self.title = pageTitle
         self.ignore = ignore
 
         self.common_methods = [
@@ -257,11 +286,12 @@ class CosmeticChangesToolkit:
         self.show_diff = bool(value)
 
     @classmethod
+    @deprecated('CosmeticChangesToolkit with pywikibot.Page object',
+                future_warning=True, since='20200415')
     @deprecated_args(diff='show_diff')
     def from_page(cls, page, show_diff=False, ignore=CANCEL_ALL):
         """Create toolkit based on the page."""
-        return cls(page.site, show_diff=show_diff, namespace=page.namespace(),
-                   pageTitle=page.title(), ignore=ignore)
+        return cls(page, show_diff=show_diff, ignore=ignore)
 
     def safe_execute(self, method, text):
         """Execute the method and catch exceptions if enabled."""
@@ -293,8 +323,7 @@ class CosmeticChangesToolkit:
                                   .format(self.title))
                 pywikibot.exception(e)
                 return False
-            else:
-                raise
+            raise
         else:
             if self.show_diff:
                 pywikibot.showDiff(text, new_text)
@@ -376,7 +405,7 @@ class CosmeticChangesToolkit:
 
     def translateAndCapitalizeNamespaces(self, text):
         """Use localized namespace names."""
-        # arz uses english stylish codes
+        # arz uses English stylish codes
         if self.site.sitename == 'wikipedia:arz':
             return text
         # wiki links aren't parsed here.
@@ -389,8 +418,7 @@ class CosmeticChangesToolkit:
             # a clone is needed. Won't change the namespace dict
             namespaces = list(namespace)
             if namespace == 6 and self.site.family.name == 'wikipedia':
-                if self.site.code in ('en', 'fr') \
-                        and self.site.mw_version >= '1.14':
+                if self.site.code in ('en', 'fr'):
                     # do not change "Image" on en-wiki and fr-wiki
                     assert 'Image' in namespaces
                     namespaces.remove('Image')
@@ -438,7 +466,7 @@ class CosmeticChangesToolkit:
     def translateMagicWords(self, text):
         """Use localized magic words."""
         # not wanted at ru
-        # arz uses english stylish codes
+        # arz uses English stylish codes
         # no need to run on English wikis
         if self.site.code in ['arz', 'en', 'ru']:
             return text
@@ -473,14 +501,14 @@ class CosmeticChangesToolkit:
                 split[0], '|'.join(cache.get(x.strip(), x) for x in split[1:]))
 
         cache = {}
-        exceptions = ['nowiki', 'comment', 'pre', 'source']
+        exceptions = ['comment', 'nowiki', 'pre', 'syntaxhighlight']
         regex = re.compile(
             FILE_LINK_REGEX % '|'.join(self.site.namespaces[6]),
             flags=re.X)
         return textlib.replaceExcept(
             text, regex, replace_magicword, exceptions)
 
-    def cleanUpLinks(self, text):
+    def cleanUpLinks(self, text: str) -> str:
         """Tidy up wikilinks found in a string.
 
         This function will:
@@ -497,9 +525,7 @@ class CosmeticChangesToolkit:
         * Capitalize the article title of the link, if appropriate
 
         @param text: string to perform the clean-up on
-        @type text: str
         @return: text with tidied wikilinks
-        @rtype: str
         """
         # helper function which works on one link and either returns it
         # unmodified, or returns a replacement.
@@ -549,7 +575,7 @@ class CosmeticChangesToolkit:
                 titleWithSection = titleWithSection.rstrip()
                 hadTrailingSpaces = len(titleWithSection) != titleLength
 
-            # Convert URL-encoded characters to unicode
+            # Convert URL-encoded characters to str
             titleWithSection = url2unicode(titleWithSection,
                                            encodings=self.site)
 
@@ -637,7 +663,7 @@ class CosmeticChangesToolkit:
         return text
 
     def resolveHtmlEntities(self, text):
-        """Replace HTML entities with unicode."""
+        """Replace HTML entities with string."""
         ignore = [
             38,     # Ampersand (&amp;)
             39,     # Single quotation mark (&quot;) per T26093
@@ -661,7 +687,7 @@ class CosmeticChangesToolkit:
         # TODO: T254350 - what other extension tags should be avoided?
         # (graph, math, score, timeline, etc.)
         text = pywikibot.html2unicode(
-            text, ignore=ignore, exceptions=['comment', 'source'])
+            text, ignore=ignore, exceptions=['comment', 'syntaxhighlight'])
         return text
 
     def removeEmptySections(self, text):
@@ -710,8 +736,8 @@ class CosmeticChangesToolkit:
 
     def removeUselessSpaces(self, text):
         """Cleanup multiple or trailing spaces."""
-        exceptions = ['comment', 'math', 'nowiki', 'pre', 'startspace',
-                      'source', 'table']
+        exceptions = ['comment', 'math', 'nowiki', 'pre', 'syntaxhighlight',
+                      'startspace', 'table']
         if self.site.sitename != 'wikipedia:cs':
             exceptions.append('template')
         text = textlib.replaceExcept(text, r'(?m)[\t ]+( |$)', r'\1',
@@ -758,8 +784,9 @@ class CosmeticChangesToolkit:
         other wikis. If there are any complaints, please file a bug report.
         """
         if not self.template:
-            exceptions = ['comment', 'math', 'nowiki', 'pre', 'source',
-                          'template', 'timeline', self.site.redirectRegex()]
+            exceptions = ['comment', 'math', 'nowiki', 'pre',
+                          'syntaxhighlight', 'template', 'timeline',
+                          self.site.redirect_regex]
             text = textlib.replaceExcept(
                 text,
                 r'(?m)'
@@ -773,8 +800,8 @@ class CosmeticChangesToolkit:
         exceptions = ['comment', 'math', 'nowiki', 'pre']
         builder = _MultiTemplateMatchBuilder(self.site)
 
-        if self.site.family.name in deprecatedTemplates and \
-           self.site.code in deprecatedTemplates[self.site.family.name]:
+        if self.site.family.name in deprecatedTemplates \
+           and self.site.code in deprecatedTemplates[self.site.family.name]:
             for template in deprecatedTemplates[
                     self.site.family.name][self.site.code]:
                 old, new = template
@@ -805,8 +832,8 @@ class CosmeticChangesToolkit:
                 replacement += '|' + match.group('title')
             return replacement + ']]'
 
-        exceptions = ['nowiki', 'comment', 'math', 'pre', 'source',
-                      'startspace']
+        exceptions = ['comment', 'math', 'nowiki', 'pre', 'startspace',
+                      'syntaxhighlight']
         # link to the wiki working on
         # Only use suffixes for article paths
         for suffix in self.site._interwiki_urls(True):
@@ -868,8 +895,8 @@ class CosmeticChangesToolkit:
 
         # Everything case-insensitive (?i)
         # Keep in mind that MediaWiki automatically converts <br> to <br />
-        exceptions = ['nowiki', 'comment', 'math', 'pre', 'source',
-                      'startspace']
+        exceptions = ['comment', 'math', 'nowiki', 'pre', 'startspace',
+                      'syntaxhighlight']
         text = textlib.replaceExcept(text, r'(?i)<(b|strong)>(.*?)</\1>',
                                      r"'''\2'''", exceptions, site=self.site)
         text = textlib.replaceExcept(text, r'(?i)<(i|em)>(.*?)</\1>',
@@ -895,7 +922,7 @@ class CosmeticChangesToolkit:
         """Fix references tags."""
         # See also
         # https://en.wikipedia.org/wiki/User:AnomieBOT/source/tasks/OrphanReferenceFixer.pm
-        exceptions = ['nowiki', 'comment', 'math', 'pre', 'source',
+        exceptions = ['comment', 'math', 'nowiki', 'pre', 'syntaxhighlight',
                       'startspace']
 
         # it should be name = " or name=" NOT name   ="
@@ -911,8 +938,8 @@ class CosmeticChangesToolkit:
 
     def fixStyle(self, text):
         """Convert prettytable to wikitable class."""
-        exceptions = ['nowiki', 'comment', 'math', 'pre', 'source',
-                      'startspace']
+        exceptions = ['comment', 'math', 'nowiki', 'pre', 'startspace',
+                      'syntaxhighlight']
         if self.site.code in ('de', 'en'):
             text = textlib.replaceExcept(text,
                                          r'(class="[^"]*)prettytable([^"]*")',
@@ -921,9 +948,8 @@ class CosmeticChangesToolkit:
 
     def fixTypo(self, text):
         """Fix units."""
-        exceptions = ['nowiki', 'comment', 'math', 'pre', 'source',
-                      'startspace', 'gallery', 'hyperlink', 'interwiki',
-                      'link']
+        exceptions = ['comment', 'gallery', 'hyperlink', 'interwiki', 'link',
+                      'nowiki', 'math', 'pre', 'startspace', 'syntaxhighlight']
         # change <number> ccm -> <number> cm³
         text = textlib.replaceExcept(text, r'(\d)\s*(?:&nbsp;)?ccm',
                                      r'\1&nbsp;cm³', exceptions,
@@ -944,11 +970,13 @@ class CosmeticChangesToolkit:
         """Fix arabic and persian letters."""
         if self.site.code not in ['ckb', 'fa']:
             return text
+
         exceptions = [
-            'gallery',
             'file',
+            'gallery',
             'hyperlink',
             'interwiki',
+            'inputbox',
             # FIXME: but changes letters inside wikilinks
             # 'link',
             'math',
@@ -956,9 +984,8 @@ class CosmeticChangesToolkit:
             'template',
             'timeline',
             'ref',
-            'source',
             'startspace',
-            'inputbox',
+            'syntaxhighlight',
         ]
 
         digits = textlib.NON_LATIN_DIGITS
@@ -1000,7 +1027,7 @@ class CosmeticChangesToolkit:
 
         # section headers to {{int:}} versions
         exceptions = ['comment', 'includeonly', 'math', 'noinclude', 'nowiki',
-                      'pre', 'source', 'ref', 'timeline']
+                      'pre', 'syntaxhighlight', 'ref', 'timeline']
         text = textlib.replaceExcept(text,
                                      r'([\r\n]|^)\=\= *Summary *\=\=',
                                      r'\1== {{int:filedesc}} ==',
@@ -1053,5 +1080,4 @@ class CosmeticChangesToolkit:
 
     def fix_ISBN(self, text):
         """Hyphenate ISBN numbers."""
-        return _reformat_ISBNs(
-            text, strict=False if self.ignore == CANCEL_MATCH else True)
+        return _reformat_ISBNs(text, strict=self.ignore != CANCEL_MATCH)
