@@ -1,7 +1,6 @@
-# -*- coding: utf-8 -*-
 """Diff module."""
 #
-# (C) Pywikibot team, 2014-2020
+# (C) Pywikibot team, 2014-2021
 #
 # Distributed under the terms of the MIT license.
 #
@@ -159,8 +158,7 @@ class Hunk:
                 colored_line = color_format('{color}{0}{default}',
                                             line, color=self.colors[color])
                 return colored_line
-            else:
-                return line
+            return line
 
         colored_line = ''
         color_closed = True
@@ -388,6 +386,7 @@ class PatchManager:
             for pending in range(start, end, step):
                 if super_hunks[pending].reviewed == Hunk.PENDING:
                     return pending
+            return None
 
         # TODO: Missing commands (compared to git --patch): edit and search
         help_msg = {'y': 'accept this hunk',
@@ -579,7 +578,7 @@ def cherry_pick(oldtext, newtext, n=0, by_letter=False):
     return text
 
 
-def html_comparator(compare_string):
+def html_comparator(compare_string: str) -> dict:
     """List of added and deleted contexts from 'action=compare' html string.
 
     This function is useful when combineds with site.py's "compare" method.
@@ -587,9 +586,7 @@ def html_comparator(compare_string):
     Here we use BeautifulSoup to get the un-HTML-ify the context of changes.
     Finally we present the added and deleted contexts.
     @param compare_string: HTML string from mediawiki API
-    @type compare_string: str
     @return: deleted and added list of contexts
-    @rtype: dict
     """
     from bs4 import BeautifulSoup
 
