@@ -1,5 +1,4 @@
 #!/usr/bin/python
-# -*- coding: utf-8 -*-
 """Text editor class for your favourite editor."""
 #
 # (C) Pywikibot team, 2004-2020
@@ -18,7 +17,6 @@ from typing import Optional
 import pywikibot
 
 from pywikibot import config
-from pywikibot.tools import deprecated
 
 try:
     from pywikibot.userinterfaces import gui
@@ -71,13 +69,8 @@ class TextEditor:
         return ' '.join("'{0}'".format(part) if ' ' in part else part
                         for part in command)
 
-    @deprecated('_command (should not be used from the outside)',
-                since='20150111', future_warning=True)
-    def command(self, tempFilename, text, jumpIndex=None):
-        """Return editor selected in user-config.py."""
-        return TextEditor._concat(self._command(tempFilename, text, jumpIndex))
-
-    def edit(self, text: str, jumpIndex=None, highlight=None) -> Optional[str]:
+    def edit(self, text: str, jumpIndex: Optional[int] = None,
+             highlight: Optional[str] = None) -> Optional[str]:
         """
         Call the editor and thus allows the user to change the text.
 
@@ -85,9 +78,7 @@ class TextEditor:
 
         @param text: the text to be edited
         @param jumpIndex: position at which to put the caret
-        @type jumpIndex: int
         @param highlight: each occurrence of this substring will be highlighted
-        @type highlight: str
         @return: the modified text, or None if the user didn't save the text
             file in his text editor
         """
