@@ -1,11 +1,9 @@
-# -*- coding: utf-8 -*-
 """
 Exception and warning classes used throughout the framework.
 
 Error: Base class, all exceptions should the subclass of this class.
 
   - NoUsername: Username is not in user-config.py, or it is invalid.
-  - UserBlocked: Username or IP has been blocked
   - AutoblockUser: requested action on a virtual autoblock user not valid
   - UserRightsError: insufficient rights for requested action
   - BadTitle: Server responded with BadTitle
@@ -87,6 +85,10 @@ UserWarning: warnings targeted at users
 #
 # Distributed under the terms of the MIT license.
 #
+import re
+
+from typing import Optional, Union
+
 from pywikibot.tools import _NotImplementedWarning
 
 
@@ -133,12 +135,11 @@ class PageRelatedError(Error):
     Page, and when a generic message can be written once for all.
     """
 
-    # Preformatted UNICODE message where the page title will be inserted
+    # Preformatted message where the page title will be inserted.
     # Override this in subclasses.
-    # 'Oh noes! Page %s is too funky, we should not delete it ;('
-    message = None
+    message = ''
 
-    def __init__(self, page, message=None):
+    def __init__(self, page, message: Optional[str] = None):
         """
         Initializer.
 
@@ -155,14 +156,11 @@ class PageRelatedError(Error):
         self.title = page.title(as_link=True)
         self.site = page.site
 
-        if '%(' in self.message and ')s' in self.message:
-            super().__init__(self.message % self.__dict__)
+        if re.search(r'%\(\w+\)s', self.message):
+            values = self.__dict__
         else:
-            super().__init__(self.message % page)
-
-    def getPage(self):
-        """Return the page related to the exception."""
-        return self.page
+            values = page
+        super().__init__(self.message % values)
 
 
 class PageSaveRelatedError(PageRelatedError):
@@ -187,11 +185,10 @@ class OtherPageSaveError(PageSaveRelatedError):
 
     message = 'Edit to page %(title)s failed:\n%(reason)s'
 
-    def __init__(self, page, reason):
+    def __init__(self, page, reason: Union[str, Exception]):
         """Initializer.
 
         @param reason: Details of the problem
-        @type reason: Exception or basestring
         """
         self.reason = reason
         super().__init__(page)
@@ -244,13 +241,12 @@ class InconsistentTitleReceived(PageLoadRelatedError):
 
     """Page receives a title inconsistent with query."""
 
-    def __init__(self, page, actual):
+    def __init__(self, page, actual: str):
         """Initializer.
 
         @param page: Page that caused the exception
         @type page: Page object
         @param actual: title obtained by query
-        @type reason: basestring
 
         """
         self.message = "Query on %s returned data on '{0}'".format(actual)
@@ -335,7 +331,7 @@ class InterwikiRedirectPage(PageRelatedError):
         """Initializer.
 
         @param target_page: Target page of the redirect.
-        @type reason: Page
+        @type target_page: Page
         """
         self.target_page = target_page
         self.target_site = target_page.site
@@ -471,16 +467,6 @@ class BadTitle(Error):
     pass
 
 
-# UserBlocked exceptions should in general not be caught. If the bot has
-# been blocked, the bot operator should address the reason for the block
-# before continuing.
-class UserBlocked(Error):
-
-    """Your username or IP has been blocked."""
-
-    pass
-
-
 class CaptchaError(Error):
 
     """Captcha is asked and config.solve_captcha == False."""
