@@ -1,7 +1,6 @@
-# -*- coding: utf-8 -*-
 """Family module for Wikipedia."""
 #
-# (C) Pywikibot team, 2004-2020
+# (C) Pywikibot team, 2004-2021
 #
 # Distributed under the terms of the MIT license.
 #
@@ -18,7 +17,8 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
     closed_wikis = [
         # See:
         # https://noc.wikimedia.org/conf/highlight.php?file=dblists/closed.dblist
-        'aa', 'cho', 'ho', 'hz', 'ii', 'kj', 'kr', 'mh', 'mus', 'ng', 'ten',
+        'aa', 'cho', 'ho', 'hz', 'ii', 'kj', 'kr', 'lrc', 'mh', 'mus', 'ng',
+        'ten',
     ]
 
     removed_wikis = [
@@ -29,35 +29,35 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
 
     languages_by_size = [
         'en', 'ceb', 'sv', 'de', 'fr', 'nl', 'ru', 'it', 'es', 'pl', 'war',
-        'vi', 'ja', 'zh', 'arz', 'ar', 'pt', 'uk', 'fa', 'ca', 'sr', 'id',
-        'no', 'ko', 'fi', 'hu', 'cs', 'sh', 'ro', 'zh-min-nan', 'eu', 'tr',
-        'ms', 'eo', 'ce', 'hy', 'he', 'bg', 'da', 'azb', 'sk', 'kk', 'min',
-        'hr', 'et', 'lt', 'be', 'el', 'az', 'simple', 'sl', 'gl', 'ur', 'nn',
-        'hi', 'ka', 'th', 'uz', 'la', 'cy', 'ta', 'tt', 'vo', 'mk', 'ast',
-        'lv', 'tg', 'zh-yue', 'bn', 'mg', 'af', 'oc', 'bs', 'sq', 'ky', 'nds',
-        'new', 'be-tarask', 'ml', 'te', 'tl', 'br', 'pms', 'su', 'mr', 'sw',
-        'ht', 'lb', 'sco', 'vec', 'jv', 'pnb', 'ga', 'ba', 'szl', 'is', 'my',
-        'fy', 'cv', 'lmo', 'wuu', 'an', 'pa', 'ne', 'yo', 'bar', 'ku', 'diq',
-        'io', 'gu', 'als', 'ckb', 'kn', 'scn', 'bpy', 'qu', 'ia', 'mn',
-        'bat-smg', 'or', 'nv', 'si', 'cdo', 'ilo', 'gd', 'am', 'yi', 'nap',
-        'bug', 'wa', 'xmf', 'sd', 'hsb', 'mai', 'map-bms', 'fo', 'mzn', 'li',
-        'sah', 'eml', 'os', 'ps', 'sa', 'frr', 'bcl', 'zh-classical', 'ace',
-        'mrj', 'mhr', 'hif', 'hak', 'roa-tara', 'avk', 'pam', 'gor', 'hyw',
-        'km', 'nso', 'rue', 'se', 'crh', 'bh', 'shn', 'vls', 'as', 'mi',
-        'nds-nl', 'nah', 'sc', 'vep', 'gan', 'myv', 'ab', 'glk', 'bo', 'sn',
-        'co', 'so', 'tk', 'fiu-vro', 'lrc', 'csb', 'kv', 'ha', 'ie', 'gv',
-        'udm', 'pcd', 'ay', 'zea', 'kab', 'nrm', 'sat', 'ug', 'lij', 'kw',
-        'lez', 'stq', 'frp', 'lfn', 'zu', 'gn', 'mwl', 'gom', 'rm', 'ban',
-        'mt', 'lo', 'lad', 'koi', 'fur', 'olo', 'dty', 'ang', 'dsb', 'bjn',
-        'ext', 'ln', 'cbk-zam', 'dv', 'ksh', 'gag', 'pfl', 'tyv', 'pag', 'pi',
-        'av', 'awa', 'haw', 'bxr', 'ary', 'xal', 'krc', 'pap', 'za', 'pdc',
-        'kaa', 'rw', 'szy', 'arc', 'to', 'nov', 'jam', 'tpi', 'kbp', 'kbd',
-        'ig', 'na', 'tet', 'inh', 'wo', 'tcy', 'ki', 'jbo', 'atj', 'roa-rup',
-        'bi', 'lbe', 'kg', 'ty', 'lg', 'mdf', 'srn', 'xh', 'gcr', 'ltg', 'fj',
-        'chr', 'ak', 'sm', 'lld', 'kl', 'om', 'got', 'pih', 'cu', 'ny', 'tn',
-        'tw', 'nqo', 'st', 'ts', 'rmy', 'bm', 'mnw', 'chy', 'rn', 'tum', 'ss',
-        'ch', 'iu', 'pnt', 'ady', 'ks', 've', 'ee', 'ik', 'ff', 'sg', 'dz',
-        'ti', 'cr', 'din',
+        'vi', 'ja', 'arz', 'zh', 'ar', 'uk', 'pt', 'fa', 'ca', 'sr', 'id',
+        'no', 'ko', 'fi', 'hu', 'cs', 'sh', 'zh-min-nan', 'ro', 'tr', 'eu',
+        'ce', 'ms', 'eo', 'he', 'hy', 'bg', 'tt', 'da', 'azb', 'sk', 'kk',
+        'min', 'et', 'hr', 'be', 'lt', 'el', 'simple', 'az', 'gl', 'sl', 'ur',
+        'nn', 'ka', 'hi', 'uz', 'th', 'ta', 'la', 'cy', 'vo', 'mk', 'ast',
+        'zh-yue', 'lv', 'bn', 'tg', 'my', 'af', 'mg', 'oc', 'bs', 'sq', 'nds',
+        'ky', 'be-tarask', 'ml', 'new', 'te', 'mr', 'br', 'vec', 'pms', 'jv',
+        'ht', 'pnb', 'sw', 'su', 'lb', 'tl', 'ba', 'ga', 'szl', 'is', 'cv',
+        'lmo', 'fy', 'wuu', 'sco', 'diq', 'an', 'ku', 'pa', 'yo', 'ne', 'bar',
+        'io', 'gu', 'ckb', 'als', 'kn', 'scn', 'bpy', 'ia', 'qu', 'mn',
+        'bat-smg', 'si', 'nv', 'xmf', 'or', 'cdo', 'ilo', 'gd', 'yi', 'am',
+        'nap', 'bug', 'sd', 'hsb', 'mai', 'wa', 'fo', 'map-bms', 'mzn', 'li',
+        'sah', 'os', 'frr', 'avk', 'eml', 'ace', 'ps', 'sa', 'gor', 'bcl',
+        'zh-classical', 'mrj', 'mhr', 'hif', 'lij', 'hak', 'roa-tara', 'crh',
+        'pam', 'hyw', 'ha', 'nso', 'as', 'km', 'shn', 'rue', 'se', 'zu', 'bh',
+        'vls', 'nds-nl', 'mi', 'nah', 'sc', 'sn', 'vep', 'ban', 'gan', 'myv',
+        'glk', 'ab', 'kab', 'so', 'bo', 'tk', 'co', 'sat', 'fiu-vro', 'ie',
+        'kv', 'csb', 'pcd', 'gv', 'udm', 'ug', 'ay', 'zea', 'frp', 'nrm', 'kw',
+        'lez', 'gn', 'lfn', 'stq', 'mwl', 'mt', 'rm', 'lo', 'lad', 'olo',
+        'gom', 'ary', 'koi', 'skr', 'fur', 'bjn', 'ang', 'dty', 'dsb', 'ext',
+        'ln', 'tyv', 'cbk-zam', 'dv', 'ksh', 'gag', 'pfl', 'smn', 'pi', 'pag',
+        'av', 'awa', 'haw', 'mni', 'bxr', 'xal', 'ig', 'pap', 'krc', 'rw',
+        'za', 'pdc', 'kaa', 'szy', 'arc', 'to', 'inh', 'nov', 'kbp', 'jam',
+        'tpi', 'wo', 'kbd', 'na', 'atj', 'ki', 'tet', 'tcy', 'ak', 'lld', 'lg',
+        'jbo', 'roa-rup', 'bi', 'lbe', 'kg', 'ty', 'mdf', 'xh', 'fj', 'srn',
+        'om', 'gcr', 'ltg', 'sm', 'chr', 'nqo', 'nia', 'mnw', 'got', 'pih',
+        'st', 'kl', 'cu', 'ny', 'tw', 'mad', 'tn', 'ts', 'bm', 'rmy', 'chy',
+        'rn', 'tum', 'ss', 'ch', 'iu', 'ks', 'pnt', 'ady', 've', 'alt', 'ee',
+        'ik', 'ff', 'sg', 'din', 'dz', 'ti', 'cr', 'tay', 'trv',
     ]
 
     # Sites we want to edit but not count as real languages
@@ -68,6 +68,7 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
     category_redirect_templates = {
         '_default': (),
         'ar': ('تحويل تصنيف',),
+        'ary': ('Category redirect',),
         'arz': ('تحويل تصنيف',),
         'bn': ('বিষয়শ্রেণী পুনর্নির্দেশ',),
         'bs': ('Category redirect',),
@@ -110,18 +111,6 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
         'zh-yue': ('分類彈去',),
     }
 
-    # families that redirect their interlanguage links here.
-    interwiki_forwarded_from = [
-        'commons',
-        'incubator',
-        'mediawiki',
-        'meta',
-        'outreach',
-        'species',
-        'test',
-        'wikimania'
-    ]
-
     # Global bot allowed languages on
     # https://meta.wikimedia.org/wiki/BPI#Current_implementation
     # & https://meta.wikimedia.org/wiki/Special:WikiSets/2
@@ -153,16 +142,12 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
         'zh-classical', 'zh-min-nan', 'zh-yue', 'zu',
     ]
 
-    # On most Wikipedias page names must start with a capital letter,
-    # but some languages don't use this.
-    nocapitalize = ['jbo']
-
     # Languages that used to be coded in iso-8859-1
     latin1old = {
-        'af', 'bs', 'co', 'cs', 'da', 'de', 'en', 'es', 'es', 'et', 'eu', 'fi',
-        'fr', 'fy', 'ga', 'gl', 'ia', 'id', 'it', 'la', 'lt', 'lv', 'mi', 'mr',
-        'na', 'nds', 'nl', 'no', 'pt', 'simple', 'sl', 'sv', 'sw', 'test',
-        'tt', 'uk', 'vi', 'vo'
+        'af', 'bs', 'co', 'cs', 'da', 'de', 'en', 'es', 'et', 'eu', 'fi', 'fr',
+        'fy', 'ga', 'gl', 'ia', 'id', 'it', 'la', 'lt', 'lv', 'mi', 'mr', 'na',
+        'nds', 'nl', 'no', 'pt', 'simple', 'sl', 'sv', 'sw', 'test', 'tt',
+        'uk', 'vi', 'vo'
     }
 
     # Subpages for documentation.
@@ -170,10 +155,12 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
     # TODO: Remove comments for appropriate pages
     doc_subpages = {
         '_default': (('/doc', ),
-                     ['ar', 'bn', 'cs', 'da', 'en', 'es', 'hr', 'hu', 'id',
+                     ['arz', 'bn', 'cs', 'da', 'en', 'es', 'hr', 'hu', 'id',
                       'ilo', 'ja', 'ms', 'pt', 'ro', 'ru', 'simple', 'sh',
                       'vi', 'zh']
                      ),
+        'ar': ('/شرح', '/doc', ),
+        'ary': ('/توثيق', '/شرح', '/doc', ),
         'bs': ('/dok', ),
         'ca': ('/ús', ),
         'de': ('Doku', '/Meta'),
@@ -200,6 +187,8 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
     # Templates that indicate an edit should be avoided
     edit_restricted_templates = {
         'ar': ('تحرر',),
+        'ary': ('كاتبدل دابا',),
+        'arz': ('بتتطور',),
         'bs': ('Izmjena u toku',),
         'cs': ('Pracuje se',),
         'de': ('Inuse', 'In use', 'In bearbeitung', 'Inbearbeitung',),
@@ -209,6 +198,7 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
         'he': ('בעבודה',),
         'hr': ('Radovi',),
         'sr': ('Радови у току', 'Рут',),
+        'test': ('In use',),
         'ur': ('زیر ترمیم',),
         'zh': ('Inuse',),
     }
@@ -216,24 +206,14 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
     # Archive templates that indicate an edit of non-archive bots
     # should be avoided
     archived_page_templates = {
+        'ar': ('أرشيف نقاش',),
+        'arz': ('صفحة ارشيف',),
         'cs': ('Archiv', 'Archiv Wikipedie', 'Archiv diskuse',
                'Archivace start', 'Posloupnost archivů', 'Rfa-archiv-start',
                'Rfc-archiv-start',),
         'de': ('Archiv',),
     }
 
-    def get_known_families(self, site):
-        """Override the family interwiki prefixes for each site."""
-        # In Swedish Wikipedia 's:' is part of page title not a family
-        # prefix for 'wikisource'.
-        if site.code == 'sv':
-            d = self.known_families.copy()
-            d.pop('s')
-            d['src'] = 'wikisource'
-            return d
-        else:
-            return self.known_families
-
     def encodings(self, code):
         """Return a list of historical encodings for a specific site."""
         # Historic compatibility
