@@ -1,7 +1,6 @@
-# -*- coding: utf-8 -*-
 """Family module for Wikiquote."""
 #
-# (C) Pywikibot team, 2005-2020
+# (C) Pywikibot team, 2005-2021
 #
 # Distributed under the terms of the MIT license.
 #
@@ -17,28 +16,28 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
 
     closed_wikis = [
         # See https://noc.wikimedia.org/conf/highlight.php?file=dblists/closed.dblist  # noqa
-        'als', 'am', 'ang', 'ast', 'bm', 'co', 'cr', 'ga', 'kk',
-        'kr', 'ks', 'kw', 'lb', 'na', 'nds', 'qu', 'simple',
-        'tk', 'tt', 'ug', 'vo', 'za', 'zh-min-nan',
+        'am', 'ang', 'ast', 'bm', 'co', 'cr', 'ga', 'kk', 'kr', 'ks', 'kw',
+        'lb', 'na', 'nds', 'qu', 'simple', 'tk', 'tt', 'ug', 'vo', 'za',
+        'zh-min-nan',
     ]
 
     removed_wikis = [
         # See https://noc.wikimedia.org/conf/highlight.php?file=dblists/deleted.dblist  # noqa
-        'tokipona',
+        'als', 'tokipona',
     ]
 
     languages_by_size = [
-        'en', 'it', 'pl', 'ru', 'cs', 'fa', 'de', 'pt', 'es', 'fr', 'uk', 'he',
-        'sk', 'bs', 'tr', 'ca', 'fi', 'sl', 'az', 'lt', 'eo', 'zh', 'bg', 'hr',
-        'hy', 'ar', 'el', 'su', 'id', 'nn', 'et', 'sv', 'li', 'hu', 'ko', 'nl',
-        'ja', 'la', 'ta', 'sah', 'sr', 'gu', 'gl', 'th', 'ur', 'te', 'be',
-        'cy', 'no', 'ml', 'sq', 'kn', 'ro', 'ku', 'eu', 'uz', 'hi', 'ka', 'da',
-        'sa', 'is', 'vi', 'br', 'mr', 'af', 'wo', 'ky',
+        'en', 'it', 'pl', 'ru', 'cs', 'fa', 'de', 'pt', 'fr', 'es', 'uk', 'et',
+        'he', 'sk', 'tr', 'bs', 'ca', 'fi', 'az', 'sl', 'eo', 'lt', 'zh', 'ar',
+        'bg', 'hr', 'hy', 'el', 'su', 'nn', 'id', 'sv', 'li', 'hu', 'ko', 'nl',
+        'ja', 'ta', 'la', 'sah', 'sr', 'gu', 'gl', 'ur', 'th', 'te', 'be',
+        'cy', 'no', 'ml', 'sq', 'kn', 'ro', 'ku', 'eu', 'uz', 'hi', 'vi', 'ka',
+        'da', 'sa', 'is', 'br', 'mr', 'af', 'wo', 'ky',
     ]
 
     category_redirect_templates = {
         '_default': (),
-        'ar': ('قالب:تحويل تصنيف',),
+        'ar': ('تحويل تصنيف',),
         'en': ('Category redirect',),
         'ro': ('Redirect categorie',),
         'sq': ('Kategori e zhvendosur',),
@@ -49,13 +48,12 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
     # https://meta.wikimedia.org/wiki/BPI#Current_implementation
     # & https://meta.wikimedia.org/wiki/Special:WikiSets/2
     cross_allowed = [
-        'af', 'ar', 'az', 'be', 'bg', 'br', 'bs', 'ca', 'cs',
-        'cy', 'da', 'el', 'eo', 'es', 'et', 'eu', 'fa', 'fi',
-        'fr', 'gl', 'gu', 'he', 'hi', 'hu', 'hy', 'id', 'is',
-        'it', 'ja', 'ka', 'kn', 'ko', 'ku', 'ky', 'la', 'li',
-        'lt', 'ml', 'mr', 'nl', 'nn', 'no', 'pt', 'ro', 'ru',
-        'sa', 'sah', 'sk', 'sl', 'sq', 'sr', 'su', 'sv', 'ta',
-        'te', 'th', 'tr', 'uk', 'ur', 'uz', 'vi', 'wo', 'zh',
+        'af', 'ar', 'az', 'be', 'bg', 'br', 'bs', 'ca', 'cs', 'cy', 'da', 'el',
+        'eo', 'es', 'et', 'eu', 'fa', 'fi', 'fr', 'gl', 'gu', 'he', 'hi', 'hu',
+        'hy', 'id', 'is', 'it', 'ja', 'ka', 'kn', 'ko', 'ku', 'ky', 'la', 'li',
+        'lt', 'ml', 'mr', 'nl', 'nn', 'no', 'pt', 'ro', 'ru', 'sa', 'sah',
+        'sk', 'sl', 'sq', 'sr', 'su', 'sv', 'ta', 'te', 'th', 'tr', 'uk', 'ur',
+        'uz', 'vi', 'wo', 'zh',
     ]
 
     # Subpages for documentation.
@@ -64,6 +62,7 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
         '_default': (('/doc', ),
                      ['en']
                      ),
+        'ar': ('/شرح', '/doc'),
         'sr': ('/док', ),
     }
 
