@@ -1,7 +1,6 @@
-# -*- coding: utf-8 -*-
 """Family module for Wikisource."""
 #
-# (C) Pywikibot team, 2004-2020
+# (C) Pywikibot team, 2004-2021
 #
 # Distributed under the terms of the MIT license.
 #
@@ -27,12 +26,12 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
 
     languages_by_size = [
         'pl', 'en', 'ru', 'de', 'fr', 'zh', 'he', 'it', 'es', 'ar', 'uk', 'cs',
-        'mul', 'pt', 'gu', 'sr', 'fa', 'hu', 'sv', 'ml', 'bn', 'ko', 'sa',
-        'hy', 'sl', 'te', 'el', 'ta', 'ro', 'fi', 'ja', 'th', 'nap', 'vi',
-        'az', 'tr', 'br', 'la', 'ca', 'nl', 'kn', 'hr', 'eo', 'no', 'is',
-        'vec', 'pms', 'et', 'be', 'da', 'id', 'lij', 'mk', 'yi', 'hi', 'mr',
-        'bg', 'as', 'li', 'lt', 'eu', 'or', 'pa', 'gl', 'bs', 'sah', 'sk',
-        'cy', 'zh-min-nan', 'fo',
+        'gu', 'mul', 'pt', 'sr', 'fa', 'hu', 'sv', 'ml', 'ko', 'bn', 'sa',
+        'te', 'sl', 'ta', 'hy', 'el', 'ro', 'fi', 'ja', 'th', 'tr', 'nap',
+        'vi', 'az', 'ca', 'la', 'br', 'nl', 'kn', 'hr', 'no', 'eo', 'is',
+        'vec', 'pms', 'id', 'hi', 'be', 'et', 'lij', 'da', 'mk', 'yi', 'mr',
+        'as', 'bg', 'li', 'lt', 'eu', 'wa', 'or', 'pa', 'cy', 'gl', 'bs',
+        'sah', 'sk', 'zh-min-nan', 'fo',
     ]
 
     # Sites we want to edit but not count as real languages
@@ -40,9 +39,10 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
 
     category_redirect_templates = {
         '_default': (),
-        'ar': ('قالب:تحويل تصنيف',),
+        'ar': ('تحويل تصنيف',),
         'bn': ('বিষয়শ্রেণী পুনর্নির্দেশ',),
         'en': ('Category redirect',),
+        'es': ('Categoría redirigida',),
         'ro': ('Redirect categorie',),
         'zh': ('分類重定向',),
     }
@@ -80,10 +80,10 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
     # https://meta.wikimedia.org/wiki/BPI#Current_implementation
     # https://meta.wikimedia.org/wiki/Special:WikiSets/2
     cross_allowed = [
-        'ar', 'be', 'bg', 'bn', 'br', 'bs', 'ca', 'cy', 'da', 'el', 'et',
-        'eu', 'fa', 'fi', 'fo', 'gl', 'hr', 'hy', 'id', 'it', 'ja', 'kn',
-        'ko', 'ml', 'nap', 'no', 'or', 'pa', 'pl', 'pt', 'ro', 'sa', 'sk',
-        'sr', 'ta', 'te', 'th', 'uk', 'vi', 'zh',
+        'ar', 'be', 'bg', 'bn', 'br', 'bs', 'ca', 'cy', 'da', 'el', 'et', 'eu',
+        'fa', 'fi', 'fo', 'gl', 'hr', 'hy', 'id', 'it', 'ja', 'kn', 'ko', 'ml',
+        'nap', 'no', 'or', 'pa', 'pl', 'pt', 'ro', 'sa', 'sk', 'sr', 'ta',
+        'te', 'th', 'uk', 'vi', 'zh',
     ]
 
     authornamespaces = {
@@ -97,6 +97,7 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
         'da': [102],
         'en': [102],
         'eo': [102],
+        'es': [106],
         'et': [106],
         'fa': [102],
         'fr': [102],
@@ -126,11 +127,11 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
     # TODO: Remove comments for appropriate pages
     doc_subpages = {
         '_default': (('/doc', ),
-                     ['ar', 'as', 'az', 'bn', 'en', 'es',
-                      'et', 'gu', 'hu', 'it', 'ja', 'kn', 'ml',
-                      'mk', 'mr', 'pt', 'ro', 'sa', 'sah', 'ta',
-                      'te', 'th', 'vi']
+                     ['as', 'az', 'bn', 'en', 'es', 'et', 'gu', 'hu', 'it',
+                      'ja', 'kn', 'ml', 'mk', 'mr', 'pt', 'ro', 'sa', 'sah',
+                      'ta', 'te', 'th', 'vi']
                      ),
+        'ar': ('/شرح', '/doc'),
         'be': ('/Дакументацыя', ),
         'bn': ('/নথি', ),
         'br': ('/diellerezh', ),
