@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """Family module for Wikiversity."""
 #
 # (C) Pywikibot team, 2007-2020
@@ -17,8 +16,8 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
     name = 'wikiversity'
 
     languages_by_size = [
-        'de', 'en', 'fr', 'zh', 'ru', 'it', 'cs', 'pt', 'es', 'sv', 'sl', 'fi',
-        'ar', 'el', 'hi', 'ko', 'ja',
+        'de', 'en', 'fr', 'zh', 'it', 'ru', 'cs', 'pt', 'es', 'sl', 'ar', 'sv',
+        'fi', 'el', 'hi', 'ko', 'ja',
     ]
 
     test_codes = ['beta']
@@ -31,7 +30,7 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
 
     category_redirect_templates = {
         '_default': (),
-        'ar': ('قالب:تحويل تصنيف',),
+        'ar': ('تحويل تصنيف',),
         'en': ('Category redirect',),
     }
 
