@@ -1,11 +1,10 @@
-# -*- coding: utf-8 -*-
 """Family module for Wikivoyage."""
 #
 # (C) Pywikibot team, 2012-2020
 #
 # Distributed under the terms of the MIT license.
 #
-# The new wikivoyage family that is hosted at wikimedia
+# The new Wikivoyage family that is hosted at Wikimedia
 from pywikibot import family
 
 
@@ -17,7 +16,7 @@ class Family(family.SubdomainFamily, family.WikimediaFamily):
 
     languages_by_size = [
         'en', 'de', 'pl', 'it', 'fa', 'fr', 'ru', 'zh', 'nl', 'pt', 'es', 'he',
-        'vi', 'fi', 'sv', 'el', 'ro', 'uk', 'bn', 'ps', 'hi', 'ja',
+        'vi', 'fi', 'sv', 'el', 'uk', 'ro', 'bn', 'eo', 'tr', 'ps', 'ja', 'hi',
     ]
 
     category_redirect_templates = {
