@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """File containing all standard fixes."""
 #
 # (C) Pywikibot team, 2008-2020
@@ -128,12 +127,12 @@ fixes = {
             'inside-tags': [
                 'nowiki',
                 'comment',
+                'gallery',          # because of filenames
+                'hyperlink',        # e.g. commas in URLs
                 'math',
-                'pre',           # because of code examples
-                'source',        # because of code examples
-                'startspace',    # because of code examples
-                'hyperlink',     # e.g. commas in URLs
-                'gallery',       # because of filenames
+                'pre',              # because of code examples
+                'startspace',       # because of code examples
+                'syntaxhighlight',  # because of code examples
                 'timeline',
             ],
             'text-contains': [
@@ -220,8 +219,8 @@ fixes = {
                 'comment',
                 'math',
                 'pre',
-                'source',        # because of code examples
-                'startspace',    # because of code examples
+                'startspace',       # because of code examples
+                'syntaxhighlight',  # because of code examples
             ],
             'text-contains': [
                 r'http://.*?object=tx\|',                # regular dash in URL
@@ -266,8 +265,8 @@ fixes = {
                 'comment',
                 'math',
                 'pre',
-                'source',        # because of code examples
-                'startspace',    # because of code examples
+                'startspace',       # because of code examples
+                'syntaxhighlight',  # because of code examples
             ],
         }
     },
@@ -428,145 +427,146 @@ fixes = {
     # python pwb.py replace -fix:correct-ar -start:! -always
 
     'correct-ar': {
-        'regex': True,
+        'regex': False,
         'msg': {
-            'ar': 'تدقيق إملائي',
+            'ar': 'بوت: تدقيق إملائي',
         },
         'replacements': [
-            (r'(\A|\s)إمرأة(\Z|\s)', '\\1امرأة\\2'),
-            (r'(\A|\s)الى(\Z|\s)', '\\1إلى\\2'),
-            (r'(\A|\s)إسم(\Z|\s)', '\\1اسم\\2'),
-            (r'(\A|\s)الأن(\Z|\s)', '\\1الآن\\2'),
-            (r'(\A|\s)اول(\Z|\s)', '\\1أول\\2'),
-            (r'(\A|\s)الة(\Z|\s)', '\\1آلة\\2'),
-            (r'(\A|\s)فى(\Z|\s)', '\\1في\\2'),
-            (r'(\A|\s)اثقل(\Z|\s)', '\\1أثقل\\2'),
-            (r'(\A|\s)إبن(\Z|\s)', '\\1ابن\\2'),
-            (r'(\A|\s)إبنة(\Z|\s)', '\\1ابنة\\2'),
-            (r'(\A|\s)إقتصاد(\Z|\s)', '\\1اقتصاد\\2'),
-            (r'(\A|\s)إجتماع(\Z|\s)', '\\1اجتماع\\2'),
-            (r'(\A|\s)انجيل(\Z|\s)', '\\1إنجيل\\2'),
-            (r'(\A|\s)اجماع(\Z|\s)', '\\1إجماع\\2'),
-            (r'(\A|\s)امريكا(\Z|\s)', '\\1أمريكا\\2'),
-            (r'(\A|\s)اوروبا(\Z|\s)', '\\1أوروبا\\2'),
-            (r'(\A|\s)انجلترا(\Z|\s)', '\\1إنجلترا\\2'),
-            (r'(\A|\s)اكتوبر(\Z|\s)', '\\1أكتوبر\\2'),
-            (r'(\A|\s)اسرائيل(\Z|\s)', '\\1إسرائيل\\2'),
-            (r'(\A|\s)المانيا(\Z|\s)', '\\1ألمانيا\\2'),
-            (r'(\A|\s)ايطاليا(\Z|\s)', '\\1إيطاليا\\2'),
-            (r'(\A|\s)ايران(\Z|\s)', '\\1إيران\\2'),
-            (r'(\A|\s)إستخراج(\Z|\s)', '\\1استخراج\\2'),
-            (r'(\A|\s)إستعمال(\Z|\s)', '\\1استعمال\\2'),
-            (r'(\A|\s)إستبدال(\Z|\s)', '\\1استبدال\\2'),
-            (r'(\A|\s)إشتراك(\Z|\s)', '\\1اشتراك\\2'),
-            (r'(\A|\s)إستعادة(\Z|\s)', '\\1استعادة\\2'),
-            (r'(\A|\s)إستقلال(\Z|\s)', '\\1استقلال\\2'),
-            (r'(\A|\s)إنتقال(\Z|\s)', '\\1انتقال\\2'),
-            (r'(\A|\s)إتحاد(\Z|\s)', '\\1اتحاد\\2'),
-            (r'(\A|\s)املاء(\Z|\s)', '\\1إملاء\\2'),
-            (r'(\A|\s)إستخدام(\Z|\s)', '\\1استخدام\\2'),
-            (r'(\A|\s)أحدى(\Z|\s)', '\\1إحدى\\2'),
-            (r'(\A|\s)لاكن(\Z|\s)', '\\1لكن\\2'),
-            (r'(\A|\s)الاردن(\Z|\s)', '\\1الأردن\\2'),
-            (r'(\A|\s)إثنان(\Z|\s)', '\\1اثنان\\2'),
-            (r'(\A|\s)شيئ(\Z|\s)', '\\1شيء\\2'),
-            (r'(\A|\s)إحتياط(\Z|\s)', '\\1احتياط\\2'),
-            (r'(\A|\s)إقتباس(\Z|\s)', '\\1اقتباس\\2'),
-            (r'(\A|\s)الامارات(\Z|\s)', '\\1الإمارات\\2'),
-            (r'(\A|\s)اكثر(\Z|\s)', '\\1أكثر\\2'),
-            (r'(\A|\s)افضل(\Z|\s)', '\\1أفضل\\2'),
-            (r'(\A|\s)اكبر(\Z|\s)', '\\1أكبر\\2'),
-            (r'(\A|\s)اشهر(\Z|\s)', '\\1أشهر\\2'),
-            (r'(\A|\s)ادارة(\Z|\s)', '\\1إدارة\\2'),
-            (r'(\A|\s)ابناء(\Z|\s)', '\\1أبناء\\2'),
-            (r'(\A|\s)الانصار(\Z|\s)', '\\1 الأنصار\\2'),
-            (r'(\A|\s)اشارة(\Z|\s)', '\\1إشارة\\2'),
-            (r'(\A|\s)إقرأ(\Z|\s)', '\\1اقرأ\\2'),
-            (r'(\A|\s)إمتياز(\Z|\s)', '\\1امتياز\\2'),
-            (r'(\A|\s)ارق(\Z|\s)', '\\1أرق\\2'),
-            (r'(\A|\s)أرثوذوكس(\Z|\s)', '\\1أرثوذكس\\2'),
-            (r'(\A|\s)الأرثوذوكس(\Z|\s)', '\\1الأرثوذكس\\2'),
-            (r'(\A|\s)أرثوذوكسية(\Z|\s)', '\\1أرثوذكسية\\2'),
-            (r'(\A|\s)الأرثوذوكسية(\Z|\s)', '\\1الأرثوذكسية\\2'),
-            (r'(\A|\s)الأرثوذوكسي(\Z|\s)', '\\1الأرثوذكسي\\2'),
-            (r'(\A|\s)ارثوذوكس(\Z|\s)', '\\1أرثوذكس\\2'),
-            (r'(\A|\s)ارثوذوكسي(\Z|\s)', '\\1أرثوذكسي\\2'),
-            (r'(\A|\s)ارثوذوكسية(\Z|\s)', '\\1أرثوذكسية\\2'),
-            (r'(\A|\s)الارثوذوكسية(\Z|\s)', '\\1الأرثوذكسية\\2'),
-            (r'(\A|\s)اللة(\Z|\s)', '\\1الله\\2'),
-            (r'(\A|\s)إختبار(\Z|\s)', '\\1اختبار\\2'),
-            (r'(\A|\s)== روابط خارجية ==(\Z|\s)', '\\1== وصلات خارجية ==\\2'),
-            (r'(\A|\s)==روابط خارجية==(\Z|\s)', '\\1== وصلات خارجية ==\\2'),
-            (r'(\A|\s)ارسال(\Z|\s)', '\\1إرسال\\2'),
-            (r'(\A|\s)إتصالات(\Z|\s)', '\\1اتصالات\\2'),
-            (r'(\A|\s)اسامة(\Z|\s)', '\\1أسامة\\2'),
-            (r'(\A|\s)ابراهيم(\Z|\s)', '\\1إبراهيم\\2'),
-            (r'(\A|\s)اسماعيل(\Z|\s)', '\\1إسماعيل\\2'),
-            (r'(\A|\s)ايوب(\Z|\s)', '\\1أيوب\\2'),
-            (r'(\A|\s)ايمن(\Z|\s)', '\\1أيمن\\2'),
-            (r'(\A|\s)ابو(\Z|\s)', '\\1أبو\\2'),
-            (r'(\A|\s)ابا(\Z|\s)', '\\1أبا\\2'),
-            (r'(\A|\s)اخو(\Z|\s)', '\\1أخو\\2'),
-            (r'(\A|\s)اخا(\Z|\s)', '\\1أخا\\2'),
-            (r'(\A|\s)اخي(\Z|\s)', '\\1أخي\\2'),
-            (r'(\A|\s)احد(\Z|\s)', '\\1أحد\\2'),
-            (r'(\A|\s)اربعاء(\Z|\s)', '\\1أربعاء\\2'),
-            (r'(\A|\s)اهم(\Z|\s)', '\\1أهم\\2'),
-            (r'(\A|\s)اوزبكستان(\Z|\s)', '\\1أوزبكستان\\2'),
-            (r'(\A|\s)اذربيجان(\Z|\s)', '\\1أذربيجان\\2'),
-            (r'(\A|\s)افغانستان(\Z|\s)', '\\1أفغانستان\\2'),
-            (r'(\A|\s)امجد(\Z|\s)', '\\1أمجد\\2'),
-            (r'(\A|\s)اوسط(\Z|\s)', '\\1أوسط\\2'),
-            (r'(\A|\s)اشقر(\Z|\s)', '\\1أشقر\\2'),
-            (r'(\A|\s)انور(\Z|\s)', '\\1أنور\\2'),
-            (r'(\A|\s)اصعب(\Z|\s)', '\\1أصعب\\2'),
-            (r'(\A|\s)اسهل(\Z|\s)', '\\1أسهل\\2'),
-            (r'(\A|\s)اجمل(\Z|\s)', '\\1أجمل\\2'),
-            (r'(\A|\s)اقبح(\Z|\s)', '\\1أقبح\\2'),
-            (r'(\A|\s)اطول(\Z|\s)', '\\1أطول\\2'),
-            (r'(\A|\s)اقصر(\Z|\s)', '\\1أقصر\\2'),
-            (r'(\A|\s)اسمن(\Z|\s)', '\\1أسمن\\2'),
-            (r'(\A|\s)اذكى(\Z|\s)', '\\1أذكى\\2'),
-            (r'(\A|\s)اماني(\Z|\s)', '\\1أماني\\2'),
-            (r'(\A|\s)احلام(\Z|\s)', '\\1أحلام\\2'),
-            (r'(\A|\s)اسماء(\Z|\s)', '\\1أسماء\\2'),
-            (r'(\A|\s)ابطأ(\Z|\s)', '\\1أبطأ\\2'),
-            (r'(\A|\s)اوربا(\Z|\s)', '\\1أوروبا\\2'),
-            (r'(\A|\s)أوربا(\Z|\s)', '\\1أوروبا\\2'),
-            (r'(\A|\s)امريكي(\Z|\s)', '\\1أمريكي\\2'),
-            (r'(\A|\s)امريكية(\Z|\s)', '\\1أمريكية\\2'),
-            (r'(\A|\s)امريكيان(\Z|\s)', '\\1أمريكيان\\2'),
-            (r'(\A|\s)امريكيتان(\Z|\s)', '\\1أمريكيتان\\2'),
-            (r'(\A|\s)امريكيون(\Z|\s)', '\\1أمريكيون\\2'),
-            (r'(\A|\s)امريكيات(\Z|\s)', '\\1أمريكيات\\2'),
-            (r'(\A|\s)الامريكي(\Z|\s)', '\\1الأمريكي\\2'),
-            (r'(\A|\s)الامريكية(\Z|\s)', '\\1الأمريكية\\2'),
-            (r'(\A|\s)الامريكيان(\Z|\s)', '\\1الأمريكيان\\2'),
-            (r'(\A|\s)الامريكيتان(\Z|\s)', '\\1الأمريكيتان\\2'),
-            (r'(\A|\s)الامريكيون(\Z|\s)', '\\1الأمريكيون\\2'),
-            (r'(\A|\s)الامريكيات(\Z|\s)', '\\1الأمريكيات\\2'),
-            (r'(\A|\s)اوروبي(\Z|\s)', '\\1أوروبي\\2'),
-            (r'(\A|\s)اوروبية(\Z|\s)', '\\1أوروبية\\2'),
-            (r'(\A|\s)اوروبيان(\Z|\s)', '\\1أوروبيان\\2'),
-            (r'(\A|\s)اوروبيتان(\Z|\s)', '\\1أوروبيتان\\2'),
-            (r'(\A|\s)اوروبيون(\Z|\s)', '\\1أوروبيون\\2'),
-            (r'(\A|\s)اوروبيات(\Z|\s)', '\\1أوروبيات\\2'),
-            (r'(\A|\s)الاوروبي(\Z|\s)', '\\1الأوروبي\\2'),
-            (r'(\A|\s)الاوروبية(\Z|\s)', '\\1الأوروبية\\2'),
-            (r'(\A|\s)الاوروبيان(\Z|\s)', '\\1الأوروبيان\\2'),
-            (r'(\A|\s)الاوروبيتان(\Z|\s)', '\\1الأوروبيتان\\2'),
-            (r'(\A|\s)الاوروبيون(\Z|\s)', '\\1الأوروبيون\\2'),
-            (r'(\A|\s)الاوروبيات(\Z|\s)', '\\1الأوروبيات\\2'),
-            (r'(\A|\s)اسرائيلي(\Z|\s)', '\\1إسرائيلي\\2'),
-            (r'(\A|\s)اسرائيلية(\Z|\s)', '\\1إسرائيلية\\2'),
-            (r'(\A|\s)اسرائيليان(\Z|\s)', '\\1إسرائيليان\\2'),
-            (r'(\A|\s)اسرائيليتان(\Z|\s)', '\\1إسرائيليتان\\2'),
+            (' إمرأة ', ' امرأة '),
+            (' الى ', ' إلى '),
+            (' إسم ', ' اسم '),
+            (' الأن ', ' الآن '),
+            (' اول ', ' أول '),
+            (' الة ', ' آلة '),
+            (' فى ', ' في '),
+            (' اثقل ', ' أثقل '),
+            (' إبن ', ' ابن '),
+            (' إبنة ', ' ابنة '),
+            (' إقتصاد ', ' اقتصاد '),
+            (' إجتماع ', ' اجتماع '),
+            (' انجيل ', ' إنجيل '),
+            (' اجماع ', ' إجماع '),
+            (' امريكا ', ' أمريكا '),
+            (' اوروبا ', ' أوروبا '),
+            (' انجلترا ', ' إنجلترا '),
+            (' اكتوبر ', ' أكتوبر '),
+            (' اسرائيل ', ' إسرائيل '),
+            (' المانيا ', ' ألمانيا '),
+            (' ايطاليا ', ' إيطاليا '),
+            (' ايران ', ' إيران '),
+            (' إستخراج ', ' استخراج '),
+            (' إستعمال ', ' استعمال '),
+            (' إستبدال ', ' استبدال '),
+            (' إشتراك ', ' اشتراك '),
+            (' إستعادة ', ' استعادة '),
+            (' إستقلال ', ' استقلال '),
+            (' إنتقال ', ' انتقال '),
+            (' إتحاد ', ' اتحاد '),
+            (' الإتحاد ', ' الاتحاد '),
+            (' املاء ', ' إملاء '),
+            (' إستخدام ', ' استخدام '),
+            (' أحدى ', ' إحدى '),
+            (' لاكن ', ' لكن '),
+            (' الاردن ', ' الأردن '),
+            (' إثنان ', ' اثنان '),
+            (' شيئ ', ' شيء '),
+            (' إحتياط ', ' احتياط '),
+            (' إقتباس ', ' اقتباس '),
+            (' الامارات ', ' الإمارات '),
+            (' اكثر ', ' أكثر '),
+            (' افضل ', ' أفضل '),
+            (' اكبر ', ' أكبر '),
+            (' اشهر ', ' أشهر '),
+            (' ادارة ', ' إدارة '),
+            (' ابناء ', ' أبناء '),
+            (' الانصار ', '  الأنصار '),
+            (' اشارة ', ' إشارة '),
+            (' إقرأ ', ' اقرأ '),
+            (' إمتياز ', ' امتياز '),
+            (' ارق ', ' أرق '),
+            (' أرثوذوكس ', ' أرثوذكس '),
+            (' الأرثوذوكس ', ' الأرثوذكس '),
+            (' أرثوذوكسية ', ' أرثوذكسية '),
+            (' الأرثوذوكسية ', ' الأرثوذكسية '),
+            (' الأرثوذوكسي ', ' الأرثوذكسي '),
+            (' ارثوذوكس ', ' أرثوذكس '),
+            (' ارثوذوكسي ', ' أرثوذكسي '),
+            (' ارثوذوكسية ', ' أرثوذكسية '),
+            (' الارثوذوكسية ', ' الأرثوذكسية '),
+            (' اللة ', ' الله '),
+            (' إختبار ', ' اختبار '),
+            (' ارسال ', ' إرسال '),
+            (' إتصالات ', ' اتصالات '),
+            (' اسامة ', ' أسامة '),
+            (' ابراهيم ', ' إبراهيم '),
+            (' اسماعيل ', ' إسماعيل '),
+            (' ايوب ', ' أيوب '),
+            (' ايمن ', ' أيمن '),
+            (' ابو ', ' أبو '),
+            (' ابا ', ' أبا '),
+            (' اخو ', ' أخو '),
+            (' اخا ', ' أخا '),
+            (' اخي ', ' أخي '),
+            (' احد ', ' أحد '),
+            (' اربعاء ', ' أربعاء '),
+            (' اهم ', ' أهم '),
+            (' اوزبكستان ', ' أوزبكستان '),
+            (' اذربيجان ', ' أذربيجان '),
+            (' افغانستان ', ' أفغانستان '),
+            (' امجد ', ' أمجد '),
+            (' اوسط ', ' أوسط '),
+            (' اشقر ', ' أشقر '),
+            (' انور ', ' أنور '),
+            (' اصعب ', ' أصعب '),
+            (' اسهل ', ' أسهل '),
+            (' اجمل ', ' أجمل '),
+            (' اقبح ', ' أقبح '),
+            (' اطول ', ' أطول '),
+            (' اقصر ', ' أقصر '),
+            (' اسمن ', ' أسمن '),
+            (' اذكى ', ' أذكى '),
+            (' اماني ', ' أماني '),
+            (' احلام ', ' أحلام '),
+            (' اسماء ', ' أسماء '),
+            (' ابطأ ', ' أبطأ '),
+            (' اوربا ', ' أوروبا '),
+            (' أوربا ', ' أوروبا '),
+            (' امريكي ', ' أمريكي '),
+            (' امريكية ', ' أمريكية '),
+            (' امريكيان ', ' أمريكيان '),
+            (' امريكيتان ', ' أمريكيتان '),
+            (' امريكيون ', ' أمريكيون '),
+            (' امريكيات ', ' أمريكيات '),
+            (' الامريكي ', ' الأمريكي '),
+            (' الامريكية ', ' الأمريكية '),
+            (' الامريكيان ', ' الأمريكيان '),
+            (' الامريكيتان ', ' الأمريكيتان '),
+            (' الامريكيون ', ' الأمريكيون '),
+            (' الامريكيات ', ' الأمريكيات '),
+            (' اوروبي ', ' أوروبي '),
+            (' اوروبية ', ' أوروبية '),
+            (' اوروبيان ', ' أوروبيان '),
+            (' اوروبيتان ', ' أوروبيتان '),
+            (' اوروبيون ', ' أوروبيون '),
+            (' اوروبيات ', ' أوروبيات '),
+            (' الاوروبي ', ' الأوروبي '),
+            (' الاوروبية ', ' الأوروبية '),
+            (' الاوروبيان ', ' الأوروبيان '),
+            (' الاوروبيتان ', ' الأوروبيتان '),
+            (' الاوروبيون ', ' الأوروبيون '),
+            (' الاوروبيات ', ' الأوروبيات '),
+            (' اسرائيلي ', ' إسرائيلي '),
+            (' اسرائيلية ', ' إسرائيلية '),
+            (' اسرائيليان ', ' إسرائيليان '),
+            (' اسرائيليتان ', ' إسرائيليتان '),
         ],
         'exceptions': {
             'inside-tags': [
-                'gallery',  # because of filenames
+                'file',         # because of filenames
+                'gallery',      # because of filenames
+                'header',       # section headers
+                'hyperlink',    # URLs
                 'interwiki',
-                'math',
                 'ref',
             ],
         }
@@ -576,6 +576,7 @@ fixes = {
         'regex': False,
         'msg': {
             'ar': 'روبوت: إصلاح حالة حروف الصفحات الخاصة',
+            'arz': 'روبوت: تصليح حاله الحروف بتاعه الصفحات الخصوصيه',
             'en': 'Robot: Fixing special page capitalisation',
             'fa': 'ربات: تصحیح بزرگی و کوچکی حروف صفحه‌های ویژه',
         },
@@ -601,13 +602,14 @@ fixes = {
         ],
     },
     # yu top-level domain will soon be disabled, see
-    # http://lists.wikimedia.org/pipermail/wikibots-l/2009-February/000290.html
+    # https://lists.wikimedia.org/pipermail/wikibots-l/2009-February/000290.html
     # The following are domains that are often-used.
     'yu-tld': {
         'regex': False,
         'nocase': True,
         'msg': {
             'ar':  'روبوت: إصلاح الوصلات إلى نطاقات .yu',
+            'arz':  'روبوت: تصليح اللينكات للدومين .yu',
             'de':  'Bot: Ersetze Links auf .yu-Domains',
             'en':  'Robot: Replacing links to .yu domains',
             'fa':  'ربات: جایگزینی پیوندها به دامنه‌ها با پسوند yu',
