@@ -1,40 +1,36 @@
-# -*- coding: utf-8 -*-
 """Objects representing Flow entities, like boards, topics, and posts."""
 #
 # (C) Pywikibot team, 2015-2020
 #
 # Distributed under the terms of the MIT license.
 #
+import abc
 import logging
 
 from urllib.parse import urlparse, parse_qs
 
 from pywikibot.exceptions import NoPage, UnknownExtension, LockedPage
 from pywikibot.page import BasePage, User
+from pywikibot.tools import deprecate_arg
 
 
 logger = logging.getLogger('pywiki.wiki.flow')
 
 
 # Flow page-like objects (boards and topics)
-class FlowPage(BasePage):
+class FlowPage(BasePage, abc.ABC):
 
-    """
-    The base page for the Flow extension.
+    """The base page meta class for the Flow extension.
 
-    There should be no need to instantiate this directly.
-
-    Subclasses must provide a _load() method to load and cache
-    the object's internal data from the API.
+    It cannot be instantiated directly.
     """
 
-    def __init__(self, source, title=''):
+    def __init__(self, source, title: str = ''):
         """Initializer.
 
         @param source: A Flow-enabled site or a Link or Page on such a site
         @type source: Site, pywikibot.page.Link, or pywikibot.page.Page
         @param title: normalized title of the page
-        @type title: str
 
         @raises TypeError: incorrect use of parameters
         @raises ValueError: use of non-Flow-enabled Site
@@ -44,25 +40,31 @@ class FlowPage(BasePage):
         if not self.site.has_extension('Flow'):
             raise UnknownExtension('site is not Flow-enabled')
 
-    def _load_uuid(self):
-        """Load and save the UUID of the page."""
-        self._uuid = self._load()['workflowId']
+    @abc.abstractmethod
+    def _load(self, force: bool = False):
+        """Abstract method to load and cache the Flow data.
+
+        Subclasses must overwrite _load() method to load and cache
+        the object's internal data from the API.
+        """
+        raise NotImplementedError
 
     @property
-    def uuid(self):
+    def uuid(self) -> str:
         """Return the UUID of the page.
 
         @return: UUID of the page
-        @rtype: str
         """
         if not hasattr(self, '_uuid'):
-            self._load_uuid()
+            self._uuid = self._load()['workflowId']
         return self._uuid
 
     def get(self, force=False, get_redirect=False):
         """Get the page's content."""
         if get_redirect or force:
-            raise NotImplementedError
+            raise NotImplementedError(
+                "Neither 'force' nor 'get_redirect' parameter is implemented "
+                'in {}.get()'.format(self.__class__.__name__))
 
         # TODO: Return more useful data
         return self._data
@@ -72,8 +74,11 @@ class Board(FlowPage):
 
     """A Flow discussion board."""
 
-    def _load(self, force=False):
-        """Load and cache the Board's data, derived from its topic list."""
+    def _load(self, force: bool = False):
+        """Load and cache the Board's data, derived from its topic list.
+
+        @param force: Whether to force a reload if the data is already loaded
+        """
         if not hasattr(self, '_data') or force:
             self._data = self.site.load_board(self)
         return self._data
@@ -93,34 +98,31 @@ class Board(FlowPage):
                     new_params[key] = value
         return new_params
 
-    def topics(self, format='wikitext', limit=100, sort_by='newest',
-               offset=None, offset_uuid='', reverse=False,
-               include_offset=False, toc_only=False):
+    @deprecate_arg('format', 'content_format')
+    def topics(self, content_format: str = 'wikitext', limit: int = 100,
+               sort_by: str = 'newest', offset=None, offset_uuid: str = '',
+               reverse: bool = False, include_offset: bool = False,
+               toc_only: bool = False):
         """Load this board's topics.
 
-        @param format: The content format to request the data in.
-        @type format: str (either 'wikitext', 'html', or 'fixed-html')
+        @param content_format: The content format to request the data in;
+            must be either 'wikitext', 'html', or 'fixed-html'
         @param limit: The number of topics to fetch in each request.
-        @type limit: int
-        @param sort_by: Algorithm to sort topics by.
-        @type sort_by: str (either 'newest' or 'updated')
+        @param sort_by: Algorithm to sort topics by;
+            must be either 'newest' or 'updated'
         @param offset: The timestamp to start at (when sortby is 'updated').
         @type offset: Timestamp or equivalent str
         @param offset_uuid: The UUID to start at (when sortby is 'newest').
-        @type offset_uuid: str (in the form of a UUID)
         @param reverse: Whether to reverse the topic ordering.
-        @type reverse: bool
         @param include_offset: Whether to include the offset topic.
-        @type include_offset: bool
         @param toc_only: Whether to only include information for the TOC.
-        @type toc_only: bool
         @return: A generator of this board's topics.
         @rtype: generator of Topic objects
         """
-        data = self.site.load_topiclist(self, format=format, limit=limit,
-                                        sortby=sort_by, toconly=toc_only,
-                                        offset=offset, offset_id=offset_uuid,
-                                        reverse=reverse,
+        data = self.site.load_topiclist(self, content_format=content_format,
+                                        limit=limit, sortby=sort_by,
+                                        toconly=toc_only, offset=offset,
+                                        offset_id=offset_uuid, reverse=reverse,
                                         include_offset=include_offset)
         while data['roots']:
             for root in data['roots']:
@@ -129,29 +131,33 @@ class Board(FlowPage):
             cont_args = self._parse_url(data['links']['pagination'])
             data = self.site.load_topiclist(self, **cont_args)
 
-    def new_topic(self, title, content, format='wikitext'):
+    @deprecate_arg('format', 'content_format')
+    def new_topic(self, title: str, content: str,
+                  content_format: str = 'wikitext'):
         """Create and return a Topic object for a new topic on this Board.
 
         @param title: The title of the new topic (must be in plaintext)
-        @type title: str
         @param content: The content of the topic's initial post
-        @type content: str
-        @param format: The content format of the value supplied for content
-        @type format: str (either 'wikitext' or 'html')
+        @param content_format: The content format of the supplied content;
+            either 'wikitext' or 'html'
         @return: The new topic
         @rtype: Topic
         """
-        return Topic.create_topic(self, title, content, format)
+        return Topic.create_topic(self, title, content, content_format)
 
 
 class Topic(FlowPage):
 
     """A Flow discussion topic."""
 
-    def _load(self, format='wikitext', force=False):
-        """Load and cache the Topic's data."""
+    def _load(self, force: bool = False, content_format: str = 'wikitext'):
+        """Load and cache the Topic's data.
+
+        @param force: Whether to force a reload if the data is already loaded
+        @param content_format: The post format in which to load
+        """
         if not hasattr(self, '_data') or force:
-            self._data = self.site.load_topic(self, format)
+            self._data = self.site.load_topic(self, content_format)
         return self._data
 
     def _reload(self):
@@ -159,33 +165,32 @@ class Topic(FlowPage):
         self.root._load(load_from_topic=True)
 
     @classmethod
-    def create_topic(cls, board, title, content, format='wikitext'):
+    @deprecate_arg('format', 'content_format')
+    def create_topic(cls, board, title: str, content: str,
+                     content_format: str = 'wikitext'):
         """Create and return a Topic object for a new topic on a Board.
 
         @param board: The topic's parent board
         @type board: Board
         @param title: The title of the new topic (must be in plaintext)
-        @type title: str
         @param content: The content of the topic's initial post
-        @type content: str
-        @param format: The content format of the value supplied for content
-        @type format: str (either 'wikitext' or 'html')
+        @param content_format: The content format of the supplied content;
+            either 'wikitext' or 'html'
         @return: The new topic
         @rtype: Topic
         """
-        data = board.site.create_new_topic(board, title, content, format)
+        data = board.site.create_new_topic(board, title, content,
+                                           content_format)
         return cls(board.site, data['topic-page'])
 
     @classmethod
-    def from_topiclist_data(cls, board, root_uuid, topiclist_data):
+    def from_topiclist_data(cls, board, root_uuid: str, topiclist_data: dict):
         """Create a Topic object from API data.
 
         @param board: The topic's parent Flow board
         @type board: Board
         @param root_uuid: The UUID of the topic and its root post
-        @type root_uuid: str
         @param topiclist_data: The data returned by view-topiclist
-        @type topiclist_data: dict
         @return: A Topic object derived from the supplied data
         @rtype: Topic
         @raises TypeError: any passed parameters have wrong types
@@ -218,81 +223,75 @@ class Topic(FlowPage):
         """Whether this topic is moderated."""
         return self.root._current_revision['isModerated']
 
-    def replies(self, format='wikitext', force=False):
+    @deprecate_arg('format', 'content_format')
+    def replies(self, content_format: str = 'wikitext', force: bool = False):
         """A list of replies to this topic's root post.
 
-        @param format: Content format to return contents in
-        @type format: str ('wikitext', 'html', or 'fixed-html')
+        @param content_format: Content format to return contents in;
+            must be 'wikitext', 'html', or 'fixed-html'
         @param force: Whether to reload from the API instead of using the cache
-        @type force: bool
         @return: The replies of this topic's root post
         @rtype: list of Posts
         """
-        return self.root.replies(format=format, force=force)
+        return self.root.replies(content_format=content_format, force=force)
 
-    def reply(self, content, format='wikitext'):
+    @deprecate_arg('format', 'content_format')
+    def reply(self, content: str, content_format: str = 'wikitext'):
         """A convenience method to reply to this topic's root post.
 
         @param content: The content of the new post
-        @type content: str
-        @param format: The format of the given content
-        @type format: str ('wikitext' or 'html')
+        @param content_format: The format of the given content;
+            must be 'wikitext' or 'html')
         @return: The new reply to this topic's root post
         @rtype: Post
         """
-        return self.root.reply(content, format)
+        return self.root.reply(content, content_format)
 
     # Moderation
-    def lock(self, reason):
+    def lock(self, reason: str):
         """Lock this topic.
 
         @param reason: The reason for locking this topic
-        @type reason: str
         """
         self.site.lock_topic(self, True, reason)
         self._reload()
 
-    def unlock(self, reason):
+    def unlock(self, reason: str):
         """Unlock this topic.
 
         @param reason: The reason for unlocking this topic
-        @type reason: str
         """
         self.site.lock_topic(self, False, reason)
         self._reload()
 
-    def delete_mod(self, reason):
+    def delete_mod(self, reason: str):
         """Delete this topic through the Flow moderation system.
 
         @param reason: The reason for deleting this topic.
-        @type reason: str
         """
         self.site.delete_topic(self, reason)
         self._reload()
 
-    def hide(self, reason):
+    def hide(self, reason: str):
         """Hide this topic.
 
         @param reason: The reason for hiding this topic.
-        @type reason: str
         """
         self.site.hide_topic(self, reason)
         self._reload()
 
-    def suppress(self, reason):
+    def suppress(self, reason: str):
         """Suppress this topic.
 
         @param reason: The reason for suppressing this topic.
-        @type reason: str
         """
         self.site.suppress_topic(self, reason)
         self._reload()
 
-    def restore(self, reason):
+    def restore(self, reason: str):
         """Restore this topic.
 
         @param reason: The reason for restoring this topic.
-        @type reason: str
         """
         self.site.restore_topic(self, reason)
         self._reload()
@@ -303,14 +302,13 @@ class Post:
 
     """A post to a Flow discussion topic."""
 
-    def __init__(self, page, uuid):
+    def __init__(self, page, uuid: str):
         """
         Initializer.
 
         @param page: Flow topic
         @type page: Topic
         @param uuid: UUID of a Flow post
-        @type uuid: str
 
         @raises TypeError: incorrect types of parameters
         """
@@ -327,16 +325,14 @@ class Post:
         self._content = {}
 
     @classmethod
-    def fromJSON(cls, page, post_uuid, data):
+    def fromJSON(cls, page, post_uuid: str, data: dict):  # noqa: N802
         """
         Create a Post object using the data returned from the API call.
 
         @param page: A Flow topic
         @type page: Topic
         @param post_uuid: The UUID of the post
-        @type post_uuid: str
         @param data: The JSON data returned from the API
-        @type data: dict
 
         @return: A Post object
         @raises TypeError: data is not a dict
@@ -347,11 +343,10 @@ class Post:
 
         return post
 
-    def _set_data(self, data):
+    def _set_data(self, data: dict):
         """Set internal data and cache content.
 
         @param data: The data to store internally
-        @type data: dict
         @raises TypeError: data is not a dict
         @raises ValueError: missing data entries or post/revision not found
         """
@@ -374,22 +369,25 @@ class Post:
             assert isinstance(content['content'], str)
             self._content[content['format']] = content['content']
 
-    def _load(self, format='wikitext', load_from_topic=False):
-        """Load and cache the Post's data using the given content format."""
+    def _load(self, force: bool = True, content_format: str = 'wikitext',
+              load_from_topic: bool = False):
+        """Load and cache the Post's data using the given content format.
+
+        @param load_from_topic: Whether to load the post from the whole topic
+        """
         if load_from_topic:
-            data = self.page._load(format=format, force=True)
+            data = self.page._load(force=force, content_format=content_format)
         else:
             data = self.site.load_post_current_revision(self.page, self.uuid,
-                                                        format)
+                                                        content_format)
         self._set_data(data)
         return self._current_revision
 
     @property
-    def uuid(self):
+    def uuid(self) -> str:
         """Return the UUID of the post.
 
         @return: UUID of the post
-        @rtype: str
         """
         return self._uuid
 
@@ -407,7 +405,7 @@ class Post:
         """Return the page associated with the post.
 
         @return: Page associated with the post
-        @rtype: FlowPage
+        @rtype: Topic
         """
         return self._page
 
@@ -428,31 +426,30 @@ class Post:
                                  self._current_revision['creator']['name'])
         return self._creator
 
-    def get(self, format='wikitext', force=False):
+    @deprecate_arg('format', 'content_format')
+    def get(self, content_format: str = 'wikitext',
+            force: bool = False) -> str:
         """Return the contents of the post in the given format.
 
         @param force: Whether to reload from the API instead of using the cache
-        @type force: bool
-        @param format: Content format to return contents in
-        @type format: str
+        @param content_format: Content format to return contents in
         @return: The contents of the post in the given content format
-        @rtype: str
         """
-        if format not in self._content or force:
-            self._load(format)
-        return self._content[format]
+        if content_format not in self._content or force:
+            self._load(content_format=content_format)
+        return self._content[content_format]
 
-    def replies(self, format='wikitext', force=False):
+    @deprecate_arg('format', 'content_format')
+    def replies(self, content_format: str = 'wikitext', force: bool = False):
         """Return this post's replies.
 
-        @param format: Content format to return contents in
-        @type format: str ('wikitext', 'html', or 'fixed-html')
+        @param content_format: Content format to return contents in;
+            must be 'wikitext', 'html', or 'fixed-html'
         @param force: Whether to reload from the API instead of using the cache
-        @type force: bool
         @return: This post's replies
         @rtype: list of Posts
         """
-        if format not in ('wikitext', 'html', 'fixed-html'):
+        if content_format not in ('wikitext', 'html', 'fixed-html'):
             raise ValueError('Invalid content format.')
 
         if hasattr(self, '_replies') and not force:
@@ -461,20 +458,20 @@ class Post:
         # load_from_topic workaround due to T106733
         # (replies not returned by view-post)
         if not hasattr(self, '_current_revision') or force:
-            self._load(format, load_from_topic=True)
+            self._load(content_format=content_format, load_from_topic=True)
 
         reply_uuids = self._current_revision['replies']
         self._replies = [Post(self.page, uuid) for uuid in reply_uuids]
 
         return self._replies
 
-    def reply(self, content, format='wikitext'):
+    @deprecate_arg('format', 'content_format')
+    def reply(self, content: str, content_format: str = 'wikitext'):
         """Reply to this post.
 
         @param content: The content of the new post
-        @type content: str
-        @param format: The format of the given content
-        @type format: str ('wikitext' or 'html')
+        @param content_format: The format of the given content;
+            must be 'wikitext' or 'html'
         @return: The new reply post
         @rtype: Post
         """
@@ -489,43 +486,40 @@ class Post:
         if self.uuid == reply_to:
             del self._current_revision
             del self._replies
-        data = self.site.reply_to_post(self.page, reply_to, content, format)
+        data = self.site.reply_to_post(self.page, reply_to, content,
+                                       content_format)
         post = Post(self.page, data['post-id'])
         return post
 
     # Moderation
-    def delete(self, reason):
+    def delete(self, reason: str):
         """Delete this post through the Flow moderation system.
 
         @param reason: The reason for deleting this post.
-        @type reason: str
         """
         self.site.delete_post(self, reason)
         self._load()
 
-    def hide(self, reason):
+    def hide(self, reason: str):
         """Hide this post.
 
         @param reason: The reason for hiding this post.
-        @type reason: str
         """
         self.site.hide_post(self, reason)
         self._load()
 
-    def suppress(self, reason):
+    def suppress(self, reason: str):
         """Suppress this post.
 
         @param reason: The reason for suppressing this post.
-        @type reason: str
         """
         self.site.suppress_post(self, reason)
         self._load()
 
-    def restore(self, reason):
+    def restore(self, reason: str):
         """Restore this post.
 
         @param reason: The reason for restoring this post.
-        @type reason: str
         """
         self.site.restore_post(self, reason)
         self._load()
