@@ -1,23 +1,21 @@
-# -*- coding: utf-8 -*-
-"""
-Various i18n functions.
+"""Various i18n functions.
 
-Helper functions for both the internal translation system
-and for TranslateWiki-based translations.
+Helper functions for both the internal localization system and for
+TranslateWiki-based translations.
 
 By default messages are assumed to reside in a package called
-'scripts.i18n'. In pywikibot 3+, that package is not packaged
-with pywikibot, and pywikibot 3+ does not have a hard dependency
-on any i18n messages. However, there are three user input questions
-in pagegenerators which will use i18n messages if they can be loaded.
+'scripts.i18n'. In pywikibot 3+, that package is not packaged with
+pywikibot, and pywikibot 3+ does not have a hard dependency on any i18n
+messages. However, there are three user input questions in pagegenerators
+which will use i18n messages if they can be loaded.
 
 The default message location may be changed by calling
-L{set_message_package} with a package name. The package must contain
-an __init__.py, and a message bundle called 'pywikibot' containing
-messages. See L{twtranslate} for more information on the messages.
+L{set_message_package} with a package name. The package must contain an
+__init__.py, and a message bundle called 'pywikibot' containing messages.
+See L{twtranslate} for more information on the messages.
 """
 #
-# (C) Pywikibot team, 2004-2020
+# (C) Pywikibot team, 2004-2021
 #
 # Distributed under the terms of the MIT license.
 #
@@ -28,18 +26,22 @@ import re
 
 from collections.abc import Mapping
 from collections import defaultdict
+from contextlib import suppress
 from textwrap import fill
+from typing import Optional, Union
 from warnings import warn
 
 import pywikibot
 
 from pywikibot import __url__
+from pywikibot.backports import cache, List
 from pywikibot import config2 as config
 from pywikibot.exceptions import Error
-from pywikibot.plural import plural_rules
+from pywikibot.plural import plural_rule
 from pywikibot.tools import (
     deprecated, deprecated_args, issue_deprecation_warning)
 
+
 PLURAL_PATTERN = r'{{PLURAL:(?:%\()?([^\)]*?)(?:\)d)?\|(.*?)}}'
 
 # Package name for the translation messages. The messages data must loaded
@@ -50,10 +52,6 @@ _messages_package_name = 'scripts.i18n'
 # Flag to indicate whether translation messages are available
 _messages_available = None
 
-# Cache of translated messages
-_cache = defaultdict(dict)
-
-
 _LANG_TO_GROUP_NAME = defaultdict(str, {
     'aa': 'aa',
     'ab': 'ab',
@@ -65,6 +63,7 @@ _LANG_TO_GROUP_NAME = defaultdict(str, {
     'an': 'an',
     'arc': 'arc',
     'arn': 'an',
+    'ary': 'arc',
     'arz': 'arc',
     'as': 'as',
     'ast': 'an',
@@ -270,6 +269,7 @@ _LANG_TO_GROUP_NAME = defaultdict(str, {
     'zh-min-nan': 'zh-min-nan',
     'zh-tw': 'zh-classical',
     'zh-yue': 'cdo'})
+
 _GROUP_NAME_TO_FALLBACKS = {
     '': [],
     'aa': ['am'],
@@ -355,7 +355,7 @@ _GROUP_NAME_TO_FALLBACKS = {
         'cdo', 'zh', 'zh-hans', 'zh-tw', 'zh-cn', 'zh-classical', 'lzh']}
 
 
-def set_messages_package(package_name):
+def set_messages_package(package_name: str):
     """Set the package name where i18n messages are located."""
     global _messages_package_name
     global _messages_available
@@ -363,15 +363,13 @@ def set_messages_package(package_name):
     _messages_available = None
 
 
-def messages_available():
+def messages_available() -> bool:
     """
     Return False if there are no i18n messages available.
 
     To determine if messages are available, it looks for the package name
     set using L{set_messages_package} for a message bundle called 'pywikibot'
     containing messages.
-
-    @rtype: bool
     """
     global _messages_available
     if _messages_available is not None:
@@ -390,7 +388,7 @@ def messages_available():
     return True
 
 
-def _altlang(lang):
+def _altlang(lang: str) -> List[str]:
     """Define fallback languages for particular languages.
 
     If no translation is available to a specified language, translate() will
@@ -403,9 +401,7 @@ def _altlang(lang):
     This code is used by other translating methods below.
 
     @param lang: The language code
-    @type lang: str
     @return: language codes
-    @rtype: list of str
     """
     return _GROUP_NAME_TO_FALLBACKS[_LANG_TO_GROUP_NAME[lang]]
 
@@ -421,39 +417,32 @@ class TranslationError(Error, ImportError):
     pass
 
 
-def _get_translation(lang, twtitle):
+@cache
+def _get_translation(lang: str, twtitle: str) -> Optional[str]:
     """
     Return message of certain twtitle if exists.
 
     For internal use, don't use it directly.
     """
-    if twtitle in _cache[lang]:
-        return _cache[lang][twtitle]
     message_bundle = twtitle.split('-')[0]
-    filename = '%s/%s.json' % (message_bundle, lang)
+    filename = '{}/{}.json'.format(message_bundle, lang)
     try:
         trans_text = pkgutil.get_data(
             _messages_package_name, filename).decode('utf-8')
-    except (OSError, IOError):  # file open can cause several exceptions
-        _cache[lang][twtitle] = None
+    except OSError:  # file open can cause several exceptions
         return None
+
     transdict = json.loads(trans_text)
-    _cache[lang].update(transdict)
-    try:
-        return transdict[twtitle]
-    except KeyError:
-        return None
+    return transdict.get(twtitle)
 
 
-def _extract_plural(code, message, parameters):
+def _extract_plural(lang: str, message: str, parameters: Mapping) -> str:
     """Check for the plural variants in message and replace them.
 
     @param message: the message to be replaced
-    @type message: str
     @param parameters: plural parameters passed from other methods
     @type parameters: Mapping of str to int
     @return: The message with the plural instances replaced
-    @rtype: str
     """
     def static_plural_value(n):
         return rule['plural']
@@ -466,12 +455,13 @@ def _extract_plural(code, message, parameters):
             issue_deprecation_warning(
                 'type {0} for value {1} ({2})'
                 .format(type(num), selector, num),
-                'an int', 1, since='20151009')
+                'an int', 1,
+                warning_class=FutureWarning, since='20151009')
             num = int(num)
 
         plural_entries = []
         specific_entries = {}
-        # A plural entry can not start at the end of the variants list,
+        # A plural entry cannot start at the end of the variants list,
         # and must end with | or the end of the variants list.
         for number, plural in re.findall(
             r'(?!$)(?: *(\d+) *= *)?(.*?)(?:\||$)', variants
@@ -499,11 +489,9 @@ def _extract_plural(code, message, parameters):
         return plural_entries[index]
 
     assert isinstance(parameters, Mapping), \
-        'parameters is not Mapping but {0}'.format(type(parameters))
-    try:
-        rule = plural_rules[code]
-    except KeyError:
-        rule = plural_rules['_default']
+        'parameters is not Mapping but {}'.format(type(parameters))
+
+    rule = plural_rule(lang)
     plural_value = rule['plural']
     if not callable(plural_value):
         assert rule['nplurals'] == 1
@@ -526,7 +514,7 @@ class _PluralMappingAlias(Mapping):
         else:
             self.source = source
         self.index = -1
-        super(_PluralMappingAlias, self).__init__()
+        super().__init__()
 
     def __getitem__(self, key):
         self.index += 1
@@ -541,45 +529,50 @@ class _PluralMappingAlias(Mapping):
         return self.source
 
     def __iter__(self):
-        raise NotImplementedError
+        raise NotImplementedError  # pragma: no cover
 
     def __len__(self):
-        raise NotImplementedError
+        raise NotImplementedError  # pragma: no cover
 
 
 DEFAULT_FALLBACK = ('_default', )
 
 
-def translate(code, xdict, parameters=None, fallback=False):
-    """Return the most appropriate translation from a translation dict.
+def translate(code,
+              xdict: Union[dict, str],
+              parameters: Union[dict, str, int, None] = None,
+              fallback=False) -> str:
+    """Return the most appropriate localization from a localization dict.
 
-    Given a language code and a dictionary, returns the dictionary's value for
+    Given a site code and a dictionary, returns the dictionary's value for
     key 'code' if this key exists; otherwise tries to return a value for an
-    alternative language that is most applicable to use on the wiki in
-    language 'code' except fallback is False.
+    alternative code that is most applicable to use on the wiki in language
+    'code' except fallback is False.
 
-    The language itself is always checked first, then languages that
-    have been defined to be alternatives, and finally English. If none of
-    the options gives result, we just take the one language from xdict which
-    may not be always the same. When fallback is iterable it'll return None if
-    no code applies (instead of returning one).
+    The code itself is always checked first, then these codes that have
+    been defined to be alternatives, and finally English.
+
+    If fallback is False and the code is not found in the
 
     For PLURAL support have a look at the twtranslate method.
 
-    @param code: The language code
+    @param code: The site code as string or Site object. If xdict is an
+        extended dictionary the Site object should be used in favour of the
+        code string. Otherwise localizations from a wrong family might be
+        used.
     @type code: str or Site object
-    @param xdict: dictionary with language codes as keys or extended dictionary
-                  with family names as keys containing language dictionaries or
-                  a single (unicode) string. May contain PLURAL tags as
-                  described in twtranslate
-    @type xdict: dict, string, unicode
+    @param xdict: dictionary with language codes as keys or extended
+        dictionary with family names as keys containing code dictionaries
+        or a single string. May contain PLURAL tags as described in
+        twtranslate
     @param parameters: For passing (plural) parameters
-    @type parameters: dict, string, unicode, int
     @param fallback: Try an alternate language code. If it's iterable it'll
         also try those entries and choose the first match.
     @type fallback: boolean or iterable
-    @raise IndexError: If the language supports and requires more plurals than
-        defined for the given translation template.
+    @return: the localized string
+    @raise IndexError: If the language supports and requires more plurals
+        than defined for the given PLURAL pattern.
+    @raise KeyError: No fallback key found if fallback is not False
     """
     family = pywikibot.config.family
     # If a site is given instead of a code, use its language
@@ -587,17 +580,19 @@ def translate(code, xdict, parameters=None, fallback=False):
         family = code.family.name
         code = code.code
 
-    # Check whether xdict has multiple projects
-    if isinstance(xdict, dict):
-        if family in xdict:
-            xdict = xdict[family]
-        elif 'wikipedia' in xdict:
-            xdict = xdict['wikipedia']
+    try:
+        lookup = xdict[code]
+    except (KeyError, TypeError):
+        # Check whether xdict has multiple projects
+        if isinstance(xdict, dict) and family in xdict:
+            lookup = xdict[family]
+        else:
+            lookup = xdict
 
     # Get the translated string
-    if not isinstance(xdict, dict):
-        trans = xdict
-    elif not xdict:
+    if not isinstance(lookup, dict):
+        trans = lookup
+    elif not lookup:
         trans = None
     else:
         codes = [code]
@@ -606,21 +601,28 @@ def translate(code, xdict, parameters=None, fallback=False):
         elif fallback is not False:
             codes.extend(fallback)
         for code in codes:
-            if code in xdict:
-                trans = xdict[code]
+            if code in lookup:
+                trans = lookup[code]
                 break
         else:
-            if fallback is False:
-                return None
-            raise KeyError('No fallback key found in lookup dict for "{}"'
-                           .format(code))
+            if fallback is not False:
+                raise KeyError('No fallback key found in lookup dict for "{}"'
+                               .format(code))
+            trans = None
+
     if trans is None:
+        if 'wikipedia' in xdict:
+            # fallback to wikipedia family
+            return translate(code, xdict['wikipedia'],
+                             parameters=parameters, fallback=fallback)
         return None  # return None if we have no translation found
+
     if parameters is None:
         return trans
 
     if not isinstance(parameters, Mapping):
         issue_deprecation_warning('parameters not being a mapping',
+                                  warning_class=FutureWarning,
                                   since='20151008')
         plural_parameters = _PluralMappingAlias(parameters)
     else:
@@ -629,18 +631,20 @@ def translate(code, xdict, parameters=None, fallback=False):
     # else we check for PLURAL variants
     trans = _extract_plural(code, trans, plural_parameters)
     if parameters:
-        try:
-            return trans % parameters
-        except (KeyError, TypeError):
-            # parameter is for PLURAL variants only, don't change the string
-            pass
+        # On error: parameter is for PLURAL variants only,
+        # don't change the string
+        with suppress(KeyError, TypeError):
+            trans = trans % parameters
     return trans
 
 
 @deprecated_args(code='source')
-def twtranslate(
-    source, twtitle, parameters=None, fallback=True, only_plural=False
-):
+def twtranslate(source,
+                twtitle: str,
+                parameters: Optional[Mapping] = None, *,
+                fallback: bool = True,
+                fallback_prompt: Optional[str] = None,
+                only_plural: bool = False) -> Optional[str]:
     r"""
     Translate a message using JSON files in messages_package_name.
 
@@ -672,6 +676,11 @@ def twtranslate(
 
     and so on.
 
+    >>> # this code snippet is running in test environment
+    >>> # ignore test message "tests: max_retries reduced from 15 to 1"
+    >>> import os
+    >>> os.environ['PYWIKIBOT_TEST_QUIET'] = '1'
+
     >>> from pywikibot import i18n
     >>> i18n.set_messages_package('tests.i18n')
     >>> # use a dictionary
@@ -696,16 +705,20 @@ def twtranslate(
         They are also used for plural entries in which case they must be a
         Mapping and will cause a TypeError otherwise.
     @param fallback: Try an alternate language code
-    @type fallback: boolean
+    @param fallback_prompt: The English message if i18n is not available
     @param only_plural: Define whether the parameters should be only applied to
         plural instances. If this is False it will apply the parameters also
         to the resulting string. If this is True the placeholders must be
         manually applied afterwards.
-    @type only_plural: bool
     @raise IndexError: If the language supports and requires more plurals than
         defined for the given translation template.
     """
     if not messages_available():
+        if fallback_prompt:
+            if parameters and not only_plural:
+                return fallback_prompt % parameters
+            return fallback_prompt
+
         raise TranslationError(
             'Unable to load messages package %s for bundle %s'
             '\nIt can happen due to lack of i18n submodule or files. '
@@ -762,25 +775,24 @@ def twtranslate(
         # This is called due to the old twntranslate function which ignored
         # KeyError. Instead only_plural should be used.
         if isinstance(parameters.source, dict):
-            try:
+            with suppress(KeyError):
                 trans %= parameters.source
-            except KeyError:
-                pass
         parameters = None
 
     if parameters is not None and not isinstance(parameters, Mapping):
         issue_deprecation_warning('parameters not being a Mapping',
+                                  warning_class=FutureWarning,
                                   since='20151008')
 
     if not only_plural and parameters:
         return trans % parameters
-    else:
-        return trans
+    return trans
 
 
-@deprecated('twtranslate', since='20151009')
+@deprecated('twtranslate', since='20151009', future_warning=True)
 @deprecated_args(code='source')
-def twntranslate(source, twtitle, parameters=None):
+def twntranslate(source, twtitle: str,
+                 parameters: Optional[Mapping] = None) -> Optional[str]:
     """DEPRECATED: Get translated string for the key."""
     if parameters is not None:
         parameters = _PluralMappingAlias(parameters)
@@ -788,7 +800,7 @@ def twntranslate(source, twtitle, parameters=None):
 
 
 @deprecated_args(code='source')
-def twhas_key(source, twtitle):
+def twhas_key(source, twtitle: str) -> bool:
     """
     Check if a message has a translation in the specified language code.
 
@@ -808,13 +820,13 @@ def twhas_key(source, twtitle):
     return transdict is not None
 
 
-def twget_keys(twtitle):
+def twget_keys(twtitle: str) -> List[str]:
     """
     Return all language codes for a special message.
 
     @param twtitle: The TranslateWiki string title, in <package>-<key> format
 
-    @raises OSError: the package i18n can not be loaded
+    @raises OSError: the package i18n cannot be loaded
     """
     # obtain the directory containing all the json files for this package
     package = twtitle.split('-')[0]
@@ -832,7 +844,10 @@ def twget_keys(twtitle):
             if lang != 'qqq' and _get_translation(lang, twtitle)]
 
 
-def input(twtitle, parameters=None, password=False, fallback_prompt=None):
+def input(twtitle: str,
+          parameters: Optional[Mapping] = None,
+          password: bool = False,
+          fallback_prompt: Optional[str] = None) -> str:
     """
     Ask the user a question, return the user's answer.
 
@@ -843,17 +858,14 @@ def input(twtitle, parameters=None, password=False, fallback_prompt=None):
     @param parameters: The values which will be applied to the translated text
     @param password: Hides the user's input (for password entry)
     @param fallback_prompt: The English prompt if i18n is not available.
-    @rtype: str
     """
-    if not messages_available():
-        if not fallback_prompt:
-            raise TranslationError(
-                'Unable to load messages package %s for bundle %s'
-                % (_messages_package_name, twtitle))
-        else:
-            prompt = fallback_prompt
-    else:
+    if messages_available():
         code = config.userinterface_lang
-
         prompt = twtranslate(code, twtitle, parameters)
+    elif fallback_prompt:
+        prompt = fallback_prompt
+    else:
+        raise TranslationError(
+            'Unable to load messages package {} for bundle {}'
+            .format(_messages_package_name, twtitle))
     return pywikibot.input(prompt, password)
