@@ -1,8 +1,7 @@
 #!/usr/bin/python
-# -*- coding: utf-8 -*-
 """Library to log the bot in to a wiki account."""
 #
-# (C) Pywikibot team, 2003-2020
+# (C) Pywikibot team, 2003-2021
 #
 # Distributed under the terms of the MIT license.
 #
@@ -10,11 +9,14 @@ import codecs
 import os
 import webbrowser
 
+from enum import IntEnum
+from typing import Optional
 from warnings import warn
 
 import pywikibot
 
 from pywikibot import config, __url__
+from pywikibot.comms import http
 from pywikibot.exceptions import NoUsername
 from pywikibot.tools import (
     deprecated_args, file_mode_checker, normalize_username, remove_last_args,
@@ -54,12 +56,42 @@ botList = {
 }
 
 
+class LoginStatus(IntEnum):
+
+    """
+    Enum for Login statuses.
+
+    >>> LoginStatus.NOT_ATTEMPTED
+    LoginStatus(-3)
+    >>> LoginStatus.IN_PROGRESS.value
+    -2
+    >>> LoginStatus.NOT_LOGGED_IN.name
+    'NOT_LOGGED_IN'
+    >>> int(LoginStatus.AS_USER)
+    0
+    >>> LoginStatus(-3).name
+    'NOT_ATTEMPTED'
+    >>> LoginStatus(0).name
+    'AS_USER'
+    """
+
+    NOT_ATTEMPTED = -3
+    IN_PROGRESS = -2
+    NOT_LOGGED_IN = -1
+    AS_USER = 0
+
+    def __repr__(self):
+        """Return internal representation."""
+        return 'LoginStatus({})'.format(self)
+
+
 class LoginManager:
 
     """Site login manager."""
 
     @deprecated_args(username='user', verbose=None, sysop=None)
-    def __init__(self, password=None, site=None, user=None):
+    def __init__(self, password: Optional[str] = None,
+                 site=None, user: Optional[str] = None):
         """
         Initializer.
 
@@ -69,9 +101,7 @@ class LoginManager:
         @type site: BaseSite
         @param user: username to use.
             If user is None, the username is loaded from config.usernames.
-        @type user: basestring
         @param password: password to use
-        @type password: basestring
 
         @raises pywikibot.exceptions.NoUsername: No username is configured
             for the requested site.
@@ -155,32 +185,15 @@ class LoginManager:
         # No bot policies on other sites
         return True
 
-    @remove_last_args(['remember', 'captcha'])
-    def getCookie(self):
-        """
-        Login to the site.
-
-        @see: U{https://www.mediawiki.org/wiki/API:Login}
-
-        @return: cookie data if successful, None otherwise.
-        """
+    def login_to_site(self):
+        """Login to the site."""
         # THIS IS OVERRIDDEN IN data/api.py
-        return None
+        raise NotImplementedError
 
-    def storecookiedata(self, data):
-        """
-        Store cookie data.
-
-        @param data: The raw data as returned by getCookie()
-        @type data: str
-
-        @return: None
-        """
-        # THIS IS OVERRIDDEN IN data/api.py
-        filename = config.datafilepath('pywikibot.lwp')
-        pywikibot.debug('Storing cookies to {}'.format(filename), _logger)
-        with open(filename, 'w') as f:
-            f.write(data)
+    @remove_last_args(['data'])
+    def storecookiedata(self) -> None:
+        """Store cookie data."""
+        http.cookie_jar.save(ignore_discard=True)
 
     def readPassword(self):
         """
@@ -296,7 +309,7 @@ class LoginManager:
         pywikibot.output('Logging in to {site} as {name}'
                          .format(name=self.login_name, site=self.site))
         try:
-            cookiedata = self.getCookie()
+            self.login_to_site()
         except pywikibot.data.api.APIError as e:
             error_code = e.code
             pywikibot.error('Login failed ({}).'.format(error_code))
@@ -310,12 +323,14 @@ class LoginManager:
             # TODO: investigate other unhandled API codes (bug T75539)
             if retry:
                 self.password = None
-                return self.login(retry=True)
-            else:
-                return False
-        self.storecookiedata(cookiedata)
-        pywikibot.log('Should be logged in now')
-        return True
+                return self.login(retry=False)
+
+        else:
+            self.storecookiedata()
+            pywikibot.log('Should be logged in now')
+            return True
+
+        return False
 
 
 class BotPassword:
@@ -357,7 +372,8 @@ class OauthLoginManager(LoginManager):
     # authentication process
 
     @deprecated_args(sysop=None)
-    def __init__(self, password=None, site=None, user=None):
+    def __init__(self, password: Optional[str] = None, site=None,
+                 user: Optional[str] = None):
         """
         Initializer.
 
@@ -366,9 +382,7 @@ class OauthLoginManager(LoginManager):
         @param site: Site object to log into
         @type site: BaseSite
         @param user: consumer key
-        @type user: str
         @param password: consumer secret
-        @type password: str
 
         @raises pywikibot.exceptions.NoUsername: No username is configured
             for the requested site.
@@ -447,12 +461,8 @@ class OauthLoginManager(LoginManager):
         return self._access_token
 
     @property
-    def identity(self):
-        """
-        Get identifying information about a user via an authorized token.
-
-        @rtype: None or dict
-        """
+    def identity(self) -> Optional[dict]:
+        """Get identifying information about a user via an authorized token."""
         if self.access_token is None:
             pywikibot.error('Access token not set')
             return None
