@@ -1,9 +1,6 @@
-# -*- coding: utf-8 -*-
 """
 Objects used with ProofreadPage Extension.
 
-The extension is supported by MW 1.21+.
-
 This module includes objects:
 
 * ProofreadPage(Page)
@@ -22,7 +19,7 @@ OCR support of page scans via:
 
 """
 #
-# (C) Pywikibot team, 2015-2020
+# (C) Pywikibot team, 2015-2021
 #
 # Distributed under the terms of the MIT license.
 #
@@ -32,9 +29,10 @@ import time
 
 from functools import partial
 from requests.exceptions import ReadTimeout
+from typing import Optional
 
 try:
-    from bs4 import BeautifulSoup, FeatureNotFound
+    from bs4 import BeautifulSoup
 except ImportError as e:
     BeautifulSoup = e
 
@@ -42,6 +40,7 @@ except ImportError as e:
         """Raise BeautifulSoup when called, if bs4 is not available."""
         raise BeautifulSoup
 else:
+    from bs4 import FeatureNotFound
     try:
         BeautifulSoup('', 'lxml')
     except FeatureNotFound:
@@ -54,11 +53,36 @@ import pywikibot
 from pywikibot.comms import http
 from pywikibot.data.api import Request
 from pywikibot.exceptions import OtherPageSaveError
+from pywikibot import textlib
 from pywikibot.tools import ModuleDeprecationWrapper
 
 _logger = 'proofreadpage'
 
 
+def decompose(fn):  # noqa: N805
+    """Decorator.
+
+    Decompose text if needed and recompose text.
+    """
+    def wrapper(obj, *args, **kwargs):
+        if not hasattr(obj, '_full_header'):
+            obj._decompose_page()
+        _res = fn(obj, *args, **kwargs)
+        obj._compose_page()
+        return _res
+
+    return wrapper
+
+
+def check_if_cached(fn):  # noqa: N805
+    """Decorator to check if data are cached and cache them if needed."""
+    def wrapper(self, *args, **kwargs):
+        if self._cached is False:
+            self._get_page_mappings()
+        return fn(self, *args, **kwargs)
+    return wrapper
+
+
 class FullHeader:
 
     """Header of a ProofreadPage object."""
@@ -94,8 +118,7 @@ class FullHeader:
         """Return a string representation."""
         if self._has_div:
             return FullHeader.TEMPLATE_V1.format(self)
-        else:
-            return FullHeader.TEMPLATE_V2.format(self)
+        return FullHeader.TEMPLATE_V2.format(self)
 
 
 class ProofreadPage(pywikibot.Page):
@@ -206,7 +229,7 @@ class ProofreadPage(pywikibot.Page):
         left, sep, right = base.rpartition('.')
         ext = right if sep else ''
 
-        return (base, ext, num)
+        return base, ext, num
 
     @property
     def index(self):
@@ -260,7 +283,7 @@ class ProofreadPage(pywikibot.Page):
     @index.setter
     def index(self, value):
         if not isinstance(value, IndexPage):
-            raise TypeError('value {} must be a IndexPage object.'
+            raise TypeError('value {} must be an IndexPage object.'
                             .format(value))
         self._index = (value, None)
 
@@ -290,19 +313,6 @@ class ProofreadPage(pywikibot.Page):
             return int(self._quality)
         return self.ql
 
-    def decompose(fn):  # noqa: N805
-        """Decorator.
-
-        Decompose text if needed and recompose text.
-        """
-        def wrapper(obj, *args, **kwargs):
-            if not hasattr(obj, '_full_header'):
-                obj._decompose_page()
-            _res = fn(obj, *args, **kwargs)
-            obj._compose_page()
-            return _res
-        return wrapper
-
     @property
     @decompose
     def ql(self):
@@ -425,14 +435,13 @@ class ProofreadPage(pywikibot.Page):
         return self._text
 
     @text.setter
-    def text(self, value):
+    def text(self, value: str):
         """Update current text.
 
         Mainly for use within the class, called by other methods.
         Use self.header, self.body and self.footer to set page content,
 
         @param value: New value or None
-        @param value: basestring
 
         @raise Error: the page is not formatted according to ProofreadPage
             extension.
@@ -595,11 +604,10 @@ class ProofreadPage(pywikibot.Page):
             try:
                 response = http.fetch(cmd_uri)
             except ReadTimeout as e:
-                timeout = e
                 pywikibot.warning('ReadTimeout %s: %s' % (cmd_uri, e))
             except Exception as e:
                 pywikibot.error('"{}": {}'.format(cmd_uri, e))
-                return (True, e)
+                return True, e
             else:
                 pywikibot.debug('{}: {}'.format(ocr_tool, response.text),
                                 _logger)
@@ -608,10 +616,10 @@ class ProofreadPage(pywikibot.Page):
             pywikibot.warning('retrying in {} seconds ...'.format(retry))
             time.sleep(retry)
         else:
-            return True, timeout
+            return True, ReadTimeout
 
-        if 400 <= response.status < 600:
-            return (True, 'Http response status {}'.format(response.status))
+        if 400 <= response.status_code < 600:
+            return True, 'Http response status {}'.format(response.status_code)
 
         data = json.loads(response.text)
 
@@ -628,9 +636,8 @@ class ProofreadPage(pywikibot.Page):
 
         if error:
             pywikibot.error('OCR query %s: %s' % (cmd_uri, _text))
-            return (error, _text)
-        else:
-            return (error, parser_func(_text))
+            return error, _text
+        return error, parser_func(_text)
 
     def _do_hocr(self):
         """Do hocr using https://phetools.toolforge.org/hocr_cgi.py?cmd=hocr.
@@ -671,7 +678,7 @@ class ProofreadPage(pywikibot.Page):
         except ValueError:
             error_text = 'No prp-page-image src found for %s.' % self
             pywikibot.error(error_text)
-            return (True, error_text)
+            return True, error_text
 
         try:
             cmd_fmt = self._OCR_CMDS[ocr_tool]
@@ -689,7 +696,7 @@ class ProofreadPage(pywikibot.Page):
 
         return self._ocr_callback(cmd_uri, ocr_tool=ocr_tool)
 
-    def ocr(self, ocr_tool=None):
+    def ocr(self, ocr_tool: Optional[str] = None):
         """Do OCR of ProofreadPage scan.
 
         The text returned by this function shall be assigned to self.body,
@@ -698,7 +705,6 @@ class ProofreadPage(pywikibot.Page):
         It is the user's responsibility to reset quality level accordingly.
 
         @param ocr_tool: 'phetools' or 'googleOCR', default is 'phetools'
-        @type ocr_tool: basestring
 
         @return: OCR text for the page.
 
@@ -726,9 +732,8 @@ class ProofreadPage(pywikibot.Page):
 
         if not error:
             return text
-        else:
-            raise ValueError(
-                '{0}: not possible to perform OCR. {1}'.format(self, text))
+        raise ValueError(
+            '{0}: not possible to perform OCR. {1}'.format(self, text))
 
 
 class PurgeRequest(Request):
@@ -755,7 +760,7 @@ class IndexPage(pywikibot.Page):
     INDEX_TEMPLATE = ':MediaWiki:Proofreadpage_index_template'
 
     def __init__(self, source, title=''):
-        """Instantiate a IndexPage object.
+        """Instantiate an IndexPage object.
 
         In this class:
         page number is the number in the page title in the Page namespace, if
@@ -795,23 +800,15 @@ class IndexPage(pywikibot.Page):
 
         self._cached = False
 
-    def check_if_cached(fn):  # noqa: N805
-        """Decorator to check if data are cached and cache them if needed."""
-        def wrapper(self, *args, **kwargs):
-            if self._cached is False:
-                self._get_page_mappings()
-            return fn(self, *args, **kwargs)
-        return wrapper
-
-    def _parse_redlink(self, href):
+    @staticmethod
+    def _parse_redlink(href):
         """Parse page title when link in Index is a redlink."""
         p_href = re.compile(
             r'/w/index\.php\?title=(.+?)&action=edit&redlink=1')
         title = p_href.search(href)
         if title:
             return title.group(1)
-        else:
-            return None
+        return None
 
     def save(self, *args, **kwargs):  # See Page.save().
         """
@@ -835,9 +832,9 @@ class IndexPage(pywikibot.Page):
             return False
 
         # Discard all inner templates as only top-level ones matter
-        tmplts = pywikibot.textlib.extract_templates_and_params_regex_simple(
+        templates = textlib.extract_templates_and_params_regex_simple(
             self.text)
-        if len(tmplts) != 1 or tmplts[0][0] != self.INDEX_TEMPLATE:
+        if len(templates) != 1 or templates[0][0] != self.INDEX_TEMPLATE:
             # Only a single call to the INDEX_TEMPLATE is allowed
             return False
 
@@ -963,24 +960,21 @@ class IndexPage(pywikibot.Page):
         """
         return len(self._page_from_numbers)
 
-    def page_gen(self, start=1, end=None, filter_ql=None,
-                 only_existing=False, content=True):
+    def page_gen(self, start: Optional[int] = 1,
+                 end: Optional[int] = None, filter_ql=None,
+                 only_existing: bool = False, content: bool = True):
         """Return a page generator which yields pages contained in Index page.
 
         Range is [start ... end], extremes included.
 
         @param start: first page, defaults to 1
-        @type start: int
         @param end: num_pages if end is None
-        @type end: int
         @param filter_ql: filters quality levels
                           if None: all but 'Without Text'.
         @type filter_ql: list of ints (corresponding to ql constants
                          defined in ProofreadPage).
         @param only_existing: yields only existing pages.
-        @type only_existing: bool
         @param content: preload content.
-        @type content: bool
         """
         if end is None:
             end = self.num_pages
@@ -1038,7 +1032,8 @@ class IndexPage(pywikibot.Page):
             raise KeyError('Page number ".../{}" not in range.'
                            .format(page_number))
 
-    def _get_from_label(self, mapping_dict, label):
+    @staticmethod
+    def _get_from_label(mapping_dict, label):
         """Helper function to get info from label."""
         # Convert label to string if an integer is passed.
         if isinstance(label, int):
