@@ -1,13 +1,10 @@
 #!/usr/bin/python
-# -*- coding: utf-8 -*-
 """Module containing special bots reusable by scripts."""
 #
 # (C) Pywikibot team, 2020
 #
 # Distributed under the terms of the MIT license.
 #
-from __future__ import absolute_import, division, unicode_literals
-
 from pywikibot.specialbots._unlink import (
     EditReplacement, InteractiveUnlink, BaseUnlinkBot
 )
