@@ -1,7 +1,6 @@
-# -*- coding: utf-8 -*-
 """Miscellaneous helper functions (not wiki-dependent)."""
 #
-# (C) Pywikibot team, 2008-2020
+# (C) Pywikibot team, 2008-2021
 #
 # Distributed under the terms of the MIT license.
 #
@@ -35,9 +34,6 @@ from warnings import catch_warnings, showwarning, warn
 from pywikibot.logging import debug
 from pywikibot.tools._unidata import _first_upper_exception
 
-PYTHON_VERSION = sys.version_info[:3]
-PY2 = (PYTHON_VERSION[0] == 2)
-
 try:
     import bz2
 except ImportError as bz2_import_error:
@@ -54,6 +50,8 @@ except ImportError as lzma_import_error:
     lzma = lzma_import_error
 
 
+PYTHON_VERSION = sys.version_info[:3]
+
 _logger = 'tools'
 
 
@@ -94,9 +92,8 @@ def has_module(module, version=None):
         else:
             if module_version >= LooseVersion(version):
                 return True
-            else:
-                warn('Module version {} is lower than requested version {}'
-                     .format(module_version, version), ImportWarning)
+            warn('Module version {} is lower than requested version {}'
+                 .format(module_version, version), ImportWarning)
     return False
 
 
@@ -171,7 +168,7 @@ class suppress_warnings(catch_warnings):  # noqa: N801
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         """Stop logging warnings and show those that do not match to params."""
-        super().__exit__()
+        super().__exit__(exc_type, exc_val, exc_tb)
         for warning in self.log:
             if (
                 not issubclass(warning.category, self.category)
@@ -244,7 +241,7 @@ class DotReadableDict:
         return repr(self.__dict__)
 
 
-class FrozenDict(dict):
+class _FrozenDict(dict):
 
     """
     Frozen dict, preventing write after initialisation.
@@ -252,14 +249,13 @@ class FrozenDict(dict):
     Raises TypeError if write attempted.
     """
 
-    def __init__(self, data=None, error=None):
+    def __init__(self, data=None, error: Optional[str] = None):
         """
         Initializer.
 
         @param data: mapping to freeze
         @type data: mapping
         @param error: error message
-        @type error: basestring
         """
         if data:
             args = [data]
@@ -275,6 +271,38 @@ class FrozenDict(dict):
     __setitem__ = update
 
 
+class frozenmap(Mapping):  # noqa:  N801
+
+    """Frozen mapping, preventing write after initialisation."""
+
+    def __init__(self, data=(), **kwargs):
+        """Initialize data in same ways like a dict."""
+        self.__data = {}
+        if isinstance(data, Mapping):
+            for key in data:
+                self.__data[key] = data[key]
+        elif hasattr(data, 'keys'):
+            for key in data.keys():
+                self.__data[key] = data[key]
+        else:
+            for key, value in data:
+                self.__data[key] = value
+        for key, value in kwargs.items():
+            self.__data[key] = value
+
+    def __getitem__(self, key):
+        return self.__data[key]
+
+    def __iter__(self):
+        return iter(self.__data)
+
+    def __len__(self):
+        return len(self.__data)
+
+    def __repr__(self):
+        return '{}({!r})'.format(self.__class__.__name__, self.__data)
+
+
 class LazyRegex:
 
     """
@@ -411,8 +439,8 @@ class MediaWikiVersion(Version):
     version is greater if the normal version or dev version is greater. For
     example:
 
-        1.24 < 1.24.1 < 1.25wmf1 < 1.25alpha < 1.25beta1 < 1.25beta2
-        < 1.25-rc-1 < 1.25-rc.2 < 1.25
+        1.34 < 1.34.1 < 1.35wmf1 < 1.35alpha < 1.35beta1 < 1.35beta2
+        < 1.35-rc-1 < 1.35-rc.2 < 1.35
 
     Any other suffixes are considered invalid.
     """
@@ -521,7 +549,7 @@ class ThreadedGenerator(threading.Thread):
         if not hasattr(self, 'generator'):
             raise RuntimeError('No generator for ThreadedGenerator to run.')
         self.args, self.kwargs = args, kwargs
-        threading.Thread.__init__(self, group=group, name=name)
+        super().__init__(group=group, name=name)
         self.queue = queue.Queue(qsize)
         self.finished = threading.Event()
 
@@ -692,7 +720,7 @@ class ThreadList(list):
                   .format(thd, thd.queue.qsize()), self._logger)
 
 
-def intersect_generators(genlist):
+def intersect_generators(genlist, allow_duplicates=False):
     """
     Intersect generators listed in genlist.
 
@@ -707,6 +735,8 @@ def intersect_generators(genlist):
 
     @param genlist: list of page generators
     @type genlist: list
+    @param allow_duplicates: allow duplicates if present in all generators
+    @type allow_duplicates: bool
     """
     # If any generator is empty, no pages are going to be returned
     for source in genlist:
@@ -717,7 +747,8 @@ def intersect_generators(genlist):
 
     # Item is cached to check that it is found n_gen
     # times before being yielded.
-    cache = collections.defaultdict(set)
+    from collections import Counter
+    cache = collections.defaultdict(Counter)
     n_gen = len(genlist)
 
     # Class to keep track of alive threads.
@@ -729,6 +760,9 @@ def intersect_generators(genlist):
         threaded_gen.daemon = True
         thrlist.append(threaded_gen)
 
+    ones = Counter(thrlist)
+    seen = {}
+
     while True:
         # Get items from queues in a round-robin way.
         for t in thrlist:
@@ -736,14 +770,23 @@ def intersect_generators(genlist):
                 # TODO: evaluate if True and timeout is necessary.
                 item = t.queue.get(True, 0.1)
 
-                # Cache entry is a set of thread.
-                # Duplicates from same thread are not counted twice.
-                cache[item].add(t)
+                if not allow_duplicates and hash(item) in seen:
+                    continue
+
+                # Cache entry is a Counter of ThreadedGenerator objects.
+                cache[item].update([t])
                 if len(cache[item]) == n_gen:
-                    yield item
-                    # Remove item from cache.
-                    # No chance of seeing it again (see later: early stop).
-                    cache.pop(item)
+                    if allow_duplicates:
+                        yield item
+                        # Remove item from cache if possible.
+                        if all(el == 1 for el in cache[item].values()):
+                            cache.pop(item)
+                        else:
+                            cache[item] -= ones
+                    else:
+                        yield item
+                        cache.pop(item)
+                        seen[hash(item)] = True
 
                 active = thrlist.active_count()
                 max_cache = n_gen
@@ -761,10 +804,9 @@ def intersect_generators(genlist):
                 pass
             except KeyboardInterrupt:
                 thrlist.stop_all()
-            finally:
-                # All threads are done.
-                if thrlist.active_count() == 0:
-                    return
+            # All threads are done.
+            if thrlist.active_count() == 0:
+                return
 
 
 def roundrobin_generators(*iterables):
@@ -793,7 +835,7 @@ def filter_unique(iterable, container=None, key=None, add=None):
     already present in the local set.
 
     For large collections, this is not memory efficient, as a strong reference
-    to every item is kept in a local set which can not be cleared.
+    to every item is kept in a local set which cannot be cleared.
 
     Also, the local set can't be re-used when chaining unique operations on
     multiple generators.
@@ -859,7 +901,7 @@ class EmptyDefault(str, Mapping):
     empty collections, so it can be iterated through it safely if treated as a
     list, tuple, set or dictionary. It is also basically an empty string.
 
-    Accessing a value via __getitem__ will result in an combined KeyError and
+    Accessing a value via __getitem__ will result in a combined KeyError and
     IndexError.
     """
 
@@ -892,7 +934,9 @@ class SelfCallMixin:
         """Do nothing and just return itself."""
         if hasattr(self, '_own_desc'):
             issue_deprecation_warning('Calling {}'.format(self._own_desc),
-                                      'it directly', since='20150515')
+                                      'it directly',
+                                      warning_class=FutureWarning,
+                                      since='20150515')
         return self
 
 
@@ -915,11 +959,10 @@ class DequeGenerator(Iterator, collections.deque):
     """A generator that allows items to be added during generating."""
 
     def __next__(self):
-        """Python 3 iterator method."""
+        """Iterator method."""
         if len(self):
             return self.popleft()
-        else:
-            raise StopIteration
+        raise StopIteration
 
 
 def open_archive(filename, mode='rb', use_extension=True):
@@ -947,8 +990,7 @@ def open_archive(filename, mode='rb', use_extension=True):
     @raises ValueError: When 7za is not available or the opening mode is
         unknown or it tries to write a 7z archive.
     @raises FileNotFoundError: When the filename doesn't exist and it tries
-        to read from it or it tries to determine the compression algorithm (or
-        IOError on Python 2).
+        to read from it or it tries to determine the compression algorithm.
     @raises OSError: When it's not a 7z archive but the file extension is 7z.
         It is also raised by bz2 when its content is invalid. gzip does not
         immediately raise that error but only on reading it.
@@ -1174,8 +1216,7 @@ def add_full_name(obj):
                 and callable(outer_args[0])):
             add_decorated_full_name(outer_args[0])
             return obj(outer_args[0])
-        else:
-            return inner_wrapper
+        return inner_wrapper
 
     if not __debug__:
         return obj
@@ -1522,8 +1563,11 @@ def remove_last_args(arg_names):
     return decorator
 
 
-def redirect_func(target, source_module=None, target_module=None,
-                  old_name=None, class_name=None, since=None,
+def redirect_func(target, source_module: Optional[str] = None,
+                  target_module: Optional[str] = None,
+                  old_name: Optional[str] = None,
+                  class_name: Optional[str] = None,
+                  since: Optional[str] = None,
                   future_warning=False):
     """
     Return a function which can be used to redirect to 'target'.
@@ -1536,20 +1580,15 @@ def redirect_func(target, source_module=None, target_module=None,
     @param source_module: The module of the old function. If '.' defaults
         to target_module. If 'None' (default) it tries to guess it from the
         executing function.
-    @type source_module: basestring
     @param target_module: The module of the target function. If
         'None' (default) it tries to get it from the target. Might not work
         with nested classes.
-    @type target_module: basestring
     @param old_name: The old function name. If None it uses the name of the
         new function.
-    @type old_name: basestring
     @param class_name: The name of the class. It's added to the target and
         source module (separated by a '.').
-    @type class_name: basestring
     @param since: a timestamp string of the date when the method was
         deprecated (form 'YYYYMMDD') or a version string.
-    @type since: str
     @param future_warning: if True a FutureWarning will be thrown,
         otherwise it defaults to DeprecationWarning
     @type future_warning: bool
@@ -1607,8 +1646,10 @@ class ModuleDeprecationWrapper(types.ModuleType):
             sys.modules[module.__name__] = self
 
     def _add_deprecated_attr(self, name: str, replacement=None,
-                             replacement_name=None, warning_message=None,
-                             since=None, future_warning=False):
+                             replacement_name: Optional[str] = None,
+                             warning_message: Optional[str] = None,
+                             since: Optional[str] = None,
+                             future_warning: bool = False):
         """
         Add the name to the local deprecated names dict.
 
@@ -1622,16 +1663,12 @@ class ModuleDeprecationWrapper(types.ModuleType):
             if C{replacement} is not None and it has no __name__ attribute.
             If it contains a '.', it will be interpreted as a Python dotted
             object name, and evaluated when the deprecated object is needed.
-        @type replacement_name: str
         @param warning_message: The warning to display, with positional
             variables: {0} = module, {1} = attribute name, {2} = replacement.
-        @type warning_message: basestring
         @param since: a timestamp string of the date when the method was
             deprecated (form 'YYYYMMDD') or a version string.
-        @type since: str
         @param future_warning: if True a FutureWarning will be thrown,
             otherwise it defaults to DeprecationWarning
-        @type future_warning: bool
         """
         if '.' in name:
             raise ValueError('Deprecated name "{}" may not contain '
@@ -1757,28 +1794,6 @@ def compute_file_hash(filename: str, sha='sha1', bytes_to_read=None):
 # deprecated parts ############################################################
 
 
-@deprecated('open_archive()', since='20150915', future_warning=True)
-def open_compressed(filename, use_extension=False):
-    """DEPRECATED: Open a file and uncompress it if needed."""
-    return open_archive(filename, use_extension=use_extension)
-
-
-class IteratorNextMixin(Iterator):
-
-    """DEPRECATED. Backwards compatibility for Iterators."""
-
-    pass
-
-
-class _UnicodeMixin:
-
-    """DEPRECATED. Mixin class to add __str__ method in Python 2 or 3."""
-
-    def __str__(self):
-        """Return the unicode representation as the str representation."""
-        return self.__unicode__()
-
-
 @deprecated('bot_choice.Option and its subclasses', since='20181217')
 def concat_options(message, line_length, options):
     """DEPRECATED. Concatenate options."""
@@ -1803,26 +1818,7 @@ def concat_options(message, line_length, options):
     return '{} ({}):'.format(message, option_msg)
 
 
-@deprecated(since='20200723', future_warning=True)
-def py2_encode_utf_8(func):
-    """Decorator to optionally encode the string result of func on Python 2."""
-    return func
-
-
 wrapper = ModuleDeprecationWrapper(__name__)
-wrapper._add_deprecated_attr('UnicodeMixin', _UnicodeMixin,
-                             replacement_name='',
-                             since='20200723', future_warning=True)
-wrapper._add_deprecated_attr('IteratorNextMixin', replacement_name='',
-                             since='20200723', future_warning=True)
-wrapper._add_deprecated_attr('getargspec', inspect.getargspec,
-                             since='20200712', future_warning=True)
-wrapper._add_deprecated_attr('ArgSpec', inspect.ArgSpec,
-                             since='20200712', future_warning=True)
-wrapper._add_deprecated_attr('UnicodeType', str,
-                             since='20200813', future_warning=True)
-wrapper._add_deprecated_attr('StringTypes', (str, bytes),
-                             replacement_name='(str, bytes)',
-                             since='20200813', future_warning=True)
-wrapper._add_deprecated_attr('signature', inspect.signature,
-                             since='20200813', future_warning=True)
+wrapper._add_deprecated_attr('FrozenDict', _FrozenDict,
+                             replacement_name='tools.frozenmap',
+                             since='20201109', future_warning=True)
