@@ -1,5 +1,4 @@
 #!/usr/bin/python
-# -*- coding: utf-8 -*-
 """Wrapper around djvulibre to access djvu files properties and content."""
 #
 # (C) Pywikibot team, 2015-2020
@@ -14,7 +13,7 @@ from collections import Counter
 
 import pywikibot
 
-from pywikibot.tools import deprecated, deprecated_args
+from pywikibot.tools import deprecated_args
 
 
 def _call_cmd(args, lib='djvulibre') -> tuple:
@@ -92,12 +91,6 @@ class DjVuFile:
         """Return a string representation."""
         return str("{}('{}')").format(self.__class__.__name__, self._filename)
 
-    @property
-    @deprecated('DjVuFile.file', since='2010222', future_warning=True)
-    def file_djvu(self):
-        """Deprecated file_djvu instance variable."""
-        return self.file
-
     def check_cache(fn):
         """Decorator to check if cache shall be cleared."""
         cache = ['_page_count', '_has_text', '_page_info']
@@ -267,7 +260,7 @@ class DjVuFile:
 
         # Convert white_page to djvu.
         res, data = _call_cmd(['c44', white_ppm, '-dpi', dpi])
-        os.unlink(white_ppm)  # rm white_page.ppm before retuning.
+        os.unlink(white_ppm)  # rm white_page.ppm before returning.
         if not res:
             return False
 
