@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """Module containing various formatting related utilities."""
 #
 # (C) Pywikibot team, 2015-2020
@@ -8,7 +7,7 @@
 import math
 
 from string import Formatter
-from typing import Sequence
+from typing import Any, Mapping, Sequence
 
 from pywikibot.logging import output
 from pywikibot.userinterfaces.terminal_interface_base import colors
@@ -72,8 +71,7 @@ class _ColorFormatter(Formatter):
         """Get value, filling in 'color' when it is a valid color."""
         if key == 'color' and kwargs.get('color') in self.colors:
             return '\03{{{0}}}'.format(kwargs[key])
-        else:
-            return super().get_value(key, args, kwargs)
+        return super().get_value(key, args, kwargs)
 
     def parse(self, format_string: str):
         """Yield results similar to parse but skip colors."""
@@ -104,7 +102,8 @@ class _ColorFormatter(Formatter):
         if previous_literal:
             yield previous_literal, None, None, None
 
-    def vformat(self, format_string: str, args: Sequence, kwargs: dict) -> str:
+    def vformat(self, format_string: str, args: Sequence,
+                kwargs: Mapping[str, Any]) -> str:
         """Return the format result but verify no colors are keywords.
 
         @param format_string: The format template string
