@@ -1,16 +1,13 @@
-# -*- coding: utf-8 -*-
 """
 Platform independent terminal interface module.
 
 It imports the appropriate operating system specific implementation.
 """
 #
-# (C) Pywikibot team, 2003-2018
+# (C) Pywikibot team, 2003-2020
 #
 # Distributed under the terms of the MIT license.
 #
-from __future__ import absolute_import, division, unicode_literals
-
 import sys
 
 if sys.platform == 'win32':
