@@ -1,23 +1,24 @@
-# -*- coding: utf-8 -*-
 """Base for terminal user interfaces."""
 #
-# (C) Pywikibot team, 2003-2020
+# (C) Pywikibot team, 2003-2021
 #
 # Distributed under the terms of the MIT license.
 #
 import getpass
 import logging
-import math
 import re
 import sys
 import threading
 
+from typing import Any, Optional, Union
+
 import pywikibot
 from pywikibot import config2 as config
+
+from pywikibot.backports import Sequence
 from pywikibot.bot import VERBOSE, INFO, STDOUT, INPUT, WARNING
 from pywikibot.bot_choice import (ChoiceException, Option, OutputOption,
                                   QuitKeyboardInterrupt, StandardOption)
-from pywikibot.tools import deprecated
 from pywikibot.userinterfaces import transliteration
 
 
@@ -239,7 +240,8 @@ class UI:
         # May be overridden by subclass
         return input()
 
-    def input(self, question, password=False, default='', force=False):
+    def input(self, question: str, password: bool = False,
+              default: str = '', force: bool = False) -> str:
         """
         Ask the user a question and return the answer.
 
@@ -250,15 +252,10 @@ class UI:
         a trailing question mark.
 
         @param question: The question, without trailing whitespace.
-        @type question: basestring
         @param password: if True, hides the user's input (for password entry).
-        @type password: bool
         @param default: The default answer if none was entered. None to require
             an answer.
-        @type default: basestring
         @param force: Automatically use the default
-        @type force: bool
-        @rtype: str
         """
         assert(not password or not default)
         end_marker = ':'
@@ -281,6 +278,10 @@ class UI:
         while True:
             self.output(question + ' ')
             text = self._input_reraise_cntl_c(password)
+
+            if text is None:
+                continue
+
             if text:
                 return text
 
@@ -299,10 +300,13 @@ class UI:
                 text = self._raw_input()
         except KeyboardInterrupt:
             raise QuitKeyboardInterrupt()
+        except UnicodeDecodeError:
+            return None  # wrong terminal encoding, T258143
         return text
 
-    def input_choice(self, question, options, default=None,
-                     return_shortcut=True, automatic_quit=True, force=False):
+    def input_choice(self, question: str, options, default: str = None,
+                     return_shortcut: bool = True,
+                     automatic_quit: bool = True, force: bool = False):
         """
         Ask the user and returns a value from the options.
 
@@ -311,7 +315,6 @@ class UI:
         ambiguous index.
 
         @param question: The question, without trailing whitespace.
-        @type question: basestring
         @param options: Iterable of all available options. Each entry contains
             the full length answer and a shortcut of only one character.
             Alternatively they may be Option (or subclass) instances or
@@ -322,19 +325,15 @@ class UI:
             Singletons of Option and its subclasses are also accepted.
         @param default: The default answer if no was entered. None to require
             an answer.
-        @type default: basestring
         @param return_shortcut: Whether the shortcut or the index in the option
             should be returned.
-        @type return_shortcut: bool
         @param automatic_quit: Adds the option 'Quit' ('q') if True and throws
             a L{QuitKeyboardInterrupt} if selected.
-        @type automatic_quit: bool
         @param force: Automatically use the default
-        @type force: bool
         @return: If return_shortcut the shortcut of options or the value of
             default (if it's not None). Otherwise the index of the answer in
             options. If default is not a shortcut, it'll return -1.
-        @rtype: int (if not return_shortcut), lowercased basestring (otherwise)
+        @rtype: int (if not return_shortcut), lowercased str (otherwise)
         """
         if force and default is None:
             raise ValueError('With no default option it cannot be forced')
@@ -381,59 +380,52 @@ class UI:
             return index
         return answer
 
-    @deprecated('input_choice', since='20140825', future_warning=True)
-    def inputChoice(self, question, options, hotkeys, default=None):
+    def input_list_choice(self, question: str, answers: Sequence[Any],
+                          default: Union[int, str, None] = None,
+                          force: bool = False) -> Any:
+        """Ask the user to select one entry from a list of entries.
+
+        @param question: The question, without trailing whitespace.
+        @param answers: A sequence of options to be choosen.
+        @param default: The default answer if no was entered. None to require
+            an answer.
+        @param force: Automatically use the default.
+        @return: Return a single Sequence entry.
         """
-        Ask the user a question with a predefined list of acceptable answers.
-
-        DEPRECATED: Use L{input_choice} instead!
-
-        Directly calls L{input_choice} with the options and hotkeys zipped
-        into a tuple list. It always returns the hotkeys and throws no
-        L{QuitKeyboardInterrupt} if quit was selected.
-        """
-        return self.input_choice(question=question, options=zip(options,
-                                                                hotkeys),
-                                 default=default, return_shortcut=True,
-                                 automatic_quit=False)
-
-    def input_list_choice(self, question, answers, default=None, force=False):
-        """Ask the user to select one entry from a list of entries."""
-        message = question
-        clist = answers
-
-        line_template = '{{0: >{0}}}: {{1}}'.format(
-            int(math.log10(len(clist)) + 1))
-        for n, i in enumerate(clist):
-            pywikibot.output(line_template.format(n + 1, i))
+        if not force:
+            line_template = '{{0: >{0}}}: {{1}}'.format(len(str(len(answers))))
+            for i, entry in enumerate(answers, start=1):
+                pywikibot.output(line_template.format(i, entry))
 
         while True:
-            choice = self.input(message, default=default, force=force)
+            choice = self.input(question, default=default, force=force)
+
             try:
                 choice = int(choice) - 1
-            except ValueError:
-                try:
-                    choice = clist.index(choice)
-                except IndexError:
-                    choice = -1
+            except (TypeError, ValueError):
+                if choice in answers:
+                    return choice
+                choice = -1
 
             # User typed choice number
-            if 0 <= choice < len(clist):
-                return clist[choice]
-            else:
-                pywikibot.error('Invalid response')
+            if 0 <= choice < len(answers):
+                return answers[choice]
 
-    def editText(self, text, jumpIndex=None, highlight=None):
+            if force:
+                raise ValueError('Invalid value "{}" for default during force.'
+                                 .format(default))
+
+            pywikibot.error('Invalid response')
+
+    def editText(self, text: str, jumpIndex: Optional[int] = None,
+                 highlight: Optional[str] = None):
         """Return the text as edited by the user.
 
         Uses a Tkinter edit box because we don't have a console editor
 
         @param text: the text to be edited
-        @type text: str
         @param jumpIndex: position at which to put the caret
-        @type jumpIndex: int
         @param highlight: each occurrence of this substring will be highlighted
-        @type highlight: str
         @return: the modified text, or None if the user didn't save the text
             file in his text editor
         @rtype: str or None
@@ -472,7 +464,7 @@ class TerminalHandler(logging.Handler):
         If strm is not specified, sys.stderr is used.
 
         """
-        logging.Handler.__init__(self)
+        super().__init__()
         # replace Handler's instance-specific lock with the shared class lock
         # to ensure that only one instance of this handler can write to
         # the console at a time
@@ -493,7 +485,7 @@ class TerminalHandler(logging.Handler):
             # Each warning appears twice
             # the second time it has a 'message'
             if 'message' in record.__dict__:
-                return
+                return None
 
             record.__dict__.setdefault('newline', '\n')
 
@@ -525,5 +517,4 @@ class MaxLevelFilter(logging.Filter):
         """Return true if the level is below or equal to the set level."""
         if self.level:
             return record.levelno <= self.level
-        else:
-            return True
+        return True
