@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """Module to transliterate text."""
 #
 # (C) Pywikibot team, 2006-2020
@@ -1087,7 +1086,7 @@ _trans = {
 }
 
 
-class transliterator:
+class transliterator:  # noqa: N801
 
     """Class to transliterating text."""
 
@@ -1110,17 +1109,15 @@ class transliterator:
             trans[char] = value
         self.trans = trans
 
-    def transliterate(self, char: str, default='?', prev='-', next='-') -> str:
+    def transliterate(self, char: str, default: str = '?',
+                      prev: str = '-', next: str = '-') -> str:
         """
         Transliterate the character.
 
         @param char: The character to transliterate.
         @param default: The character used when there is no transliteration.
-        @type default: str
         @param prev: The previous character
-        @type prev: str
         @param next: The next character
-        @type next: str
         @return: The transliterated character which may be an empty string
         """
         if char in self.trans:
@@ -1137,6 +1134,5 @@ class transliterator:
         if char == 'ຫ':
             if next in 'ງຍນຣລຼຼວ':
                 return ''
-            else:
-                return 'h'
+            return 'h'
         return default
