@@ -1,11 +1,9 @@
-# -*- coding: utf-8 -*-
 """Module to determine the pywikibot version (tag, revision and date)."""
 #
-# (C) Pywikibot team, 2007-2020
+# (C) Pywikibot team, 2007-2021
 #
 # Distributed under the terms of the MIT license.
 #
-import codecs
 import datetime
 import json
 import os
@@ -17,18 +15,20 @@ import time
 import xml.dom.minidom
 
 from contextlib import closing, suppress
-from distutils import log
 from distutils.sysconfig import get_python_lib
 from importlib import import_module
 from io import BytesIO
+from typing import Optional
 from warnings import warn
 
 import pywikibot
 
+from pywikibot.backports import cache
+from pywikibot.comms.http import fetch
 from pywikibot import config2 as config
 from pywikibot.tools import deprecated
 
-cache = None
+
 _logger = 'version'
 
 
@@ -55,26 +55,35 @@ def get_toolforge_hostname():
     return None
 
 
-def getversion(online=True):
+def getversion(online: bool = True) -> str:
     """Return a pywikibot version string.
 
-    @param online: (optional) Include information obtained online
+    @param online: Include information obtained online
     """
-    data = dict(getversiondict())  # copy dict to prevent changes in 'cache'
+    branches = {
+        'master': 'branches/master',
+        'stable': 'branches/stable',
+    }
+    data = getversiondict()
     data['cmp_ver'] = 'n/a'
+    local_hsh = data.get('hsh', '')
+    hsh = {}
 
     if online:
-        with suppress(Exception):
-            hsh3 = getversion_onlinerepo('tags/stable')
-            hsh2 = getversion_onlinerepo()
-            hsh1 = data['hsh']
-            data['cmp_ver'] = 'UNKNOWN' if not hsh1 else (
-                'OUTDATED' if hsh1 not in (hsh2, hsh3) else 'ok')
+        if not local_hsh:
+            data['cmp_ver'] = 'UNKNOWN'
+        else:
+            for branch, path in branches.items():
+                with suppress(Exception):
+                    hsh[getversion_onlinerepo(path)] = branch
+            if hsh:
+                data['cmp_ver'] = hsh.get(local_hsh, 'OUTDATED')
 
-    data['hsh'] = data['hsh'][:7]  # make short hash from full hash
+    data['hsh'] = local_hsh[:7]  # make short hash from full hash
     return '{tag} ({hsh}, {rev}, {date}, {cmp_ver})'.format_map(data)
 
 
+@cache
 def getversiondict():
     """Get version info for the package.
 
@@ -85,15 +94,10 @@ def getversiondict():
         - hash (git hash for the current revision)
     @rtype: C{dict} of four C{str}
     """
-    global cache
-    if cache:
-        return cache
-
     _program_dir = _get_program_dir()
     exceptions = {}
 
     for vcs_func in (getversion_git,
-                     getversion_svn_setuptools,
                      getversion_svn,
                      getversion_nightly,
                      getversion_package):
@@ -109,14 +113,14 @@ def getversiondict():
         # pywikibot was imported without using version control at all.
         tag, rev, date, hsh = (
             '', '-1 (unknown)', '0 (unknown)', '(unknown)')
+        warn('Unable to detect version; exceptions raised:\n{!r}'
+             .format(exceptions), UserWarning)
+        exceptions = None
 
     # git and svn can silently fail, as it may be a nightly.
-    if getversion_package in exceptions:
-        warn('Unable to detect version; exceptions raised:\n%r'
-             % exceptions, UserWarning)
-    elif exceptions:
-        pywikibot.debug('version algorithm exceptions:\n%r'
-                        % exceptions, _logger)
+    if exceptions:
+        pywikibot.debug('version algorithm exceptions:\n{!r}'
+                        .format(exceptions), _logger)
 
     if isinstance(date, str):
         datestring = date
@@ -126,12 +130,11 @@ def getversiondict():
         warn('Unable to detect package date', UserWarning)
         datestring = '-2 (unknown)'
 
-    cache = {'tag': tag, 'rev': rev, 'date': datestring, 'hsh': hsh}
-    return cache
+    return {'tag': tag, 'rev': rev, 'date': datestring, 'hsh': hsh}
 
 
-def svn_rev_info(path):
-    """Fetch information about the current revision of an Subversion checkout.
+def svn_rev_info(path):  # pragma: no cover
+    """Fetch information about the current revision of a Subversion checkout.
 
     @param path: directory of the Subversion checkout
     @return:
@@ -181,65 +184,27 @@ order by revision desc, changed_date desc""")
     return tag, rev, date
 
 
-def github_svn_rev2hash(tag, rev):
+def github_svn_rev2hash(tag: str, rev):  # pragma: no cover
     """Convert a Subversion revision to a Git hash using Github.
 
     @param tag: name of the Subversion repo on Github
     @param rev: Subversion revision identifier
     @return: the git hash
-    @rtype: str
     """
-    from pywikibot.comms import http
-
-    uri = 'https://github.com/wikimedia/%s/!svn/vcc/default' % tag
-    request = http.fetch(uri=uri, method='PROPFIND',
-                         body="<?xml version='1.0' encoding='utf-8'?>"
-                              '<propfind xmlns=\"DAV:\"><allprop/></propfind>',
-                         headers={'label': str(rev),
-                                  'user-agent': 'SVN/1.7.5 {pwb}'})
-
-    dom = xml.dom.minidom.parse(BytesIO(request.raw))
+    uri = 'https://github.com/wikimedia/{}/!svn/vcc/default'.format(tag)
+    request = fetch(uri, method='PROPFIND',
+                    data="<?xml version='1.0' encoding='utf-8'?>"
+                         '<propfind xmlns=\"DAV:\"><allprop/></propfind>',
+                    headers={'label': str(rev),
+                             'user-agent': 'SVN/1.7.5 {pwb}'})
+    dom = xml.dom.minidom.parse(BytesIO(request.content))
     hsh = dom.getElementsByTagName('C:git-commit')[0].firstChild.nodeValue
     date = dom.getElementsByTagName('S:date')[0].firstChild.nodeValue
     date = time.strptime(date[:19], '%Y-%m-%dT%H:%M:%S')
     return hsh, date
 
 
-def getversion_svn_setuptools(path=None):
-    """Get version info for a Subversion checkout using setuptools.
-
-    @param path: directory of the Subversion checkout
-    @return:
-        - tag (name for the repository),
-        - rev (current Subversion revision identifier),
-        - date (date of current revision),
-        - hash (git hash for the Subversion revision)
-    @rtype: C{tuple} of three C{str} and a C{time.struct_time}
-    """
-    try:
-        from setuptools import svn_utils
-    except ImportError:
-        from setuptools_svn import svn_utils
-
-    tag = 'pywikibot-core'
-    _program_dir = path or _get_program_dir()
-    svninfo = svn_utils.SvnInfo(_program_dir)
-    # suppress warning
-    old_level = log.set_threshold(log.ERROR)
-    rev = svninfo.get_revision()
-    log.set_threshold(old_level)
-    if not isinstance(rev, int):
-        raise TypeError('SvnInfo.get_revision() returned type %s' % type(rev))
-    if rev < 0:
-        raise ValueError('SvnInfo.get_revision() returned %d' % rev)
-    if rev == 0:
-        raise ParseError('SvnInfo: invalid workarea')
-    hsh, date = github_svn_rev2hash(tag, rev)
-    rev = 's%s' % rev
-    return (tag, rev, date, hsh)
-
-
-def getversion_svn(path=None):
+def getversion_svn(path=None):  # pragma: no cover
     """Get version info for a Subversion checkout.
 
     @param path: directory of the Subversion checkout
@@ -322,7 +287,7 @@ def getversion_git(path=None):
     return (tag, rev, date, hsh)
 
 
-def getversion_nightly(path=None):
+def getversion_nightly(path=None):  # pragma: no cover
     """Get version info for a nightly release.
 
     @param path: directory of the uncompressed nightly.
@@ -372,7 +337,7 @@ def getversion_onlinerepo(path='branches/master'):
     # Gerrit API responses include )]}' at the beginning,
     # make sure to strip it out
     buf = http.fetch(
-        uri='https://gerrit.wikimedia.org/r/projects/pywikibot%2Fcore/' + path,
+        'https://gerrit.wikimedia.org/r/projects/pywikibot%2Fcore/' + path,
         headers={'user-agent': '{pwb}'}).text[4:]
     try:
         hsh = json.loads(buf)['revision']
@@ -381,40 +346,8 @@ def getversion_onlinerepo(path='branches/master'):
         raise ParseError(repr(e) + ' while parsing ' + repr(buf))
 
 
-@deprecated('get_module_version, get_module_filename and get_module_mtime',
-            since='20150221')
-def getfileversion(filename):
-    """Retrieve revision number of file.
-
-    Extracts __version__ variable containing Id tag, without importing it.
-    (thus can be done for any file)
-
-    The version variable containing the Id tag is read and
-    returned. Because it doesn't import it, the version can
-    be retrieved from any file.
-    @param filename: Name of the file to get version
-    @type filename: str
-    """
-    _program_dir = _get_program_dir()
-    __version__ = None
-    mtime = None
-    fn = os.path.join(_program_dir, filename)
-    if os.path.exists(fn):
-        with codecs.open(fn, 'r', 'utf-8') as f:
-            for line in f.readlines():
-                if line.find('__version__') == 0:
-                    with suppress(Exception):
-                        exec(line)
-                    break
-        stat = os.stat(fn)
-        mtime = datetime.datetime.fromtimestamp(stat.st_mtime).isoformat(' ')
-    if mtime and __version__:
-        return '%s %s %s' % (filename, __version__[5:-1][:7], mtime)
-    else:
-        return None
-
-
-def get_module_version(module):
+@deprecated('pywikibot.__version__', since='20201003')
+def get_module_version(module) -> Optional[str]:  # pragma: no cover
     """
     Retrieve __version__ variable from an imported module.
 
@@ -422,14 +355,13 @@ def get_module_version(module):
     @type module: module
     @return: The version hash without the surrounding text. If not present
         return None.
-    @rtype: str or None
     """
     if hasattr(module, '__version__'):
-        return module.__version__[5:-1]
+        return module.__version__
     return None
 
 
-def get_module_filename(module):
+def get_module_filename(module) -> Optional[str]:
     """
     Retrieve filename from an imported pywikibot module.
 
@@ -440,12 +372,12 @@ def get_module_filename(module):
     @param module: The module instance.
     @type module: module
     @return: The filename if it's a pywikibot module otherwise None.
-    @rtype: str or None
     """
-    if hasattr(module, '__file__') and os.path.exists(module.__file__):
+    if hasattr(module, '__file__'):
         filename = module.__file__
-        if filename[-4:-1] == '.py' and os.path.exists(filename[:-1]):
-            filename = filename[:-1]
+        if not filename or not os.path.exists(filename):
+            return None
+
         program_dir = _get_program_dir()
         if filename[:len(program_dir)] == program_dir:
             return filename
@@ -483,7 +415,7 @@ def package_versions(modules=None, builtins=False, standard_lib=None):
     if not modules:
         modules = sys.modules.keys()
 
-    std_lib_dir = get_python_lib(standard_lib=True)
+    std_lib_dir = pathlib.Path(get_python_lib(standard_lib=True))
 
     root_packages = {key.split('.')[0] for key in modules}
 
@@ -514,20 +446,31 @@ def package_versions(modules=None, builtins=False, standard_lib=None):
 
         if '__file__' in package.__dict__:
             # Determine if this file part is of the standard library.
-            if os.path.normcase(package.__file__).startswith(
-                    os.path.normcase(std_lib_dir)):
+            # possible Namespace package
+            if not hasattr(package, '__file__') or package.__file__ is None:
+                _file = None
+                _path = pathlib.Path(package.__path__[0])
+            else:
+                _file = pathlib.Path(package.__file__)
+                _path = _file.parent
+            if _path == std_lib_dir:
                 std_lib_packages.append(name)
                 if standard_lib is False:
                     continue
-                info['type'] = 'standard libary'
+                info['type'] = 'standard library'
 
             # Strip '__init__.py' from the filename.
-            path = package.__file__
-            if '__init__.py' in path:
-                path = path[0:path.index('__init__.py')]
+            if (not hasattr(package, '__file__')
+                    or package.__file__ is None
+                    or _file.name == '__init__.py'):
+                path = _path
+            else:
+                path = _file
 
             info['path'] = path
-            assert path not in paths, 'Path of the package is in defined paths'
+            assert path not in paths, \
+                   'Path {} of the package {} is in defined paths as {}' \
+                   .format(path, name, paths[path])
             paths[path] = name
 
         if '__version__' in package.__dict__:
@@ -537,8 +480,8 @@ def package_versions(modules=None, builtins=False, standard_lib=None):
 
         # If builtins or standard_lib is None,
         # only include package if a version was found.
-        if (builtins is None and name in builtin_packages) or \
-                (standard_lib is None and name in std_lib_packages):
+        if builtins is None and name in builtin_packages \
+           or standard_lib is None and name in std_lib_packages:
             if 'ver' in info:
                 data[name] = info
             else:
@@ -553,7 +496,7 @@ def package_versions(modules=None, builtins=False, standard_lib=None):
     dir_parts = pathlib.Path(_program_dir).parts
     length = len(dir_parts)
     for path, name in paths.items():
-        lib_parts = pathlib.Path(path).parts
+        lib_parts = path.parts
         if dir_parts != lib_parts[:length]:
             continue
         if lib_parts[length] != '.tox':
