@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 XML reading module.
 
@@ -45,7 +44,7 @@ def parseRestrictions(restrictions):
     return editRestriction, moveRestriction
 
 
-class XmlEntry(object):
+class XmlEntry:
 
     """Represent a page."""
 
@@ -82,7 +81,7 @@ class XmlParserThread(threading.Thread):
 
     def __init__(self, filename, handler):
         """Initializer."""
-        threading.Thread.__init__(self)
+        super().__init__()
         self.filename = filename
         self.handler = handler
 
